@@ -1,6 +1,6 @@
-static const char *TagName   = "$Name: DataBase-Release_2_9 $";
+static const char *TagName   = "$Name:  $";
 static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.32 2006/09/28 11:18:20 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.34 2007/04/25 06:01:03 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :        DataBaseClass.cpp
@@ -17,6 +17,12 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 // $Version$
 //
 // $Log: DataBaseClass.cpp,v $
+// Revision 2.34  2007/04/25 06:01:03  pascal_verdier
+// IOR NULL checked in DbGetDeviceInfo() and DbImportDevice().
+//
+// Revision 2.33  2006/10/10 14:29:00  jensmeyer
+// Deleted db_bench.cpp
+//
 // Revision 2.32  2006/09/28 11:18:20  pascal_verdier
 // DbGetClassForDevice and DbGetClassInheritanceForDevice commands added.
 //
