@@ -12,9 +12,15 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 2.27 $
+// $Revision: 2.29 $
 //
 // $Log: DataBaseClass.h,v $
+// Revision 2.29  2006/09/28 11:18:20  pascal_verdier
+// DbGetClassForDevice and DbGetClassInheritanceForDevice commands added.
+//
+// Revision 2.28  2006/06/22 15:25:32  jlpons
+// Added history commands
+//
 // Revision 2.27  2005/10/05 07:35:54  pascal_verdier
 // Bug fixed in Timing_Info attribute read.
 // ResetTimingValues command added.
@@ -695,6 +701,30 @@ public:
 
 
 
+class DbGetPropertyHistClass : public Tango::Command
+{
+public:
+	DbGetPropertyHistClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetPropertyHistClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetPropertyHistClass() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetPropertyHist_allowed(any);}
+};
+
+
+
 class DbGetProperty : public Tango::Command
 {
 public:
@@ -911,6 +941,30 @@ public:
 
 
 
+class DbGetDevicePropertyHistClass : public Tango::Command
+{
+public:
+	DbGetDevicePropertyHistClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetDevicePropertyHistClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetDevicePropertyHistClass() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetDevicePropertyHist_allowed(any);}
+};
+
+
+
 class DbGetDeviceProperty : public Tango::Command
 {
 public:
@@ -1103,6 +1157,30 @@ public:
 
 
 
+class DbGetDeviceAttributePropertyHistCmd : public Tango::Command
+{
+public:
+	DbGetDeviceAttributePropertyHistCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetDeviceAttributePropertyHistCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetDeviceAttributePropertyHistCmd() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetDeviceAttributePropertyHist_allowed(any);}
+};
+
+
+
 class DbGetDeviceAttributeProperty2Cmd : public Tango::Command
 {
 public:
@@ -1247,6 +1325,30 @@ public:
 
 
 
+class DbGetClassPropertyHistClass : public Tango::Command
+{
+public:
+	DbGetClassPropertyHistClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetClassPropertyHistClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetClassPropertyHistClass() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetClassPropertyHist_allowed(any);}
+};
+
+
+
 class DbGetClassProperty : public Tango::Command
 {
 public:
@@ -1295,6 +1397,78 @@ public:
 
 
 
+class DbGetClassInheritanceForDeviceClass : public Tango::Command
+{
+public:
+	DbGetClassInheritanceForDeviceClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetClassInheritanceForDeviceClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetClassInheritanceForDeviceClass() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetClassInheritanceForDevice_allowed(any);}
+};
+
+
+
+class DbGetClassForDeviceClass : public Tango::Command
+{
+public:
+	DbGetClassForDeviceClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetClassForDeviceClass(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetClassForDeviceClass() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetClassForDevice_allowed(any);}
+};
+
+
+
+class DbGetClassAttributePropertyHistCmd : public Tango::Command
+{
+public:
+	DbGetClassAttributePropertyHistCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out,
+				   const char        *in_desc,
+				   const char        *out_desc,
+				   Tango::DispLevel  level)
+	:Command(name,in,out,in_desc,out_desc, level)	{};
+
+	DbGetClassAttributePropertyHistCmd(const char   *name,
+	               Tango::CmdArgType in,
+				   Tango::CmdArgType out)
+	:Command(name,in,out)	{};
+	~DbGetClassAttributePropertyHistCmd() {};
+	
+	virtual CORBA::Any *execute (Tango::DeviceImpl *dev, const CORBA::Any &any);
+	virtual bool is_allowed (Tango::DeviceImpl *dev, const CORBA::Any &any)
+	{return (static_cast<DataBase *>(dev))->is_DbGetClassAttributePropertyHist_allowed(any);}
+};
+
+
+
 class DbGetClassAttributeProperty2Cmd : public Tango::Command
 {
 public:
@@ -1830,6 +2004,7 @@ public:
 class DataBaseClass : public Tango::DeviceClass
 {
 public:
+//	properties member data
 
 //	add your own data members here
 //------------------------------------
@@ -1851,6 +2026,7 @@ protected:
 	DataBaseClass(string &);
 	static DataBaseClass *_instance;
 	void command_factory();
+	void get_class_property();
 	void attribute_factory(vector<Tango::Attr *> &);
 	void write_class_property();
 	void set_default_property();
