@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.2 2006/08/29 08:49:20 jlpons Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseUtils.cpp,v 2.5 2006/11/06 15:31:03 jlpons Exp $";
 //+=============================================================================
 //
 // file :         DataBaseUtils.cpp
@@ -405,8 +405,13 @@ unsigned int DataBase::get_id(const char *name) {
 
     // Increment id
 	sql_query.str("");
-    sql_query << "UPDATE history_ids SET id = id + 1 WHERE name='" << name << "'";
-	simple_query(sql_query.str(),"get_id()");
+	if(ret==4294967295) {
+      sql_query << "UPDATE history_ids SET id = 0 WHERE name='" << name << "'";
+	  simple_query(sql_query.str(),"get_id()");	
+	} else {
+      sql_query << "UPDATE history_ids SET id = id + 1 WHERE name='" << name << "'";
+	  simple_query(sql_query.str(),"get_id()");
+	}
 
 	return ret;			
 }
@@ -536,7 +541,7 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
   MYSQL_ROW row2;
   int j;
   
-  cout << "purge_att_property(" << object << "," << attribute << "," << name << ")" << endl;
+  //cout << "purge_att_property(" << object << "," << attribute << "," << name << ")" << endl;
 
   TimeVal before, after;
   GetTime(before);
@@ -551,7 +556,7 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
 
   GetTime(after);
   double time_elapsed = Elapsed(before, after);
-  cout << "  select DISTINCT : " << time_elapsed << endl;
+  //cout << "  select DISTINCT : " << time_elapsed << endl;
 
   GetTime(before);
   
@@ -569,7 +574,7 @@ void DataBase::purge_att_property(const char *table,const char *field,const char
 
   GetTime(after);
   time_elapsed = Elapsed(before, after);
-  cout << "  DELETE : " << time_elapsed << endl;
+  //cout << "  DELETE : " << time_elapsed << endl;
       
 }
 
