@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/Starter.cpp,v 3.26 2006/06/13 19:38:15 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/Starter.cpp,v 3.32 2007/05/25 06:08:00 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         Starter.cpp
@@ -13,9 +13,28 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.26 $
+// $Revision: 3.32 $
 //
 // $Log: Starter.cpp,v $
+// Revision 3.32  2007/05/25 06:08:00  pascal_verdier
+// Iterator management modified (strange in vc8 ?)
+//
+// Revision 3.31  2007/03/30 14:13:24  pascal_verdier
+// Remove 2 warnings on linux.
+//
+// Revision 3.30  2007/02/12 13:05:57  pascal_verdier
+// Spectrum String attribute management changed.
+//
+// Revision 3.29  2007/02/01 09:17:44  pascal_verdier
+// Monitor added on some shared data.
+// Wait 3 seconds added at startup for first ping timeout.
+//
+// Revision 3.28  2006/11/20 06:58:37  pascal_verdier
+// Mutex on start process data added.
+//
+// Revision 3.27  2006/11/10 14:53:28  pascal_verdier
+// Remove vc8 warnings.
+//
 // Revision 3.26  2006/06/13 19:38:15  pascal_verdier
 // Minor changes.
 //
@@ -303,8 +322,11 @@ void Starter::init_device()
 		//	And connect database as DeviceProxy
 		//--------------------------------------
 		dbase = new Tango::DeviceProxy(dbname);
-		dbase->set_timeout_millis(500);
-		starting_processes = 0;
+//		dbase->set_timeout_millis(500);
+//		starting_processes = 0;
+
+		//	Build a shared data for StartProcessShared
+		start_proc_data = new StartProcessShared();
 
 		//	Get hostname (In case of cluster host could be multiple)
 		//-------------------------------------------------------------
@@ -331,7 +353,7 @@ void Starter::init_device()
 			}
 			string	s = str_list.substr(start, str_list.length()-start);
 			hosts_list.push_back(s);
-			for (int i=0 ; i<hosts_list.size() ; i++)
+			for (unsigned int i=0 ; i<hosts_list.size() ; i++)
 				cout << hosts_list[i] << endl;
 		}
 		//	Create a StarterUtil instance
@@ -391,6 +413,13 @@ void Starter::init_device()
 				sleep(1);
 #			endif
 		}
+		
+//	A a wait for first ping timeout !!!!
+#	ifdef WIN32
+		_sleep(3000);
+#	else
+		sleep(3);
+#	endif
 
 		//	And Start servers for all startup levels.
 		//	The interStartupLevelWait value will be managed
@@ -422,6 +451,7 @@ void Starter::init_device()
 		//	Start a thread to start polling
 		PollingState	*poller = new PollingState(get_name());
 		poller->start();
+
 	}
 }
 
@@ -441,8 +471,6 @@ void Starter::get_device_property()
 
 	//	Read device properties from database.(Automatic code generation)
 	//-------------------------------------------------------------
-	if (Tango::Util::instance()->_UseDb==false)
-		return;
 	Tango::DbData	dev_prop;
 	dev_prop.push_back(Tango::DbDatum("StartDsPath"));
 	dev_prop.push_back(Tango::DbDatum("WaitForDriverStartup"));
@@ -453,7 +481,8 @@ void Starter::get_device_property()
 
 	//	Call database and extract values
 	//--------------------------------------------
-	get_db_device()->get_property(dev_prop);
+	if (Tango::Util::instance()->_UseDb==true)
+		get_db_device()->get_property(dev_prop);
 	Tango::DbDatum	def_prop, cl_prop;
 	StarterClass	*ds_class =
 		(static_cast<StarterClass *>(get_device_class()));
@@ -522,7 +551,7 @@ void Starter::get_device_property()
 	if (startDsPath.size()==0)
 		startDsPath.push_back(".");
 	else
-	for (int i=0 ; i<startDsPath.size() ; i++)
+	for (unsigned int i=0 ; i<startDsPath.size() ; i++)
 		INFO_STREAM << "startDsPath[" << i << "] = " << startDsPath[i] << endl;
 	INFO_STREAM << "WaitForDriverStartup = " << waitForDriverStartup << " seconds" << endl;
 	cout << "UseEvents  = " << ((useEvents==false)? "False": "True") << endl;
@@ -590,33 +619,15 @@ void Starter::read_HostState(Tango::Attribute &attr)
 void Starter::read_RunningServers(Tango::Attribute &attr)
 {
 	DEBUG_STREAM << "Starter::read_RunningServers(Tango::Attribute &attr) entering... "<< endl;
-	
-	//	Check if returned obejct already allocated.
-	//--------------------------------------------------
-	if (attr_RunningServers_read==NULL)
-	{
-		//	Retreive max_x of spectrum attribute
-		Tango::SpectrumAttr	*specAtt = (Tango::SpectrumAttr	*) &attr;
-		//cout << "Running specAtt->get_max_x()  = " << specAtt->get_max_x() << endl;
-		//attr_RunningServers_read = new Tango::DevString[specAtt->get_max_x()];
-		attr_RunningServers_read = new Tango::DevString[NB_SERVERS_MAX];
-	}
 
-	//	Check if servers object initilized
-	//---------------------------------------
-	int		nb = 0, i;
-	if (servers.size()>0)
-	{
-		//	Fill attribute
-		//-----------------------------------------------------------
-		for (i=0 ; i<servers.size() ; i++)
-			if (servers[i].running==true)
-			{
-				INFO_STREAM << "RUNNING: " << servers[i].name << endl;
-				attr_RunningServers_read[nb++] = (char*)servers[i].name.c_str();
-			}
-	}
-	attr.set_value(attr_RunningServers_read, nb);
+	//	Check running ones
+	vector<string>	runnings;
+	for (unsigned int i=0 ; i<servers.size() ; i++)
+		if (servers[i].running==true)
+			runnings.push_back(servers[i].name);
+	//	And fill attribute
+	stringArrayRunning << runnings;	
+	attr.set_value(stringArrayRunning.get_buffer(), stringArrayRunning.length());
 }
 
 //+----------------------------------------------------------------------------
@@ -630,32 +641,14 @@ void Starter::read_StoppedServers(Tango::Attribute &attr)
 {
 	DEBUG_STREAM << "Starter::read_StoppedServers(Tango::Attribute &attr) entering... "<< endl;
 
-	//	Check if returned obejct already allocated.
-	//--------------------------------------------------
-	if (attr_StoppedServers_read==NULL)
-	{
-		//	Retreive max_x of spectrum attribute
-		Tango::SpectrumAttr	*specAtt = (Tango::SpectrumAttr	*) &attr;
-		//cout << "Stopped specAtt->get_max_x()  = " << specAtt->get_max_x() << endl;
-		//attr_StoppedServers_read = new Tango::DevString[specAtt->get_max_x()];
-		attr_StoppedServers_read = new Tango::DevString[NB_SERVERS_MAX];
-	}
-
-	//	Check if servers object initilized
-	//---------------------------------------
-	int		nb = 0, i;
-	if (servers.size()>0)
-	{
-		//	And fill attribute
-		//-----------------------------------------------------------
-		for (i=0 ; i<servers.size() ; i++)
-			if (servers[i].running==false)
-			{
-				INFO_STREAM << "STOPPED: " << servers[i].name << endl;
-				attr_StoppedServers_read[nb++] = (char*)servers[i].name.c_str();
-			}
-	}
-	attr.set_value(attr_StoppedServers_read, nb);
+	//	Check running ones
+	vector<string>	stopped;
+	for (unsigned int i=0 ; i<servers.size() ; i++)
+		if (servers[i].running==false)
+			stopped.push_back(servers[i].name);
+	//	And fill attribute
+	stringArrayStopped << stopped;	
+	attr.set_value(stringArrayStopped.get_buffer(), stringArrayStopped.length());
 }
 
 //+------------------------------------------------------------------
@@ -750,8 +743,7 @@ Tango::DevState Starter::dev_state()
 	ControledServer		*p_serv;
 	int		nb_running   = 0;
 	int		nb_controled = 0;
-	int		i;
-	for (i=0 ; i<servers.size() ; i++)
+	for (unsigned int i=0 ; i<servers.size() ; i++)
 	{
 		p_serv = &servers[i];
 		//	Count how many are controlled
@@ -765,7 +757,7 @@ Tango::DevState Starter::dev_state()
 	}
 
 	//	compare nb running with nb_controlled controled to set state
-	if (starting_processes>0)
+	if (start_proc_data->get_starting_processes()>0)
 		set_state(Tango::MOVING);
 	else
 	if (nb_running==nb_controled && notifyd_state==Tango::ON)
@@ -850,7 +842,11 @@ NewProcess *Starter::processCouldStart(char *argin)
 			mkdir(TmpRoot);
 			int r = mkdir(logpath.c_str());
 #else
+#	ifdef linux
+			int r = mkdir(logpath.c_str(), (mode_t)(0775) );
+#	else
 			int r = mkdir(logpath.c_str(), (mode_t)(O_RDWR | O_CREAT, 0775) );
+#	endif
 #endif
 			if (r<0)
 			{
@@ -907,8 +903,8 @@ void Starter::startProcesses(vector<NewProcess *> v_np, int level)
 {
 	//	Start process to start processes
 	//-------------------------------------
-	start_process_thread_levels.push_back(level);
-	starting_processes++;
+	start_proc_data->push_back_level(level);
+//	starting_processes++;
 	StartProcessThread	*pt =
 		new StartProcessThread(v_np, level, this);
 	pt->start();
@@ -1002,7 +998,9 @@ Tango::DevVarStringArray *Starter::dev_get_running_servers(Tango::DevBoolean all
 
 	//	prepeare the argout for running servers list
 	//-----------------------------------------------------------
-	int		nb = 0, i, x;
+	int		nb = 0;
+	int		x;
+	unsigned int	i;
 	for (i=0 ; i<servers.size() ; i++)
 		if (all_serv || servers[i].controled)
 			if (servers[i].running)
@@ -1049,7 +1047,9 @@ Tango::DevVarStringArray *Starter::dev_get_stop_servers(Tango::DevBoolean all_se
 
 	//	prepeare the argout for NOT running servers list
 	//-----------------------------------------------------------
-	int		nb = 0, i, x;
+	int		nb = 0;
+	int		x;
+	unsigned int	i;
 	for (i=0 ; i<servers.size() ; i++)
 		if (all_serv || servers[i].controled)
 			if (servers[i].running==false)
@@ -1076,7 +1076,7 @@ Tango::DevVarStringArray *Starter::dev_get_stop_servers(Tango::DevBoolean all_se
 int	Starter::nb_servers_to_start(int level)
 {
 	int	cnt = 0;
-	for (int i=0 ; i<servers.size() ; i++)
+	for (unsigned int i=0 ; i<servers.size() ; i++)
 	{
 		ControledServer	*server = &servers[i];
 		//	server->running could not be initialized
@@ -1100,7 +1100,7 @@ int	Starter::nb_servers_to_start(int level)
 //+------------------------------------------------------------------
 void Starter::dev_start_all(Tango::DevShort level)
 {
-	INFO_STREAM << "Starter::dev_start_all(): entering for level "<< level <<"... !" << endl;
+	cout << "Starter::dev_start_all(): entering for level "<< level <<"... !" << endl;
 
 	Tango::DevBoolean	throw_it = false;
 	//	Check if servers object initilized
@@ -1125,19 +1125,27 @@ void Starter::dev_start_all(Tango::DevShort level)
 	//	And start the stopped ones
 	//---------------------------------------------------
 	vector<NewProcess *>	processes;
-	for (int i=0 ; i<servers.size() ; i++)
+	for (unsigned int i=0 ; i<servers.size() ; i++)
 	{
 		ControledServer	*server = &servers[i];
 		//	server->running could not be initialized
 		if (server->controled  &&  server->startup_level==level)
 		{
+			cout << "Check startup for " << server->name << endl;
 			server->running = is_running(server);
 			if (server->running==false)
 			{
 				NewProcess	*np = processCouldStart((char*)server->name.c_str());
 				if (np!=NULL)
+				{
 					processes.push_back(np);
+					cout << "Try to start " << np->servname << endl;
+				}
+				else
+					cout << "np is null (?)" << endl;
 			}
+			else
+				cout << "	Alread running...."<< endl;
 		}
 	}
 	if (processes.size()>0)
@@ -1175,7 +1183,7 @@ void Starter::dev_stop_all(Tango::DevShort level)
 	}
 	//	And stop the running ones
 	//---------------------------------------------------
-	for (int i=0 ; i<servers.size() ; i++)
+	for (unsigned int i=0 ; i<servers.size() ; i++)
 	{
 		ControledServer	*server = &servers[i];
 		if (server->controled             &&
