@@ -8,9 +8,18 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.17 $
+// $Revision: 3.20 $
 //
 // $Log: Starter.h,v $
+// Revision 3.20  2007/03/23 09:23:44  pascal_verdier
+// Bug in windows batch file startup fixed.
+//
+// Revision 3.19  2007/02/12 13:06:51  pascal_verdier
+// Spectrum String attribute management changed.
+//
+// Revision 3.18  2006/11/20 06:58:37  pascal_verdier
+// Mutex on start process data added.
+//
 // Revision 3.17  2006/06/13 19:39:35  pascal_verdier
 // Minor changes.
 //
@@ -127,7 +136,7 @@
 //using namespace std;
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.17 $
+ * @version	$Revision: 3.20 $
  */
 
  //	Add your own constants definitions here.
@@ -152,6 +161,7 @@ NewProcess;
 
 namespace Starter_ns
 {
+class StartProcessShared;
 
 /**
  * Class Description:
@@ -168,6 +178,7 @@ namespace Starter_ns
  */
 
 
+
 class Starter: public Tango::Device_3Impl
 {
 public :
@@ -177,6 +188,10 @@ public :
 	 *	Manage the PollingState object startup
 	 */
 	void manage_PollingState_startup();
+	/**
+	 *	Start process thread Shared data
+	 */
+	StartProcessShared	*start_proc_data;
 	/**
 	 *	Boolean used for init.
 	 */
@@ -210,15 +225,10 @@ public :
 	 */
 	Tango::DevShort	fireFromDbase;
 	bool	do_update_from_db;
-	/**
-	 *	Manage levels thread
-	 *	To be shure that (level) threads are not concurent
-	 */
-	vector<int>	start_process_thread_levels;
-	/**
-	 *	Starter is MOVING (starting servers) when > 0;
-	 */
-	int	starting_processes;
+
+	Tango::DevVarStringArray	stringArrayRunning;
+	Tango::DevVarStringArray	stringArrayStopped;
+
 
 	//	Here is the Start of the automatic code generation part
 	//-------------------------------------------------------------	
@@ -499,9 +509,53 @@ void startProcesses(vector<NewProcess *>, int level);
 
 
 
+
+//=========================================================
+/**
+ *	Shared data between DS and StartProcess thread.
+ */
+//=========================================================
+class StartProcessShared: public Tango::TangoMonitor
+{
+private:
+	/**
+	 *	Manage levels thread
+	 *	To be shure that (level) threads are not concurent
+	 */
+	vector<int>	start_process_thread_levels;
+	/**
+	 *	Starter is MOVING (starting servers) when > 0;
+	 */
+	int	starting_processes;
+
+public:
+	StartProcessShared() {starting_processes=0;};
+
+	void push_back_level(int level);
+	void remove_current_level();
+	int  get_current_level();
+	int  get_starting_processes();
+
+};
+
+
+#ifdef WIN32
+class StartWinThread: public omni_thread
+{
+	NewProcess* process;;
+public:
+	StartWinThread(NewProcess* proc) { process = proc; };
+
+	void *run_undetached(void *);
+	void start() {start_undetached();}
+};
+#endif
+
+//=========================================================
 /**
  *	Create a thread to fork a sub-process (new device server)
  */
+//=========================================================
 class StartProcessThread: public omni_thread
 {
 	vector<NewProcess *>	processes;
