@@ -1,6 +1,6 @@
+static const char *RcsId     = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v 3.18 2007/02/13 06:45:13 pascal_verdier Exp $";
 static const char *TagName   = "$Name:  $";
 static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v 3.16 2006/06/13 19:38:15 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :        StarterClass.cpp
@@ -14,9 +14,15 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.16 $
+// $Revision: 3.18 $
 //
 // $Log: StarterClass.cpp,v $
+// Revision 3.18  2007/02/13 06:45:13  pascal_verdier
+// *** empty log message ***
+//
+// Revision 3.17  2006/11/10 14:53:28  pascal_verdier
+// Remove vc8 warnings.
+//
 // Revision 3.16  2006/06/13 19:38:15  pascal_verdier
 // Minor changes.
 //
@@ -496,7 +502,7 @@ void StarterClass::command_factory()
 //+----------------------------------------------------------------------------
 Tango::DbDatum StarterClass::get_class_property(string &prop_name)
 {
-	for (int i=0 ; i<cl_prop.size() ; i++)
+	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
 		if (cl_prop[i].name == prop_name)
 			return cl_prop[i];
 	//	if not found, return  an empty DbDatum
@@ -511,7 +517,7 @@ Tango::DbDatum StarterClass::get_class_property(string &prop_name)
 //-----------------------------------------------------------------------------
 Tango::DbDatum StarterClass::get_default_device_property(string &prop_name)
 {
-	for (int i=0 ; i<dev_def_prop.size() ; i++)
+	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
 		if (dev_def_prop[i].name == prop_name)
 			return dev_def_prop[i];
 	//	if not found, return  an empty DbDatum
@@ -527,7 +533,7 @@ Tango::DbDatum StarterClass::get_default_device_property(string &prop_name)
 //-----------------------------------------------------------------------------
 Tango::DbDatum StarterClass::get_default_class_property(string &prop_name)
 {
-	for (int i=0 ; i<cl_def_prop.size() ; i++)
+	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
 		if (cl_def_prop[i].name == prop_name)
 			return cl_def_prop[i];
 	//	if not found, return  an empty DbDatum
