@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.10 2006/06/05 07:20:36 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.15 2007/05/25 06:08:00 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         StarterUtil.cpp
@@ -9,9 +9,24 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.10 $
+// $Revision: 3.15 $
 //
 // $Log: StarterUtil.cpp,v $
+// Revision 3.15  2007/05/25 06:08:00  pascal_verdier
+// Iterator management modified (strange in vc8 ?)
+//
+// Revision 3.14  2007/05/03 06:54:58  pascal_verdier
+// Re-try on DServer exported added before polling startup added.
+//
+// Revision 3.13  2007/03/30 14:21:49  pascal_verdier
+// remove 2 warnings on linux.
+//
+// Revision 3.12  2006/11/20 06:58:37  pascal_verdier
+// Mutex on start process data added.
+//
+// Revision 3.11  2006/11/10 14:53:28  pascal_verdier
+// Remove vc8 warnings.
+//
 // Revision 3.10  2006/06/05 07:20:36  pascal_verdier
 // Server startup is now delayed (with timeout) in a startup level.
 // New state MOVING added.
@@ -130,7 +145,7 @@ StarterUtil::StarterUtil(Tango::DeviceProxy *database, vector<string> host_names
 	ch_factory   = NULL;
 	
 	//	Remove the Fully Qualify Domain Name for tango less than 5.2 compatibility
-	for (int i=0 ; i<host_names.size() ; i++)
+	for (unsigned int i=0 ; i<host_names.size() ; i++)
 		hostnames.push_back(removeFQDN(host_names[i]));
 }
 //+------------------------------------------------------------------
@@ -240,7 +255,7 @@ char *StarterUtil::check_exe_file(string name)
 //-----------------------------------------------------------------------------
 char *StarterUtil::check_exe_file(char *servname, vector<string>v_path)
 {
-	int	i;	
+	unsigned int	i;	
 	for (i=0 ; i<v_path.size() ; i++)
 	{
 		string	filename(v_path[i]);
@@ -384,7 +399,7 @@ vector<string>	StarterUtil::get_host_ds_list()
 {
 	//	Read server info from database.
 	vector<string>		servnames;
-	for (int i=0 ; i<hostnames.size() ; i++)
+	for (unsigned int i=0 ; i<hostnames.size() ; i++)
 	{
 		Tango::DeviceData	argin;
 		argin << hostnames[i];
@@ -394,7 +409,6 @@ vector<string>	StarterUtil::get_host_ds_list()
 		argout >> tmp;
 
 		//	Check servers really used (Erase this one and database server
-		Tango::Util *tg = Tango::Util::instance();
 		vector<string>::iterator pos;
 		for (pos=tmp.begin() ; pos<tmp.end() ; pos++)
 		{
@@ -412,12 +426,12 @@ vector<string>	StarterUtil::get_host_ds_list()
 			}
 		}
 		//	Copy to global vector
-		for (int j=0 ; j<tmp.size() ; j++)
+		for (unsigned int j=0 ; j<tmp.size() ; j++)
 			servnames.push_back(tmp[j]);
 	}
 	cout << "----------------------------------------" << endl;
 	cout << servnames.size() << " servers found" << endl;
-	for (int j=0 ; j<servnames.size() ; j++)
+	for (unsigned int j=0 ; j<servnames.size() ; j++)
 		cout << "\t" <<  servnames[j]	<< endl;
 
 	return servnames;
@@ -462,8 +476,8 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 {
 	bool trace = false;
 	if (trace)	cout << "build_server_ctrl_object()" << endl;
-	int	i;
-	vector<string>	result;
+	unsigned int	i;
+	vector<string>	result_from_db;
 	for (i=0 ; i<hostnames.size() ; i++)
 	{
 		//	Call for servers and their info for each host
@@ -471,14 +485,15 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 		Tango::DeviceData	argin;
 		argin << hostnames[i];
 		Tango::DeviceData	argout = dbase->command_inout("DbGetHostServersInfo", argin);
-		argout >> result;
+		argout >> result_from_db;
 	}
 
 	if (trace)	cout << "--------------  Check if list of servers modified  --------------" << endl;
+	
 	//	Check servers really used (erase this one and database server
-	Tango::Util *tg = Tango::Util::instance();
 	vector<string>::iterator pos;
-	for (pos=result.begin() ; pos<result.end() ; pos+=3)
+	vector<string>	result;
+	for (pos=result_from_db.begin() ; pos<result_from_db.end() ; pos+=3)
 	{
 		int	idx = (*pos).find_first_of("/");
 		if (idx>0)
@@ -486,30 +501,38 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 			//	Get process name only in lower case before compeare
 			string	s = (*pos).substr(0, idx);
 			transform(s.begin(), s.end(), s.begin(), ::tolower);
-			if (s=="starter"  ||  s=="databaseds" || s=="logconsumer")
+			if (s!="starter"  &&  s!="databaseds" && s!="logconsumer")
 			{
-				result.erase(pos);
-				result.erase(pos);
-				result.erase(pos);
-				pos -=3;	//	because erase decrease size !
+				result.push_back(*pos);		//	Server name
+				result.push_back(*(pos+1));	//	Controlled/Not Controlled
+				result.push_back(*(pos+2));	//	Startup Level
 			}
 		}
 	}
+
 	//	Check if some servers have disappeared
 	vector<ControledServer>::iterator it;
-	for (it=servers->begin() ; it<servers->end() ; it++)
+	bool	redo = true;	//	Iterators management seems to have changed
+	                        //	between vc6 and vc8  (??)
+	while (redo)
 	{
-		string	s1(it->name);
-		bool	found = false;
-		for (i=0 ; !found && i<result.size() ; i+=3)
-			found = (s1==result[i]);
-
-		if (!found)
+		redo = false;
+		for (it=servers->begin() ; it<servers->end() ; it++)
 		{
-			if (trace)	cout << s1 << " has disappeared" << endl;
-			//	if disappeared then stop thread and remove reference
-			it->thread_data->set_stop_thread();
-			servers->erase(it);
+			string	s1(it->name);
+			bool	found = false;
+			for (i=0 ; !found && i<result.size() ; i+=3)
+				found = (s1==result[i]);
+
+			if (!found)
+			{
+				if (trace)	cout << s1 << " has disappeared" << endl;
+				//	if disappeared then stop thread and remove reference
+				it->thread_data->set_stop_thread();
+				servers->erase(it);
+				redo = true;
+				break;	//	get out of loop (vector size has changed.
+			}
 		}
 	}
 	//	Check if new servers appeared
@@ -556,7 +579,7 @@ void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 //+------------------------------------------------------------------
 ControledServer *StarterUtil::get_server_by_name(string &servname, vector<ControledServer> &servers)
 {
-	for (int i=0 ; i<servers.size() ; i++)
+	for (unsigned int i=0 ; i<servers.size() ; i++)
 	{
 		ControledServer	*server = &servers[i];
 		if (server->name == servname)
@@ -630,7 +653,7 @@ void StarterUtil::import_notifyd()
 
 	// Narrow the CORBA_Object reference to an EventChannelFactory
 	// reference so we can invoke its methods
-	CORBA::Object	*_eventChannelFactory =	event_factory_obj;
+	//CORBA::Object	*_eventChannelFactory =	event_factory_obj;
 
 	//	Test the connection
 	ch_factory =
@@ -694,13 +717,34 @@ PollingState::PollingState(string devname)
 //+----------------------------------------------------------------------------
 void *PollingState::run_undetached(void *ptr)
 {
-	//	wait a bit for dserver exported.
-	ms_sleep(1000);
-	long	upd_period = 1000;
-	add_obj_polling("attribute", "State",          upd_period);
-	add_obj_polling("attribute", "HostState",      upd_period);
-	add_obj_polling("attribute", "RunningServers", upd_period);
-	add_obj_polling("attribute", "StoppedServers", upd_period);
+	//	Check if dserver exported
+	bool		done = false;
+	Tango::Util *tg = Tango::Util::instance();
+	Tango::DServer		*adm_dev = NULL;
+	for (int i=0 ; !done && i<10 ; i++)
+	{
+		try
+		{
+			ms_sleep(500);
+			cout << "Try if DServer exported " << i << "/10" << endl;
+			adm_dev = tg->get_dserver_device();
+			adm_dev->ping();
+			done = true;
+		}
+		catch(Tango::DevFailed &e)
+		{
+			Tango::Except::print_exception(e);
+		}
+	}
+	//	If dserver not started , do nothing
+	if (adm_dev!=NULL)
+	{
+		long	upd_period = 1000;
+		add_obj_polling(adm_dev,"attribute", "State",          upd_period);
+		add_obj_polling(adm_dev,"attribute", "HostState",      upd_period);
+		add_obj_polling(adm_dev,"attribute", "RunningServers", upd_period);
+		add_obj_polling(adm_dev,"attribute", "StoppedServers", upd_period);
+	}
 	return NULL;
 }
  
@@ -709,10 +753,8 @@ void *PollingState::run_undetached(void *ptr)
  *	Add the Polling object.
  */
 //+------------------------------------------------------------------
-void PollingState::add_obj_polling(string type, string objname, int upd_period)
+void PollingState::add_obj_polling(Tango::DServer *adm_dev,string type, string objname, int upd_period)
 {
-	Tango::Util *tg = Tango::Util::instance();
-	Tango::DServer		*adm_dev = tg->get_dserver_device();
 	Tango::DevVarLongStringArray	*lsa = new Tango::DevVarLongStringArray();
 	lsa->lvalue.length(1);
 	lsa->svalue.length(3);
