@@ -8,9 +8,12 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.8 $
+// $Revision: 3.9 $
 //
 // $Log: StarterUtil.h,v $
+// Revision 3.9  2007/05/03 06:54:59  pascal_verdier
+// Re-try on DServer exported added before polling startup added.
+//
 // Revision 3.8  2006/04/24 07:06:28  pascal_verdier
 // A thread is started for each level when at servers startup.
 //
@@ -31,7 +34,7 @@
 
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.8 $
+ * @version	$Revision: 3.9 $
  */
 
 /**
@@ -263,7 +266,7 @@ private:
 	/**
 	 *	Add the object to polling list.
  	 */
-	void add_obj_polling(string type, string name, int period);
+	void add_obj_polling(Tango::DServer *adm_dev, string type, string name, int period);
 
 public:
 /**
