@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: api_util.cpp,v 3.8 2005/05/10 13:51:21 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: api_util.cpp,v 3.12 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api class ApiUtil
@@ -8,6 +8,21 @@ static const char *RcsId = "$Id: api_util.cpp,v 3.8 2005/05/10 13:51:21 taurel E
 // original 	- May 2002
 //
 // $Log: api_util.cpp,v $
+// Revision 3.12  2007/04/20 14:38:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.11  2007/04/16 14:55:16  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.10  2007/03/06 08:20:45  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.9  2007/02/16 10:38:52  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
 // Revision 3.8  2005/05/10 13:51:21  taurel
 // - Minor changes for Windows compiler
 //
@@ -216,7 +231,7 @@ ApiUtil::ApiUtil()
 	ext = new ApiUtilExt();
 		
 //
-// Create the Asynchronous poling request Id generator
+// Create the Asynchronous polling request Id generator
 //
 
 	UniqIdent *ui = new UniqIdent();
@@ -317,8 +332,8 @@ void ApiUtil::create_orb()
 //
 
 	_argc = 1;
-       	_argv = (char**)malloc(sizeof(char*));
-       	_argv[0] = (char*)"dummy";
+	_argv = (char**)malloc(sizeof(char*));
+	_argv[0] = (char*)"dummy";
 
 //
 // Init the ORB
@@ -544,7 +559,7 @@ void ApiUtil::get_asynch_replies(long call_timeout)
 //
 
 			long nb = call_timeout / 20;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 			struct timespec to_wait,inter;		
 			to_wait.tv_sec = 0;
 			to_wait.tv_nsec = 20000000;
@@ -552,7 +567,7 @@ void ApiUtil::get_asynch_replies(long call_timeout)
 				
 			while ((nb > 0) && (asyn_p_table->get_cb_request_nb() != 0))
 			{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				Sleep(20);
 #else
 				nanosleep(&to_wait,&inter);
@@ -766,11 +781,14 @@ void ApiUtil::attr_to_device(const AttributeValue *attr_value,
 	CORBA::UShort *tmp_ush;
 	const DevVarCharArray *tmp_seq_uch;
 	CORBA::Octet *tmp_uch;
-#ifdef TANGO_LONG64
+	const DevVarLong64Array *tmp_seq_64;
 	CORBA::LongLong *tmp_lolo;
-	const DevVarLong32Array *tmp_seq_lo32;
-	const DevVarLongLongArray *tmp_seq_lolo;
-#endif
+	const DevVarULongArray *tmp_seq_ulo;
+	CORBA::ULong *tmp_ulo;
+	const DevVarULong64Array *tmp_seq_u64;
+	CORBA::ULongLong *tmp_ulolo;
+	const DevVarStateArray *tmp_seq_state;
+	Tango::DevState *tmp_state;
 		
 	CORBA::ULong max,len;
 
@@ -806,212 +824,240 @@ void ApiUtil::attr_to_device(const AttributeValue *attr_value,
 		{
 			attr_value_3->value >>= dev_attr->d_state;
 			dev_attr->d_state_filled = true;
+			return;
 		}
-		else
+			
+		CORBA::TypeCode_var ty_alias = ty->content_type();
+		CORBA::TypeCode_var ty_seq = ty_alias->content_type();			
+		switch (ty_seq->kind())
 		{
-			CORBA::TypeCode_var ty_alias = ty->content_type();
-			CORBA::TypeCode_var ty_seq = ty_alias->content_type();			
-			switch (ty_seq->kind())
-			{
-				case CORBA::tk_long:
-#ifdef TANGO_LONG32
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_lo;
-					else		
-						attr_value->value >>= tmp_seq_lo;
-					max = tmp_seq_lo->maximum();
-					len = tmp_seq_lo->length();
-					if (tmp_seq_lo->release() == true)
-					{
-						tmp_lo = (const_cast<DevVarLongArray *>(tmp_seq_lo))->get_buffer((CORBA::Boolean)true);
-						dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lo,true);
-					}
-					else
-					{
-						tmp_lo = const_cast<CORBA::Long *>(tmp_seq_lo->get_buffer());
-						dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lo,false);
-					}
-#else
-					attr_value_3->value >>= tmp_seq_lo32;
-					max = tmp_seq_lo32->maximum();
-					len = tmp_seq_lo32->length();
-					if (tmp_seq_lo32->release() == true)
-					{
-						tmp_lo = (const_cast<DevVarLong32Array *>(tmp_seq_lo32))->get_buffer((CORBA::Boolean)true);
-						long *tmp_lo64 = DevVarLongArray::allocbuf(len);
-						for (unsigned int l = 0;l < len;l++)
-							tmp_lo64[l] = tmp_lo[l];
-						DevVarLong32Array::freebuf(tmp_lo);
-						dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lo64,true);
-					}
-					else
-					{
-						tmp_lo = const_cast<CORBA::Long *>(tmp_seq_lo32->get_buffer());
-						long *tmp_lo64 = DevVarLongArray::allocbuf(len);
-						for (unsigned int l = 0;l < len;l++)
-							tmp_lo64[l] = tmp_lo[l];
-						dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lo64,true);
-					}
-#endif
-					dev_attr->ext->long_from_64 = false;
-					break;
-		
-				case CORBA::tk_longlong:
-#ifdef TANGO_LONG64
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_lolo;
-					else		
-						attr_value->value >>= tmp_seq_lolo;
-					max = tmp_seq_lolo->maximum();
-					len = tmp_seq_lolo->length();
-					if (tmp_seq_lolo->release() == true)
-					{
-						tmp_lolo = (const_cast<DevVarLongLongArray *>(tmp_seq_lolo))->get_buffer((CORBA::Boolean)true);
-						dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lolo,true);
-					}
-					else
-					{
-						tmp_lolo = const_cast<CORBA::LongLong *>(tmp_seq_lolo->get_buffer());
-						dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lolo,false);
-					}
-#else
-					dev_attr->ext->long_from_64 = true;
-#endif
-					break;
-					
-				case CORBA::tk_short:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_sh;
-					else
-						attr_value->value >>= tmp_seq_sh;
-					max = tmp_seq_sh->maximum();
-					len = tmp_seq_sh->length();
-					if (tmp_seq_sh->release() == true)
-					{
-						tmp_sh = (const_cast<DevVarShortArray *>(tmp_seq_sh))->get_buffer((CORBA::Boolean)true);
-						dev_attr->ShortSeq = new DevVarShortArray(max,len,tmp_sh,true);
-					}
-					else
-					{
-						tmp_sh = const_cast<CORBA::Short *>(tmp_seq_sh->get_buffer());
-						dev_attr->ShortSeq = new DevVarShortArray(max,len,tmp_sh,false);
-					}
-					break;
-
-				case CORBA::tk_double:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_db;
-					else
-						attr_value->value >>= tmp_seq_db;
-					max = tmp_seq_db->maximum();
-					len = tmp_seq_db->length();
-					if (tmp_seq_db->release() == true)
-					{
-						tmp_db = (const_cast<DevVarDoubleArray *>(tmp_seq_db))->get_buffer((CORBA::Boolean)true);
-						dev_attr->DoubleSeq = new DevVarDoubleArray(max,len,tmp_db,true);
-					}
-					else
-					{
-						tmp_db = const_cast<CORBA::Double *>(tmp_seq_db->get_buffer());
-						dev_attr->DoubleSeq = new DevVarDoubleArray(max,len,tmp_db,false);
-					}
-					break;
-		
-				case CORBA::tk_string:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_str;
-					else
-						attr_value->value >>= tmp_seq_str;
-					max = tmp_seq_str->maximum();
-					len = tmp_seq_str->length();
-					if (tmp_seq_str->release() == true)
-					{
-						tmp_str = (const_cast<DevVarStringArray *>(tmp_seq_str))->get_buffer((CORBA::Boolean)true);
-						dev_attr->StringSeq = new DevVarStringArray(max,len,tmp_str,true);
-					}
-					else
-					{
-						tmp_str = const_cast<char **>(tmp_seq_str->get_buffer());
-						dev_attr->StringSeq = new DevVarStringArray(max,len,tmp_str,false);
-					}
-					break;
-
-				case CORBA::tk_float:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_fl;
-					else		
-						attr_value->value >>= tmp_seq_fl;
-					max = tmp_seq_fl->maximum();
-					len = tmp_seq_fl->length();
-					if (tmp_seq_fl->release() == true)
-					{
-						tmp_fl = (const_cast<DevVarFloatArray *>(tmp_seq_fl))->get_buffer((CORBA::Boolean)true);
-						dev_attr->FloatSeq = new DevVarFloatArray(max,len,tmp_fl,true);
-					}
-					else
-					{
-						tmp_fl = const_cast<CORBA::Float *>(tmp_seq_fl->get_buffer());
-						dev_attr->FloatSeq = new DevVarFloatArray(max,len,tmp_fl,false);
-					}
-					break;
+			case CORBA::tk_long:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_lo;
+				else		
+					attr_value->value >>= tmp_seq_lo;
+				max = tmp_seq_lo->maximum();
+				len = tmp_seq_lo->length();
+				if (tmp_seq_lo->release() == true)
+				{
+					tmp_lo = (const_cast<DevVarLongArray *>(tmp_seq_lo))->get_buffer((CORBA::Boolean)true);
+					dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lo,true);
+				}
+				else
+				{
+					tmp_lo = const_cast<CORBA::Long *>(tmp_seq_lo->get_buffer());
+					dev_attr->LongSeq = new DevVarLongArray(max,len,tmp_lo,false);
+				}
+				break;
+	
+			case CORBA::tk_longlong:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_64;
+				else		
+					attr_value->value >>= tmp_seq_64;
+				max = tmp_seq_64->maximum();
+				len = tmp_seq_64->length();
+				if (tmp_seq_64->release() == true)
+				{
+					tmp_lolo = (const_cast<DevVarLong64Array *>(tmp_seq_64))->get_buffer((CORBA::Boolean)true);
+					dev_attr->ext->Long64Seq = new DevVarLong64Array(max,len,tmp_lolo,true);
+				}
+				else
+				{
+					tmp_lolo = const_cast<CORBA::LongLong *>(tmp_seq_64->get_buffer());
+					dev_attr->ext->Long64Seq = new DevVarLong64Array(max,len,tmp_lolo,false);
+				}
+				break;
 				
-				case CORBA::tk_boolean:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_boo;
-					else		
-						attr_value->value >>= tmp_seq_boo;
-					max = tmp_seq_boo->maximum();
-					len = tmp_seq_boo->length();
-					if (tmp_seq_boo->release() == true)
-					{
-						tmp_boo = (const_cast<DevVarBooleanArray *>(tmp_seq_boo))->get_buffer((CORBA::Boolean)true);
-						dev_attr->BooleanSeq = new DevVarBooleanArray(max,len,tmp_boo,true);
-					}
-					else
-					{
-						tmp_boo = const_cast<CORBA::Boolean *>(tmp_seq_boo->get_buffer());
-						dev_attr->BooleanSeq = new DevVarBooleanArray(max,len,tmp_boo,false);
-					}
-					break;
+			case CORBA::tk_short:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_sh;
+				else
+					attr_value->value >>= tmp_seq_sh;
+				max = tmp_seq_sh->maximum();
+				len = tmp_seq_sh->length();
+				if (tmp_seq_sh->release() == true)
+				{
+					tmp_sh = (const_cast<DevVarShortArray *>(tmp_seq_sh))->get_buffer((CORBA::Boolean)true);
+					dev_attr->ShortSeq = new DevVarShortArray(max,len,tmp_sh,true);
+				}
+				else
+				{
+					tmp_sh = const_cast<CORBA::Short *>(tmp_seq_sh->get_buffer());
+					dev_attr->ShortSeq = new DevVarShortArray(max,len,tmp_sh,false);
+				}
+				break;
 
-				case CORBA::tk_ushort:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_ush;
-					else		
-						attr_value->value >>= tmp_seq_ush;
-					max = tmp_seq_ush->maximum();
-					len = tmp_seq_ush->length();
-					if (tmp_seq_ush->release() == true)
-					{
-						tmp_ush = (const_cast<DevVarUShortArray *>(tmp_seq_ush))->get_buffer((CORBA::Boolean)true);
-						dev_attr->UShortSeq = new DevVarUShortArray(max,len,tmp_ush,true);
-					}
-					else
-					{
-						tmp_ush = const_cast<CORBA::UShort *>(tmp_seq_ush->get_buffer());
-						dev_attr->UShortSeq = new DevVarUShortArray(max,len,tmp_ush,false);
-					}
-					break;
+			case CORBA::tk_double:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_db;
+				else
+					attr_value->value >>= tmp_seq_db;
+				max = tmp_seq_db->maximum();
+				len = tmp_seq_db->length();
+				if (tmp_seq_db->release() == true)
+				{
+					tmp_db = (const_cast<DevVarDoubleArray *>(tmp_seq_db))->get_buffer((CORBA::Boolean)true);
+					dev_attr->DoubleSeq = new DevVarDoubleArray(max,len,tmp_db,true);
+				}
+				else
+				{
+					tmp_db = const_cast<CORBA::Double *>(tmp_seq_db->get_buffer());
+					dev_attr->DoubleSeq = new DevVarDoubleArray(max,len,tmp_db,false);
+				}
+				break;
+	
+			case CORBA::tk_string:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_str;
+				else
+					attr_value->value >>= tmp_seq_str;
+				max = tmp_seq_str->maximum();
+				len = tmp_seq_str->length();
+				if (tmp_seq_str->release() == true)
+				{
+					tmp_str = (const_cast<DevVarStringArray *>(tmp_seq_str))->get_buffer((CORBA::Boolean)true);
+					dev_attr->StringSeq = new DevVarStringArray(max,len,tmp_str,true);
+				}
+				else
+				{
+					tmp_str = const_cast<char **>(tmp_seq_str->get_buffer());
+					dev_attr->StringSeq = new DevVarStringArray(max,len,tmp_str,false);
+				}
+				break;
+
+			case CORBA::tk_float:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_fl;
+				else		
+					attr_value->value >>= tmp_seq_fl;
+				max = tmp_seq_fl->maximum();
+				len = tmp_seq_fl->length();
+				if (tmp_seq_fl->release() == true)
+				{
+					tmp_fl = (const_cast<DevVarFloatArray *>(tmp_seq_fl))->get_buffer((CORBA::Boolean)true);
+					dev_attr->FloatSeq = new DevVarFloatArray(max,len,tmp_fl,true);
+				}
+				else
+				{
+					tmp_fl = const_cast<CORBA::Float *>(tmp_seq_fl->get_buffer());
+					dev_attr->FloatSeq = new DevVarFloatArray(max,len,tmp_fl,false);
+				}
+				break;
+			
+			case CORBA::tk_boolean:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_boo;
+				else		
+					attr_value->value >>= tmp_seq_boo;
+				max = tmp_seq_boo->maximum();
+				len = tmp_seq_boo->length();
+				if (tmp_seq_boo->release() == true)
+				{
+					tmp_boo = (const_cast<DevVarBooleanArray *>(tmp_seq_boo))->get_buffer((CORBA::Boolean)true);
+					dev_attr->BooleanSeq = new DevVarBooleanArray(max,len,tmp_boo,true);
+				}
+				else
+				{
+					tmp_boo = const_cast<CORBA::Boolean *>(tmp_seq_boo->get_buffer());
+					dev_attr->BooleanSeq = new DevVarBooleanArray(max,len,tmp_boo,false);
+				}
+				break;
+
+			case CORBA::tk_ushort:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_ush;
+				else		
+					attr_value->value >>= tmp_seq_ush;
+				max = tmp_seq_ush->maximum();
+				len = tmp_seq_ush->length();
+				if (tmp_seq_ush->release() == true)
+				{
+					tmp_ush = (const_cast<DevVarUShortArray *>(tmp_seq_ush))->get_buffer((CORBA::Boolean)true);
+					dev_attr->UShortSeq = new DevVarUShortArray(max,len,tmp_ush,true);
+				}
+				else
+				{
+					tmp_ush = const_cast<CORBA::UShort *>(tmp_seq_ush->get_buffer());
+					dev_attr->UShortSeq = new DevVarUShortArray(max,len,tmp_ush,false);
+				}
+				break;
+			
+			case CORBA::tk_octet:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_uch;
+				else		
+					attr_value->value >>= tmp_seq_uch;
+				max = tmp_seq_uch->maximum();
+				len = tmp_seq_uch->length();
+				if (tmp_seq_uch->release() == true)
+				{
+					tmp_uch = (const_cast<DevVarCharArray *>(tmp_seq_uch))->get_buffer((CORBA::Boolean)true);
+					dev_attr->UCharSeq = new DevVarCharArray(max,len,tmp_uch,true);
+				}
+				else
+				{
+					tmp_uch = const_cast<CORBA::Octet *>(tmp_seq_uch->get_buffer());
+					dev_attr->UCharSeq = new DevVarCharArray(max,len,tmp_uch,false);
+				}
+				break;
 				
-				case CORBA::tk_octet:
-					if (vers >= 3)
-						attr_value_3->value >>= tmp_seq_uch;
-					else		
-						attr_value->value >>= tmp_seq_uch;
-					max = tmp_seq_uch->maximum();
-					len = tmp_seq_uch->length();
-					if (tmp_seq_uch->release() == true)
-					{
-						tmp_uch = (const_cast<DevVarCharArray *>(tmp_seq_uch))->get_buffer((CORBA::Boolean)true);
-						dev_attr->UCharSeq = new DevVarCharArray(max,len,tmp_uch,true);
-					}
-					else
-					{
-						tmp_uch = const_cast<CORBA::Octet *>(tmp_seq_uch->get_buffer());
-						dev_attr->UCharSeq = new DevVarCharArray(max,len,tmp_uch,false);
-					}
-					break;
-			}
+			case CORBA::tk_ulong:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_ulo;
+				else		
+					attr_value->value >>= tmp_seq_ulo;
+				max = tmp_seq_ulo->maximum();
+				len = tmp_seq_ulo->length();
+				if (tmp_seq_ulo->release() == true)
+				{
+					tmp_ulo = (const_cast<DevVarULongArray *>(tmp_seq_ulo))->get_buffer((CORBA::Boolean)true);
+					dev_attr->ext->ULongSeq = new DevVarULongArray(max,len,tmp_ulo,true);
+				}
+				else
+				{
+					tmp_ulo = const_cast<CORBA::ULong *>(tmp_seq_ulo->get_buffer());
+					dev_attr->ext->ULongSeq = new DevVarULongArray(max,len,tmp_ulo,false);
+				}
+				break;
+				
+			case CORBA::tk_ulonglong:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_u64;
+				else		
+					attr_value->value >>= tmp_seq_u64;
+				max = tmp_seq_u64->maximum();
+				len = tmp_seq_u64->length();
+				if (tmp_seq_u64->release() == true)
+				{
+					tmp_ulolo = (const_cast<DevVarULong64Array *>(tmp_seq_u64))->get_buffer((CORBA::Boolean)true);
+					dev_attr->ext->ULong64Seq = new DevVarULong64Array(max,len,tmp_ulolo,true);
+				}
+				else
+				{
+					tmp_ulolo = const_cast<CORBA::ULongLong *>(tmp_seq_u64->get_buffer());
+					dev_attr->ext->ULong64Seq = new DevVarULong64Array(max,len,tmp_ulolo,false);
+				}
+				break;
+				
+			case CORBA::tk_enum:
+				if (vers >= 3)
+					attr_value_3->value >>= tmp_seq_state;
+				else		
+					attr_value->value >>= tmp_seq_state;
+				max = tmp_seq_state->maximum();
+				len = tmp_seq_state->length();
+				if (tmp_seq_state->release() == true)
+				{
+					tmp_state = (const_cast<DevVarStateArray *>(tmp_seq_state))->get_buffer((CORBA::Boolean)true);
+					dev_attr->ext->StateSeq = new DevVarStateArray(max,len,tmp_state,true);
+				}
+				else
+				{
+					tmp_state = const_cast<Tango::DevState *>(tmp_seq_state->get_buffer());
+					dev_attr->ext->StateSeq = new DevVarStateArray(max,len,tmp_state,false);
+				}
+				break;
 		}
 	}	
 }
@@ -1109,6 +1155,117 @@ ostream &operator<<(ostream &o_str,AttributeInfo &p)
 	return o_str;
 }
 
+//+-------------------------------------------------------------------------
+//
+// operator overloading : 	=
+// 
+// description : 	Assignement operator for the AttributeInfoEx class
+//					from a AttributeConfig_2 pointer
+//
+//--------------------------------------------------------------------------
+
+AttributeInfoEx &AttributeInfoEx::operator=(AttributeConfig_2 *att_2)
+{
+	name = att_2->name;
+	writable = att_2->writable;
+	data_format = att_2->data_format;
+	data_type = att_2->data_type;
+	max_dim_x = att_2->max_dim_x;
+	max_dim_y = att_2->max_dim_y;
+	description = att_2->description;
+	label = att_2->label;
+	unit = att_2->unit;
+	standard_unit = att_2->standard_unit;
+	display_unit = att_2->display_unit;
+	format = att_2->format;
+	min_value = att_2->min_value;
+	max_value = att_2->max_value;
+	min_alarm = att_2->min_alarm;
+	max_alarm = att_2->max_alarm;
+	writable_attr_name = att_2->writable_attr_name;
+	extensions.resize(att_2->extensions.length());
+	for (unsigned int j=0; j<att_2->extensions.length(); j++)
+	{
+		extensions[j] = att_2->extensions[j];
+	}
+	disp_level = att_2->level;
+	
+	return *this;
+}
+
+//+-------------------------------------------------------------------------
+//
+// operator overloading : 	=
+// 
+// description : 	Assignement operator for the AttributeInfoEx class
+//					from a AttributeConfig_3 pointer
+//
+//--------------------------------------------------------------------------
+
+AttributeInfoEx &AttributeInfoEx::operator=(AttributeConfig_3 *att_3)
+{
+	name = att_3->name;
+	writable = att_3->writable;
+	data_format = att_3->data_format;
+	data_type = att_3->data_type;
+	max_dim_x = att_3->max_dim_x;
+	max_dim_y = att_3->max_dim_y;
+	description = att_3->description;
+	label = att_3->label;
+	unit = att_3->unit;
+	standard_unit = att_3->standard_unit;
+	display_unit = att_3->display_unit;
+	format = att_3->format;
+	min_value = att_3->min_value;
+	max_value = att_3->max_value;
+	min_alarm = att_3->att_alarm.min_alarm;
+	max_alarm = att_3->att_alarm.max_alarm;
+	writable_attr_name = att_3->writable_attr_name;
+	extensions.resize(att_3->sys_extensions.length());
+	for (unsigned int j=0; j<att_3->sys_extensions.length(); j++)
+	{
+		extensions[j] = att_3->sys_extensions[j];
+	}
+	disp_level = att_3->level;
+	
+	alarms.min_alarm = att_3->att_alarm.min_alarm;
+	alarms.max_alarm = att_3->att_alarm.max_alarm;
+	alarms.min_warning = att_3->att_alarm.min_warning;
+	alarms.max_warning = att_3->att_alarm.max_warning;
+	alarms.delta_t = att_3->att_alarm.delta_t;
+	alarms.delta_val = att_3->att_alarm.delta_val;
+	alarms.extensions.resize(att_3->att_alarm.extensions.length());
+	for (unsigned int j=0; j<att_3->att_alarm.extensions.length(); j++)
+	{
+		alarms.extensions[j] = att_3->att_alarm.extensions[j];
+	}
+
+	events.ch_event.abs_change = att_3->event_prop.ch_event.abs_change;		
+	events.ch_event.rel_change = att_3->event_prop.ch_event.rel_change;
+	events.ch_event.extensions.resize(att_3->event_prop.ch_event.extensions.length());
+	for (unsigned int j=0; j<att_3->event_prop.ch_event.extensions.length(); j++)
+	{
+		events.ch_event.extensions[j] = att_3->event_prop.ch_event.extensions[j];
+	}
+
+	events.per_event.period = att_3->event_prop.per_event.period;		
+	events.per_event.extensions.resize(att_3->event_prop.per_event.extensions.length());
+	for (unsigned int j=0; j<att_3->event_prop.per_event.extensions.length(); j++)
+	{
+		events.per_event.extensions[j] = att_3->event_prop.per_event.extensions[j];
+	}
+	
+	events.arch_event.archive_abs_change = att_3->event_prop.arch_event.abs_change;		
+	events.arch_event.archive_rel_change = att_3->event_prop.arch_event.rel_change;
+	events.arch_event.archive_period = att_3->event_prop.arch_event.period;
+	events.arch_event.extensions.resize(att_3->event_prop.arch_event.extensions.length());
+	for (unsigned int j=0; j<att_3->event_prop.arch_event.extensions.length(); j++)
+	{
+		events.arch_event.extensions[j] = att_3->event_prop.arch_event.extensions[j];
+	}
+		
+	return *this;
+}
 
 //+-------------------------------------------------------------------------
 //
@@ -1202,30 +1359,63 @@ ostream &operator<<(ostream &o_str,AttributeInfoEx &p)
 	for (i = 0;i < p.extensions.size();i++)
 		o_str << "Attribute extensions " << i + 1 << " = " << p.extensions[i] << endl;
 
-	o_str << "Attribute alarm : min alarm = " << p.alarms.min_alarm << endl;	
-	o_str << "Attribute alarm : max alarm = " << p.alarms.max_alarm << endl;	
-	o_str << "Attribute warning alarm : min warning = " << p.alarms.min_warning << endl;	
-	o_str << "Attribute warning alarm : max warning = " << p.alarms.max_warning << endl;	
-	o_str << "Attribute rds alarm : delta time = " << p.alarms.delta_t << endl;	
-	o_str << "Attribute rds alarm : delta value = " << p.alarms.delta_val << endl;	
+	o_str << "Attribute alarm : min alarm = ";
+	p.alarms.min_alarm.empty() == true ? o_str << "Not specified" : o_str << p.alarms.min_alarm;
+	o_str << endl;
+		
+	o_str << "Attribute alarm : max alarm = ";
+	p.alarms.max_alarm.empty() == true ? o_str << "Not specified" : o_str << p.alarms.max_alarm;
+	o_str << endl;
+		
+	o_str << "Attribute warning alarm : min warning = ";
+	p.alarms.min_warning.empty() == true ? o_str << "Not specified" : o_str << p.alarms.min_warning;
+	o_str << endl;	
 
+	o_str << "Attribute warning alarm : max warning = ";
+	p.alarms.max_warning.empty() == true ? o_str << "Not specified" : o_str << p.alarms.max_warning;
+	o_str << endl;		
+
+	o_str << "Attribute rds alarm : delta time = ";
+	p.alarms.delta_t.empty() == true ? o_str << "Not specified" : o_str << p.alarms.delta_t;
+	o_str << endl;
+		
+	o_str << "Attribute rds alarm : delta value = ";
+	p.alarms.delta_val.empty() == true ? o_str << "Not specified" : o_str << p.alarms.delta_val;
+	o_str << endl;
+		
 	for (i = 0;i < p.alarms.extensions.size();i++)
 		o_str << "Attribute alarm extensions " << i + 1 << " = " << p.alarms.extensions[i] << endl;
 
-
-	o_str << "Attribute event : change event absolute change = " << p.events.ch_event.abs_change << endl;
-	o_str << "Attribute event : change event relative change = " << p.events.ch_event.rel_change << endl;
+	o_str << "Attribute event : change event absolute change = ";
+	p.events.ch_event.abs_change.empty() == true ? o_str << "Not specified" : o_str << p.events.ch_event.abs_change;
+	o_str << endl;
+		
+	o_str << "Attribute event : change event relative change = ";
+	p.events.ch_event.rel_change.empty() == true ? o_str << "Not specified" : o_str << p.events.ch_event.rel_change;
+	o_str << endl;
+		
 	for (i = 0;i < p.events.ch_event.extensions.size();i++)
 		o_str << "Attribute alarm : change event extensions " << i + 1 << " = " << p.events.ch_event.extensions[i] << endl;
 
-	o_str << "Attribute event : periodic event period = " << p.events.per_event.period << endl;
+	o_str << "Attribute event : periodic event period = ";
+	p.events.per_event.period.empty() == true ? o_str << "Not specified" : o_str << p.events.per_event.period;
+	o_str << endl;
+		
 	for (i = 0;i < p.events.per_event.extensions.size();i++)
 		o_str << "Attribute alarm : periodic event extensions " << i + 1 << " = " << p.events.per_event.extensions[i] << endl;
 					
-	o_str << "Attribute event : archive event absolute change = " << p.events.arch_event.archive_abs_change << endl;
-	o_str << "Attribute event : archive event relative change = " << p.events.arch_event.archive_rel_change << endl;
-	o_str << "Attribute event : archive event period = " << p.events.arch_event.archive_period;
-	
+	o_str << "Attribute event : archive event absolute change = ";
+	p.events.arch_event.archive_abs_change.empty() == true ? o_str << "Not specified" : o_str << p.events.arch_event.archive_abs_change;
+	o_str << endl;
+		
+	o_str << "Attribute event : archive event relative change = ";
+	p.events.arch_event.archive_rel_change.empty() == true ? o_str << "Not specified" : o_str << p.events.arch_event.archive_rel_change;
+	o_str << endl;
+		
+	o_str << "Attribute event : archive event period = ";
+	p.events.arch_event.archive_period.empty() == true ? o_str << "Not specified" : o_str << p.events.arch_event.archive_period;
+	o_str << endl;
+		
 	for (i = 0;i < p.events.arch_event.extensions.size();i++)
 	{
 		if (i == 0)
