@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: apiexcept.cpp,v 3.1 2004/07/07 08:39:55 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: apiexcept.cpp,v 3.3 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,18 @@ static const char *RcsId = "$Id: apiexcept.cpp,v 3.1 2004/07/07 08:39:55 taurel
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.1 $
+// $Revision: 3.3 $
 //
 // $Log: apiexcept.cpp,v $
+// Revision 3.3  2007/04/20 14:38:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.2  2007/04/16 14:55:16  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
 // Revision 3.1  2004/07/07 08:39:55  taurel
 //
 // - Fisrt commit after merge between Trunk and release 4 branch
@@ -109,7 +118,7 @@ NamedDevFailedList::NamedDevFailedList(const Tango::MultiDevFailed &corba_ex,
 	errors[0].reason = CORBA::string_dup(reason);
 	errors[0].origin = CORBA::string_dup(op_name);
 	
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	string st = desc.str();
 	errors[0].desc = CORBA::string_dup(st.c_str());
 #else
