@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: attr_proxy.cpp,v 1.5 2006/04/13 13:18:46 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: attr_proxy.cpp,v 1.7 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
 //
 // attr_proxy.cpp 	- C++ source code file for TANGO attribute proxy api 
 //
@@ -15,11 +15,11 @@ static const char *RcsId = "$Id: attr_proxy.cpp,v 1.5 2006/04/13 13:18:46 taurel
 
 #include <tango.h>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 #include <sys/timeb.h>
 #else
 #include <sys/time.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 #include <time.h>
 #include <signal.h>
@@ -198,7 +198,7 @@ void AttributeProxy::ctor_from_dp(const DeviceProxy *dev_ptr,string &att_name)
 		if (strcmp(dfe.errors[0].reason.in(),"API_AttrNotFound") == 0)
 		{
 			TangoSys_OMemStream desc;
-			desc << "Attribute" << attr_name << " is not supported by device " << device_name << ends;
+			desc << "Attribute " << attr_name << " is not supported by device " << device_name << ends;
 			
 			ApiWrongNameExcept::throw_exception((const char*)"API_UnsupportedAttribute",
 						desc.str(),
