@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: cbthread.cpp,v 3.2 2005/01/13 08:36:36 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: cbthread.cpp,v 3.3 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,12 @@ static const char *RcsId = "$Id: cbthread.cpp,v 3.2 2005/01/13 08:36:36 taurel E
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.2 $
+// $Revision: 3.3 $
 //
 // $Log: cbthread.cpp,v $
+// Revision 3.3  2007/04/20 14:38:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
 // Revision 3.2  2005/01/13 08:36:36  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -98,7 +101,7 @@ void *CallBackThread::run_undetached(void *ptr)
 		catch (omni_thread_fatal &)
 		{
 			cerr << "OUPS !! A omni thread fatal exception !!!!!!!!" << endl;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 			time_t t = time(NULL);
 			cerr << ctime(&t);
 #endif
