@@ -23,6 +23,7 @@ class DbDevInfo;
 class DbDevImportInfo;
 class DbDevExportInfo;
 class DbServerInfo;
+class DbHistory;
 
 class DatabaseExt;
 class DbDeviceExt;
@@ -35,7 +36,7 @@ typedef vector<DbDatum> DbData;
 typedef vector<DbDevInfo> DbDevInfos;
 typedef vector<DbDevExportInfo> DbDevExportInfos;
 typedef vector<DbDevImportInfo> DbDevImportInfos;
-
+ 
 //
 // Database - database object for implementing generic high-level
 //                 interface for TANGO database api
@@ -55,6 +56,8 @@ private :
 	FileDatabase 	*filedb; 
 	string 		file_name;
 	int		serv_version;
+	DbDatum         make_string_array(string, CORBA::Any_var &);
+	vector<DbHistory> make_history_array(bool, CORBA::Any_var &);
 
 public :
 	Database(CORBA::ORB *orb=NULL);
@@ -68,7 +71,7 @@ public :
 	inline Device_var &get_dbase() { return device;}
 	void check_tango_host(char *);
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	Database(CORBA::ORB *orb,string &,string &);
 	long get_tango_host_from_reg(char **,string &,string &);
 #endif
@@ -78,6 +81,11 @@ public :
 //
 
 	string get_info();
+	DbDatum get_host_list();
+	DbDatum get_host_list(string &);
+        DbDatum get_services(string &,string &);
+        void register_service(string &,string &,string &);
+        void unregister_service(string &,string &);
 	
 //
 // device methods
@@ -97,6 +105,11 @@ public :
 	void get_device_alias(string,string &);
 	void get_alias(string,string &);
 	DbDatum get_device_alias_list(string &);
+	string get_class_for_device(string &);
+	DbDatum get_class_inheritance_for_device(string &);
+	DbDatum get_device_exported_for_class(string &);
+	void put_device_alias(string &,string &);
+        void delete_device_alias(string &);
 	
 //
 // server methods
@@ -107,6 +120,15 @@ public :
 	void unexport_server(string &);
 	
 	DbServerInfo get_server_info(string &);
+	void put_server_info(DbServerInfo &);
+	void delete_server_info(string &);
+	DbDatum get_server_class_list(string &);
+	DbDatum get_server_name_list();
+	DbDatum get_instance_name_list(string &);
+	DbDatum get_server_list();
+	DbDatum get_server_list(string &);
+	DbDatum get_host_server_list(string &);
+        DbDatum get_device_class_list(string &);
 	
 //
 // property methods
@@ -114,30 +136,42 @@ public :
 
 	void get_property(string, DbData &);  
 	void put_property(string, DbData &);  
-	void delete_property(string, DbData &); 
-	DbDatum get_device_property_list(string &,string &);
+	void delete_property(string, DbData &);
+	vector<DbHistory> get_property_history(string &,string &);
+	DbDatum get_object_list(string &);
+	DbDatum get_object_property_list(string &,string &);
 
 	void get_device_property(string, DbData &);
 	void put_device_property(string, DbData &);
 	void delete_device_property(string, DbData &);
+	vector<DbHistory> get_device_property_history(string &,string &);
+	DbDatum get_device_property_list(string &,string &);
 	
 	void get_device_attribute_property(string, DbData &);
 	void put_device_attribute_property(string, DbData &);
 	void delete_device_attribute_property(string, DbData &);
+	vector<DbHistory> get_device_attribute_property_history(string &,string &,string &);
 	
 	void get_class_property(string, DbData &);
 	void put_class_property(string, DbData &);
 	void delete_class_property(string, DbData &);
+	vector<DbHistory> get_class_property_history(string &,string &);
+	DbDatum get_class_list(string &);
+	DbDatum get_class_property_list(string &);
 	
 	void get_class_attribute_property(string, DbData &);
 	void put_class_attribute_property(string, DbData &);
 	void delete_class_attribute_property(string, DbData &);
+	vector<DbHistory> get_class_attribute_property_history(string &,string &,string &);
+	DbDatum get_class_attribute_list(string &,string &);
 	
 
 // attribute methods
 
 	void get_attribute_alias(string, string&);
 	DbDatum get_attribute_alias_list(string &);
+	void put_attribute_alias(string &,string &);
+	void delete_attribute_alias(string &);
 
 };
 //
@@ -307,12 +341,15 @@ public :
 //
 // insert methods
 //
+
 	void operator << (bool);
 	void operator << (short);
 	void operator << (unsigned char);
 	void operator << (unsigned short);
-	void operator << (long);
-	void operator << (unsigned long);
+	void operator << (DevLong);
+	void operator << (DevULong);
+	void operator << (DevLong64);
+	void operator << (DevULong64);
 	void operator << (float);
 	void operator << (double);
 	void operator << (char *);
@@ -320,41 +357,87 @@ public :
 	void operator << (const char *);
 //	void operator << (const char *&);
 	void operator << (string&);
+	
 	void operator << (vector<string>&);
 	void operator << (vector<short>&);
 	void operator << (vector<unsigned short>&);
-	void operator << (vector<long>&);
-	void operator << (vector<unsigned long>&);
+	void operator << (vector<DevLong>&);
+	void operator << (vector<DevULong>&);
+	void operator << (vector<DevLong64>&);
+	void operator << (vector<DevULong64>&);
 	void operator << (vector<float>&);
 	void operator << (vector<double>&);
+	
 //
 // extract methods
 //
+
 	bool operator >> (bool&);
 	bool operator >> (short&);
 	bool operator >> (unsigned char&);
 	bool operator >> (unsigned short&);
-	bool operator >> (long&);
-	bool operator >> (unsigned long&);
+	bool operator >> (DevLong&);
+	bool operator >> (DevULong&);
+	bool operator >> (DevLong64&);
+	bool operator >> (DevULong64&);
 	bool operator >> (float&);
 	bool operator >> (double&);
 	bool operator >> (const char*&);
 	bool operator >> (string&);
+	
 	bool operator >> (vector<string>&);
 	bool operator >> (vector<short>&);
 	bool operator >> (vector<unsigned short>&);
-	bool operator >> (vector<long>&);
-	bool operator >> (vector<unsigned long>&);
+	bool operator >> (vector<DevLong>&);
+	bool operator >> (vector<DevULong>&);
+	bool operator >> (vector<DevLong64>&);
+	bool operator >> (vector<DevULong64>&);
 	bool operator >> (vector<float>&);
 	bool operator >> (vector<double>&);
 
 private :
 
-	int 			value_type;
-	int 			value_size;
+	int 				value_type;
+	int 				value_size;
 	bitset<numFlags> 	exceptions_flags;
 	
-	DbDatumExt		*ext;
+	DbDatumExt			*ext;
+};
+//
+// DbHistory data object for receiving data history from the
+//           TANGO database api
+//
+class DbHistory
+{
+public:
+
+//
+// constructor methods
+//
+
+  DbHistory(string ,string ,vector<string> &);
+  DbHistory(string ,string ,string ,vector<string> &);
+
+//
+// getter methods
+//
+
+  string get_name();
+  string get_attribute_name();
+  string get_date();
+  DbDatum get_value();
+  bool is_deleted();
+  
+private:
+  
+  string  propname;   // Property name
+  string  attname;    // Attribute name (Not used for device properties)
+  DbDatum value;      // Property value  
+  string  date;       // Update date
+  bool    deleted;    // Deleted flag
+  
+  string format_mysql_date(string );  
+  void make_db_datum(vector<string> &);
 };
 //
 // DbDevInfo
