@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_base.cpp,v 3.15 2006/08/11 13:51:57 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_base.cpp,v 3.18 2007/04/30 14:05:47 jlpons Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO dbapi class Database
@@ -12,6 +12,15 @@ static const char *RcsId = "$Id: dbapi_base.cpp,v 3.15 2006/08/11 13:51:57 taure
 // version 	- 1.0
 //
 // $Log: dbapi_base.cpp,v $
+// Revision 3.18  2007/04/30 14:05:47  jlpons
+// Added new commands to the database client interface.
+//
+// Revision 3.17  2007/04/20 14:38:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.16  2007/03/28 07:48:51  taurel
+// - Remove some unused 64 bits management
+//
 // Revision 3.15  2006/08/11 13:51:57  taurel
 // - Added the Database::get_device_property_list() method
 //
@@ -269,7 +278,7 @@ Database::Database(ORB *orb_in) : Connection(orb_in)
 	filedb = 0;
 	serv_version = 0;
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	tango_host_env_c_str = getenv(EnvVariable);
 #else	
        	tango_host_env_c_str = getenv(EnvVariable);
@@ -291,7 +300,7 @@ Database::Database(ORB *orb_in) : Connection(orb_in)
 	build_connection();		
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 Database::Database(ORB *orb_in,string &ds_exec_name,string &ds_inst_name) : Connection(orb_in)
 {
 //
@@ -511,7 +520,7 @@ Database::~Database()
 	}
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //+----------------------------------------------------------------------------
 //
 // method : 		Util::get_tango_host_from_reg()
@@ -694,19 +703,9 @@ DbDevImportInfo Database::import_device(string &dev)
 	{
 		DeviceData send_name;
 		send_name << dev;
-//#ifdef TANGO_LONG64
-//		received = command_inout("DbImportDevice2",send);
-//#else
 		received_cmd = command_inout("DbImportDevice",send_name);
 		received_cmd >> dev_import_list;
 	}
-//#endif
-
-//#ifdef TANGO_LONG64
-//	const DevVarLong32StringArray *dev_import_list;
-//	received.inout() >>= dev_import_list;
-//#else
-//#endif
 
 	DbDevImportInfo dev_import;
 	dev_import.exported = 0;
@@ -947,10 +946,10 @@ DbServerInfo Database::get_server_info(string &server)
 	received.inout() >>= server_info_list;
 
 	DbServerInfo server_info;
-	server_info.name = server;
-	server_info.host = string((*server_info_list)[0]);
-	server_info.mode = atoi((*server_info_list)[1]);
-	server_info.level = atoi((*server_info_list)[2]);
+	server_info.name = string((*server_info_list)[0]);
+	server_info.host = string((*server_info_list)[1]);
+	server_info.mode = atoi((*server_info_list)[2]);
+	server_info.level = atoi((*server_info_list)[3]);
 
 	return(server_info);
 }
@@ -2042,9 +2041,9 @@ DbDatum Database::get_device_domain(string &wildcard)
 	send <<= wildcard.c_str();
 
 	if (filedb != 0)
-		received = filedb->DbGetDeviceFamilyList(send);
+		received = filedb->DbGetDeviceDomainList(send);
 	else 
-		received = command_inout("DbGetDeviceFamilyList",send);
+		received = command_inout("DbGetDeviceDomainList",send);
 	const DevVarStringArray *device_domain;
 	received.inout() >>= device_domain;
 
@@ -2347,6 +2346,31 @@ DbDatum Database::get_attribute_alias_list(string &alias)
 	return db_datum;
 }
 
+//-----------------------------------------------------------------------------
+//
+// Database::make_string_array() - Build a string array DbDatum from received
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::make_string_array(string name,Any_var &received) {
+
+	const DevVarStringArray *prop_list;
+	DbDatum db_datum;
+	int n_props;
+	
+	received.inout() >>= prop_list;
+	n_props = prop_list->length();
+	db_datum.name = name;
+	db_datum.value_string.resize(n_props);
+	for (int i=0; i<n_props; i++)
+	{
+		db_datum.value_string[i] = (*prop_list)[i]; 
+	}
+		
+	return db_datum;
+
+}
+
 //-----------------------------------------------------------------------------
 //
 // Database::get_device_property_list() - public method to get a device property 
@@ -2367,20 +2391,856 @@ DbDatum Database::get_device_property_list(string &dev, string &wildcard)
 
 	received = command_inout("DbGetDevicePropertyList",send);
 	
+	return make_string_array(dev,received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_host_list() - Query the database for a list of host registered.
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_host_list()
+{
+	Any send;
+	Any_var received;
+	send <<= "*";
+
+	received = command_inout("DbGetHostList",send);
+	
+	return make_string_array(string("host"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_host_list() - Query the database for a list of host registred.
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_host_list(string &wildcard)
+{
+	Any send;
+	Any_var received;
+	send <<= wildcard.c_str();
+
+	received = command_inout("DbGetHostList",send);
+	
+	return make_string_array(string("host"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_server_class_list() - Query the database for a list of classes 
+//                                     instancied for a server
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_server_class_list(string &servname)
+{
+	Any send;
+	Any_var received;
+	send <<= servname.c_str();
+
+	received = command_inout("DbGetDeviceServerClassList",send);
+	
 	const DevVarStringArray *prop_list;
 	received.inout() >>= prop_list;
+	
+	// Extract the DServer class
 
 	DbDatum db_datum;
 	int n_props;
+	int nb_classes;
 	n_props = prop_list->length();
-	db_datum.name = dev;
-	db_datum.value_string.resize(n_props);
-	for (int i=0; i<n_props; i++)
+	if(n_props == 0 )
+	  nb_classes = 0;
+	else
+	  nb_classes = n_props - 1;
+		
+	db_datum.name = servname;
+	db_datum.value_string.resize(nb_classes);
+	for (int i=0,j=0; i<n_props; i++)
 	{
-		db_datum.value_string[i] = (*prop_list)[i]; 
+		if( strcmp( (*prop_list)[i] , "DServer" ) !=0 )
+			db_datum.value_string[j++] = (*prop_list)[i]; 
 	}
 		
 	return db_datum;
 }
 
+//-----------------------------------------------------------------------------
+//
+// Database::get_server_name_list() - Query the database for a list of server 
+//                                    names registred in the database
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_server_name_list()
+{
+	Any send;
+	Any_var received;
+	send <<= "*";
+
+	received = command_inout("DbGetServerNameList",send);
+	
+	return make_string_array(string("server"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_instance_name_list() - Query the database for a list of instance 
+//                                      names registred for specified server name
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_instance_name_list(string &servname)
+{
+	Any send;
+	Any_var received;
+	send <<= servname.c_str();
+
+	received = command_inout("DbGetInstanceNameList",send);
+	
+	return make_string_array(servname,received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_server_list() - Query the database for a list of servers registred 
+//                               in the database
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_server_list()
+{
+	Any send;
+	Any_var received;
+	send <<= "*";
+
+	received = command_inout("DbGetServerList",send);
+	
+	return make_string_array(string("server"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_server_list() - Query the database for a list of servers registred 
+//                               in the database
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_server_list(string &wildcard)
+{
+	Any send;
+	Any_var received;
+	send <<= wildcard.c_str();
+
+	received = command_inout("DbGetServerList",send);
+	
+	return make_string_array(string("server"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_host_server_list() - Query the database for a list of servers 
+//                                    registred on the specified host
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_host_server_list(string &hostname)
+{
+	Any send;
+	Any_var received;
+	send <<= hostname.c_str();
+
+	received = command_inout("DbGetHostServerList",send);
+	
+	return make_string_array(string("server"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::put_server_info() - Add/update server information in databse
+//
+//-----------------------------------------------------------------------------
+
+void Database::put_server_info(DbServerInfo &info)
+{
+
+#ifdef STRSTREAM
+	ostrstream ostream;
+#else
+	ostringstream ostream;
+#endif /* STRSTREAM */
+	Any send;
+	Any_var received;
+	DevVarStringArray *serv_info = new DevVarStringArray;
+	
+	serv_info->length(4);
+	(*serv_info)[0] = string_dup(info.name.c_str());
+	(*serv_info)[1] = string_dup(info.host.c_str());
+
+	ostream.seekp(0); ostream.clear();
+	ostream << info.mode << ends;
+#ifdef STRSTREAM
+	(*serv_info)[2] = string_dup(ostream.str());
+	ostream.rdbuf()->freeze(false);
+#else
+	string st = ostream.str();
+	(*serv_info)[2] = string_dup(st.c_str());
+#endif /* STRSTREAM */
+
+	ostream.seekp(0); ostream.clear();
+	ostream << info.level << ends;
+#ifdef STRSTREAM
+	(*serv_info)[3] = string_dup(ostream.str());
+	ostream.rdbuf()->freeze(false);
+#else
+	st = ostream.str();
+	(*serv_info)[3] = string_dup(st.c_str());
+#endif /* STRSTREAM */
+	
+	send <<= serv_info;
+
+	received = command_inout("DbPutServerInfo",send);
+
+	return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::delete_server_info() - Delete server information in databse
+//
+//-----------------------------------------------------------------------------
+
+void Database::delete_server_info(string &servname)
+{
+	Any send;
+	Any_var received;
+	send <<= servname.c_str();
+
+	received = command_inout("DbDeleteServerInfo",send);
+	
+	return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_device_class_list() - Query the database for server devices 
+//                                     and classes.
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_device_class_list(string &servname)
+{
+	Any send;
+	Any_var received;
+	send <<= servname.c_str();
+
+	received = command_inout("DbGetDeviceClassList",send);
+	
+	return make_string_array(string("server"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_object_list() - Query the database for a list of object 
+//                               (ie non-device) for which properties are defined
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_object_list(string &wildcard)
+{
+	Any send;
+	Any_var received;
+	send <<= wildcard.c_str();
+
+	received = command_inout("DbGetObjectList",send);
+	
+	return make_string_array(string("object"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_object_property_list() - Query the database for a list of
+//                                        properties defined for the specified
+//                                        object.
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_object_property_list(string &objname,string &wildcard)
+{
+	Any send;
+	Any_var received;
+	DevVarStringArray *obj_info = new DevVarStringArray;
+	
+	obj_info->length(2);
+	(*obj_info)[0] = string_dup(objname.c_str());
+	(*obj_info)[1] = string_dup(wildcard.c_str());
+	
+	send <<= obj_info;
+
+	received = command_inout("DbGetPropertyList",send);
+	
+	return make_string_array(string("object"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_property_list() - Query the database for a list of
+//                                       properties defined for the specified
+//                                       class.
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_class_property_list(string &classname)
+{
+	Any send;
+	Any_var received;	
+	send <<= classname.c_str();
+
+	received = command_inout("DbGetClassPropertyList",send);
+	
+	return make_string_array(string("class"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_for_device() - Return the class of the specified device
+//
+//-----------------------------------------------------------------------------
+
+string Database::get_class_for_device(string &devname)
+{
+	Any send;
+	Any_var received;	
+	send <<= devname.c_str();
+
+	received = command_inout("DbGetClassForDevice",send);
+	
+	const char *classname;
+	string ret_str;
+	received.inout() >>= classname;
+	ret_str = classname;
+	return ret_str;	
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_inheritance_for_device() - Return the class inheritance
+//                                                scheme of the specified device
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_class_inheritance_for_device(string &devname)
+{
+	Any send;
+	Any_var received;	
+	send <<= devname.c_str();
+
+	try {
+	
+	  received = command_inout("DbGetClassInheritanceForDevice",send);
+	  
+	} catch (DevFailed &e) {
+	
+	  // Check if an old API else re-throw
+	  if (strcmp(e.errors[0].reason.in(),"API_CommandNotFound") != 0) {
+	    throw e;
+	  } else {
+	    DbDatum db_datum;
+	    db_datum.name = "class";
+	    db_datum.value_string.resize(1);
+            db_datum.value_string[0] = "Device_3Impl"; 
+	    return db_datum;	    
+	  }
+	  
+	}
+	
+	return make_string_array(string("class"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_list() - Query the database for a list of classes
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_class_list(string &wildcard)
+{
+	Any send;
+	Any_var received;	
+	send <<= wildcard.c_str();
+
+	received = command_inout("DbGetClassList",send);
+	
+	return make_string_array(string("class"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_attribute_list() - Query the database for a list of
+//                                        attributes defined for the specified
+//                                        class.
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_class_attribute_list(string &classname,string &wildcard)
+{
+	Any send;
+	Any_var received;
+	DevVarStringArray *class_info = new DevVarStringArray;
+	
+	class_info->length(2);
+	(*class_info)[0] = string_dup(classname.c_str());
+	(*class_info)[1] = string_dup(wildcard.c_str());
+	
+	send <<= class_info;
+
+	received = command_inout("DbGetClassAttributeList",send);
+	
+	return make_string_array(string("class"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_device_exported_for_class() - Query database for list of exported
+//                                             devices for the specified class
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_device_exported_for_class(string &classname)
+{
+	Any send;
+	Any_var received;	
+	send <<= classname.c_str();
+
+	received = command_inout("DbGetExportdDeviceListForClass",send);
+	
+	return make_string_array(string("device"),received);
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::put_device_alias() - Set an alias for a device
+//
+//-----------------------------------------------------------------------------
+
+void Database::put_device_alias(string &devname,string &aliasname)
+{
+	Any send;
+	Any_var received;
+	DevVarStringArray *alias_info = new DevVarStringArray;
+	
+	alias_info->length(2);
+	(*alias_info)[0] = string_dup(devname.c_str());
+	(*alias_info)[1] = string_dup(aliasname.c_str());
+	
+	send <<= alias_info;
+
+	received = command_inout("DbPutDeviceAlias",send);
+	
+	return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::delete_device_alias() - Delete a device alias from the database
+//
+//-----------------------------------------------------------------------------
+
+void Database::delete_device_alias(string &aliasname)
+{
+	Any send;
+	Any_var received;
+	send <<= aliasname.c_str();
+
+	received = command_inout("DbDeleteDeviceAlias",send);
+	
+	return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::put_attribute_alias() - Set an alias for an attribute
+//
+//-----------------------------------------------------------------------------
+
+void Database::put_attribute_alias(string &attname,string &aliasname)
+{
+	Any send;
+	Any_var received;
+	DevVarStringArray *alias_info = new DevVarStringArray;
+	
+	alias_info->length(2);
+	(*alias_info)[0] = string_dup(attname.c_str());
+	(*alias_info)[1] = string_dup(aliasname.c_str());
+	
+	send <<= alias_info;
+
+	received = command_inout("DbPutAttributeAlias",send);
+	
+	return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::delete_attribute_alias() - Delete an attribute alias from the database
+//
+//-----------------------------------------------------------------------------
+
+void Database::delete_attribute_alias(string &aliasname)
+{
+	Any send;
+	Any_var received;
+	send <<= aliasname.c_str();
+
+	received = command_inout("DbDeleteAttributeAlias",send);
+	
+	return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::make_history_array() - Convert the result of a DbGet...PropertyHist 
+//                                  command.
+//
+//-----------------------------------------------------------------------------
+
+vector<DbHistory> Database::make_history_array(bool is_attribute, Any_var &received) {
+
+	const DevVarStringArray *ret;	
+	received.inout() >>= ret;
+	
+	vector<DbHistory> v;
+	int		  i=0;
+	int		  count=0;
+	int		  offset;
+	string	aName = "";
+	string	pName;
+	string	pDate;
+	string	pCount;
+
+	while(i<ret->length())
+	{
+		if(is_attribute)
+		{
+			aName = (*ret)[i];
+			pName = (*ret)[i+1];
+			pDate = (*ret)[i+2];
+			pCount = (*ret)[i+3];
+			offset = 4;
+		}
+		else
+		{
+			pName = (*ret)[i];
+			pDate = (*ret)[i+1];
+			pCount = (*ret)[i+2];
+			offset = 3;
+		}
+
+#ifdef STRSTREAM
+		istrstream istream(pCount.c_str());
+#else
+		istringstream istream(pCount);
+#endif /* STRSTREAM */
+		istream >> count;
+		if (!istream)
+		{
+			Except::throw_exception( (const char *)"API_HistoryInvalid",
+					         (const char *)"History format is invalid", 
+					         (const char *)"Database::make_history_array()");
+		}
+		vector<string> value;
+		for(int j=0;j<count;j++)
+		 value.push_back( string((*ret)[i+offset+j]) );
+
+		if (is_attribute)
+			v.push_back(DbHistory(aName,pName,pDate,value));
+		else
+			v.push_back(DbHistory(pName,pDate,value));
+
+		i += (count+offset);
+	}
+
+	return v;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_property_history() - Returns the history of the specified object
+//                                    property
+//
+//-----------------------------------------------------------------------------
+
+vector<DbHistory> Database::get_property_history(string &objname,string &propname) {
+
+	Any send;
+	Any_var received;
+	DevVarStringArray *obj_info = new DevVarStringArray;
+	
+	obj_info->length(2);
+	(*obj_info)[0] = string_dup(objname.c_str());
+	(*obj_info)[1] = string_dup(propname.c_str());
+	
+	send <<= obj_info;
+
+	received = command_inout("DbGetPropertyHist",send);
+	
+	return make_history_array(false,received);
+  
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_device_property_history() - Returns the history of the specified
+//                                           device property
+//
+//-----------------------------------------------------------------------------
+
+vector<DbHistory> Database::get_device_property_history(string &devname,string &propname) {
+
+	Any send;
+	Any_var received;
+	DevVarStringArray *obj_info = new DevVarStringArray;
+	
+	obj_info->length(2);
+	(*obj_info)[0] = string_dup(devname.c_str());
+	(*obj_info)[1] = string_dup(propname.c_str());
+	
+	send <<= obj_info;
+
+	received = command_inout("DbGetDevicePropertyHist",send);
+	
+	return make_history_array(false,received);
+  
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_device_attribute_property_history() - Returns the history of 
+//                                                     the specified device
+//                                                     attribute property
+//
+//-----------------------------------------------------------------------------
+
+vector<DbHistory> Database::get_device_attribute_property_history(string &devname,string &attname,string &propname) {
+
+	Any send;
+	Any_var received;
+	DevVarStringArray *obj_info = new DevVarStringArray;
+	
+	obj_info->length(3);
+	(*obj_info)[0] = string_dup(devname.c_str());
+	(*obj_info)[1] = string_dup(attname.c_str());
+	(*obj_info)[2] = string_dup(propname.c_str());
+	
+	send <<= obj_info;
+
+	received = command_inout("DbGetDeviceAttributePropertyHist",send);
+	
+	return make_history_array(true,received);
+  
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_property_history() - Returns the history of the specified
+//                                          class property
+//
+//-----------------------------------------------------------------------------
+
+vector<DbHistory> Database::get_class_property_history(string &classname,string &propname) {
+
+	Any send;
+	Any_var received;
+	DevVarStringArray *obj_info = new DevVarStringArray;
+	
+	obj_info->length(2);
+	(*obj_info)[0] = string_dup(classname.c_str());
+	(*obj_info)[1] = string_dup(propname.c_str());
+	
+	send <<= obj_info;
+
+	received = command_inout("DbGetClassPropertyHist",send);
+	
+	return make_history_array(false,received);
+  
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_class_attribute_property_history() - Returns the history of 
+//                                                    the specified class
+//                                                    attribute property
+//
+//-----------------------------------------------------------------------------
+
+vector<DbHistory> Database::get_class_attribute_property_history(string &classname,string &attname,string &propname) {
+
+	Any send;
+	Any_var received;
+	DevVarStringArray *obj_info = new DevVarStringArray;
+	
+	obj_info->length(3);
+	(*obj_info)[0] = string_dup(classname.c_str());
+	(*obj_info)[1] = string_dup(attname.c_str());
+	(*obj_info)[2] = string_dup(propname.c_str());
+	
+	send <<= obj_info;
+
+	received = command_inout("DbGetClassAttributePropertyHist",send);
+	
+	return make_history_array(true,received);
+  
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::get_services() - Query database for specified services
+//
+//-----------------------------------------------------------------------------
+
+DbDatum Database::get_services(string &servname,string &instname)
+{
+
+  DbData data;
+  DbDatum db_datum;
+  vector<string> services;
+  vector<string> filter_services;
+  
+  // Get list of services
+  data.push_back( DbDatum("Services") );
+  get_property( "CtrlSystem" , data );
+  data[0] >> services;
+  
+  // Filter 
+  string filter = servname+"/";
+  if( strcmp( instname.c_str() , "*")!=0 )
+    filter += instname + ":";
+    
+  transform(filter.begin(),filter.end(),filter.begin(),::tolower);
+    
+  for(int i=0;i<services.size();i++) {
+  
+    transform(services[i].begin(),services[i].end(),services[i].begin(),::tolower);
+    if( strncmp( services[i].c_str() , filter.c_str() , filter.length() )==0 ) {
+      string::size_type pos;
+      pos = services[i].find(':');
+      if( pos != string::npos ) {
+        filter_services.push_back( services[i].substr(pos+1) );
+      }
+    }
+    
+  }
+
+  // Build return value  
+  db_datum.name = "services";
+  db_datum.value_string.resize(filter_services.size());
+  for (int i=0; i<filter_services.size(); i++)
+    db_datum.value_string[i] = filter_services[i]; 
+  
+  return db_datum;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::register_service() - Register a new service
+//
+//-----------------------------------------------------------------------------
+
+void Database::register_service(string &servname,string &instname,string &devname)
+{
+
+  DbData data;
+  vector<string> services;
+  vector<string> new_services;
+  bool service_exists = false;
+  
+  // Get list of services
+  data.push_back( DbDatum("Services") );
+  get_property( "CtrlSystem" , data );
+  data[0] >> services;
+  
+  string full_name = servname+"/"+instname+":";
+  transform(full_name.begin(),full_name.end(),full_name.begin(),::tolower);
+
+  // Update service list    
+  for(int i=0;i<services.size();i++) {
+
+    string service_lower = services[i];
+    transform(service_lower.begin(),service_lower.end(),service_lower.begin(),::tolower);
+    if( strncmp(service_lower.c_str(),full_name.c_str(),full_name.length())==0 ) {
+
+      // The service already exists, update the device name
+      string::size_type pos;
+      pos = services[i].find(':');
+      if( pos != string::npos ) {
+        new_services.push_back( services[i].substr(0,pos) + ":" + devname );
+	service_exists = true;
+      }
+      
+    } else {
+    
+      new_services.push_back( services[i] );
+      
+    }
+    
+  }
+  
+  if( !service_exists ) {
+    // Add the new service
+    new_services.push_back( servname + "/" + instname + ":" + devname );
+  }
+
+  // Update the property
+  data[0] << new_services;
+  put_property("CtrlSystem",data);
+  
+  return;
+}
+
+//-----------------------------------------------------------------------------
+//
+// Database::unregister_service() - Unregister a service
+//
+//-----------------------------------------------------------------------------
+
+void Database::unregister_service(string &servname,string &instname) {
+
+  DbData data;
+  vector<string> services;
+  vector<string> new_services;
+  bool service_deleted = false;
+  
+  // Get list of services
+  data.push_back( DbDatum("Services") );
+  get_property( "CtrlSystem" , data );
+  data[0] >> services;
+  
+  string full_name = servname+"/"+instname+":";
+  transform(full_name.begin(),full_name.end(),full_name.begin(),::tolower);
+
+  // Update service list    
+  for(int i=0;i<services.size();i++) {
+
+    string service_lower = services[i];
+    transform(service_lower.begin(),service_lower.end(),service_lower.begin(),::tolower);
+    if( strncmp(service_lower.c_str(),full_name.c_str(),full_name.length())!=0 ) {
+      new_services.push_back( services[i] );
+    } else {
+      service_deleted = true;    
+    }
+    
+  }
+  
+  if( service_deleted ) {
+    // Update the property
+    data[0] << new_services;
+    put_property("CtrlSystem",data);
+  }
+  
+  return;
+}
+
 } // End of Tango namespace
