@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.3 2005/01/13 08:36:36 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.4 2007/03/06 08:20:45 taurel Exp $\n$Name:  $";
 //
 // dbdatum.cpp 	- C++ source code file for TANGO dbapi class DbDatum
 //
@@ -11,6 +11,9 @@ static const char *RcsId = "$Id: dbapi_datum.cpp,v 3.3 2005/01/13 08:36:36 taure
 // version 	- 1.0
 //
 // $Log: dbapi_datum.cpp,v $
+// Revision 3.4  2007/03/06 08:20:45  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.3  2005/01/13 08:36:36  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -544,11 +547,11 @@ bool DbDatum::operator >> (unsigned short& datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator <<(long &) - insert a long into DbDatum
+// DbDatum::operator <<(DevLong &) - insert a DevLong into DbDatum
 //
 //-----------------------------------------------------------------------------
 
-void DbDatum::operator << (long datum)
+void DbDatum::operator << (DevLong datum)
 {
 #ifdef STRSTREAM
 	ostrstream ostream;
@@ -570,11 +573,11 @@ void DbDatum::operator << (long datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator >>(long &) - extract a long from DbDatum
+// DbDatum::operator >>(DevLong &) - extract a long from DbDatum
 //
 //-----------------------------------------------------------------------------
 
-bool DbDatum::operator >> (long& datum)
+bool DbDatum::operator >> (DevLong& datum)
 {
 	bool ret;
 	if (value_string.size() == 0)
@@ -600,8 +603,8 @@ bool DbDatum::operator >> (long& datum)
 			if (exceptions_flags.test(wrongtype_flag))
 			{
 				ApiDataExcept::throw_exception((const char *)"API_IncompatibleArgumentType",
-							       (const char *)"Cannot extract, data in DbDatum is not a long",
-							       (const char *)"DbDatum::operator >>(long)");
+							       (const char *)"Cannot extract, data in DbDatum is not a DevLong (long 32 bits)",
+							       (const char *)"DbDatum::operator >>(DevLong)");
 			}
 			ret = false;
 		}
@@ -614,11 +617,11 @@ bool DbDatum::operator >> (long& datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator <<(unsigned long &) - insert a unsigned short into DbDatum
+// DbDatum::operator <<(DevULong &) - insert a unsigned short into DbDatum
 //
 //-----------------------------------------------------------------------------
 
-void DbDatum::operator << (unsigned long datum)
+void DbDatum::operator << (DevULong datum)
 {
 #ifdef STRSTREAM
 	ostrstream ostream;
@@ -640,11 +643,11 @@ void DbDatum::operator << (unsigned long datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator >>(unsigned long &) - extract a unsigned short from DbDatum
+// DbDatum::operator >>(DevULong &) - extract a DevULong from DbDatum
 //
 //-----------------------------------------------------------------------------
 
-bool DbDatum::operator >> (unsigned long& datum)
+bool DbDatum::operator >> (DevULong& datum)
 {
 	bool ret;
 	if (value_string.size() == 0)
@@ -670,8 +673,148 @@ bool DbDatum::operator >> (unsigned long& datum)
 			if (exceptions_flags.test(wrongtype_flag))
 			{
 				ApiDataExcept::throw_exception((const char *)"API_IncompatibleArgumentType",
-							       (const char *)"Cannot extract, data in DbDatum is not an unsigned long",
-							       (const char *)"DbDatum::operator >>(unsigned long)");
+							       (const char *)"Cannot extract, data in DbDatum is not a DevULong (unsigned long 32 bits)",
+							       (const char *)"DbDatum::operator >>(DevULong)");
+			}
+			ret = false;
+		}
+		else
+			ret = true;
+	}
+	
+	return ret;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator <<(DevLong64 &) - insert a DevLong64 into DbDatum
+//
+//-----------------------------------------------------------------------------
+
+void DbDatum::operator << (DevLong64 datum)
+{
+#ifdef STRSTREAM
+	ostrstream ostream;
+#else
+	ostringstream ostream;
+#endif /* STRSTREAM */
+	ostream << datum;
+#ifdef STRSTREAM
+	ostream << ends;
+#endif /* STRSTREAM */
+	value_string.resize(1);
+	value_string[0] = string(ostream.str());
+#ifdef STRSTREAM
+	delete [] ostream.str();
+#endif /* STRSTREAM */
+	value_type = DEV_LONG64;
+	value_size = 1;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator >>(DevLong64 &) - extract a DevLong64 from DbDatum
+//
+//-----------------------------------------------------------------------------
+
+bool DbDatum::operator >> (DevLong64 &datum)
+{
+	bool ret;
+	if (value_string.size() == 0)
+	{
+		if (exceptions_flags.test(isempty_flag))
+		{
+			ApiDataExcept::throw_exception((const char*)"API_EmptyDbDatum",
+					     (const char*)"cannot extract unsigned long, no data in DbDatum object ",
+					     (const char*)"DbDatum::operator >>(DevLong64)");
+		}
+		ret = false;
+	}
+	else
+	{
+#ifdef STRSTREAM
+		istrstream istream(value_string[0].c_str());
+#else
+		istringstream istream(value_string[0]);
+#endif /* STRSTREAM */
+		istream >> datum;
+		if (!istream)
+		{
+			if (exceptions_flags.test(wrongtype_flag))
+			{
+				ApiDataExcept::throw_exception((const char *)"API_IncompatibleArgumentType",
+							       (const char *)"Cannot extract, data in DbDatum is not a DevLong64 (long 64 bits)",
+							       (const char *)"DbDatum::operator >>(DevULong)");
+			}
+			ret = false;
+		}
+		else
+			ret = true;
+	}
+	
+	return ret;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator <<(DevULong64 &) - insert a DevULong64 into DbDatum
+//
+//-----------------------------------------------------------------------------
+
+void DbDatum::operator << (DevULong64 datum)
+{
+#ifdef STRSTREAM
+	ostrstream ostream;
+#else
+	ostringstream ostream;
+#endif /* STRSTREAM */
+	ostream << datum;
+#ifdef STRSTREAM
+	ostream << ends;
+#endif /* STRSTREAM */
+	value_string.resize(1);
+	value_string[0] = string(ostream.str());
+#ifdef STRSTREAM
+	delete [] ostream.str();
+#endif /* STRSTREAM */
+	value_type = DEV_ULONG64;
+	value_size = 1;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator >>(DevULong64 &) - extract a DevULong64 from DbDatum
+//
+//-----------------------------------------------------------------------------
+
+bool DbDatum::operator >> (DevULong64 &datum)
+{
+	bool ret;
+	if (value_string.size() == 0)
+	{
+		if (exceptions_flags.test(isempty_flag))
+		{
+			ApiDataExcept::throw_exception((const char*)"API_EmptyDbDatum",
+					     (const char*)"cannot extract unsigned long, no data in DbDatum object ",
+					     (const char*)"DbDatum::operator >>(DevLong64)");
+		}
+		ret = false;
+	}
+	else
+	{
+#ifdef STRSTREAM
+		istrstream istream(value_string[0].c_str());
+#else
+		istringstream istream(value_string[0]);
+#endif /* STRSTREAM */
+		istream >> datum;
+		if (!istream)
+		{
+			if (exceptions_flags.test(wrongtype_flag))
+			{
+				ApiDataExcept::throw_exception((const char *)"API_IncompatibleArgumentType",
+							       (const char *)"Cannot extract, data in DbDatum is not a DevULong64 (unsigned long 64 bits)",
+							       (const char *)"DbDatum::operator >>(DevULong)");
 			}
 			ret = false;
 		}
@@ -1118,11 +1261,11 @@ bool DbDatum::operator >> (vector<unsigned short>& datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator <<(vector<long> &) - insert a vector<long> into DbDatum
+// DbDatum::operator <<(vector<DevLong> &) - insert a vector<DevLong> into DbDatum
 //
 //-----------------------------------------------------------------------------
 
-void DbDatum::operator << (vector<long>& datum)
+void DbDatum::operator << (vector<DevLong>& datum)
 {
 #ifdef STRSTREAM
 	ostrstream ostream;
@@ -1150,11 +1293,11 @@ void DbDatum::operator << (vector<long>& datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator >>(vector<long> &) - extract a vector<long> from DbDatum
+// DbDatum::operator >>(vector<DevLong> &) - extract a vector<DevLong> from DbDatum
 //
 //-----------------------------------------------------------------------------
 
-bool DbDatum::operator >> (vector<long>& datum)
+bool DbDatum::operator >> (vector<DevLong>& datum)
 {
 	bool ret = true;
 	if (value_string.size() == 0)
@@ -1187,11 +1330,11 @@ bool DbDatum::operator >> (vector<long>& datum)
 				{
 					TangoSys_OMemStream desc;
 					desc << "Cannot extract long vector, elt number ";
-					desc << i+1 << " is not a long" << ends;
+					desc << i+1 << " is not a DevLong (long 32 bits)" << ends;
 
 					ApiDataExcept::throw_exception((const char*)"API_IncompatibleArgumentType",
 					     desc.str(),
-					     (const char*)"DbDatum::operator >>(vector<long>)");
+					     (const char*)"DbDatum::operator >>(vector<DevLong>)");
 				}
 				ret = false;
 				break;
@@ -1204,11 +1347,11 @@ bool DbDatum::operator >> (vector<long>& datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator <<(vector<unsigned long> &) - insert a vector<unsigned long> into DbDatum
+// DbDatum::operator <<(vector<DevULong> &) - insert a vector<DevULong> into DbDatum
 //
 //-----------------------------------------------------------------------------
 
-void DbDatum::operator << (vector<unsigned long>& datum)
+void DbDatum::operator << (vector<DevULong>& datum)
 {
 #ifdef STRSTREAM
 	ostrstream ostream;
@@ -1236,11 +1379,11 @@ void DbDatum::operator << (vector<unsigned long>& datum)
 
 //-----------------------------------------------------------------------------
 //
-// DbDatum::operator >>(vector<unsigned long> &) - extract a vector<unsigned long> from DbDatum
+// DbDatum::operator >>(vector<DevULong> &) - extract a vector<DevULong> from DbDatum
 //
 //-----------------------------------------------------------------------------
 
-bool DbDatum::operator >> (vector<unsigned long>& datum)
+bool DbDatum::operator >> (vector<DevULong>& datum)
 {
 	bool ret = true;
 	if (value_string.size() == 0)
@@ -1273,11 +1416,183 @@ bool DbDatum::operator >> (vector<unsigned long>& datum)
 				{
 					TangoSys_OMemStream desc;
 					desc << "Cannot extract unsigned long vector, elt number ";
-					desc << i+1 << " is not an unsigned long" << ends;
+					desc << i+1 << " is not a DevULong (unsigned long 32 bits)" << ends;
 
 					ApiDataExcept::throw_exception((const char*)"API_IncompatibleArgumentType",
 					     desc.str(),
+					     (const char*)"DbDatum::operator >>(vector<DevULong>)");
+				}
+				ret = false;
+				break;
+			}
+		}
+	}
+	
+	return ret;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator <<(vector<DevLong64> &) - insert a vector<DevLong64> into DbDatum
+//
+//-----------------------------------------------------------------------------
+
+void DbDatum::operator << (vector<DevLong64>& datum)
+{
+#ifdef STRSTREAM
+	ostrstream ostream;
+#else
+	ostringstream ostream;
+#endif /* STRSTREAM */
+	value_string.resize(datum.size());
+	for (unsigned int i=0; i<datum.size(); i++)
+	{
+		ostream << datum[i];
+#ifdef STRSTREAM
+		ostream << ends;
+#endif
+		value_string[i] = ostream.str();
+#ifdef STRSTREAM
+		ostream.rdbuf()->freeze(false);
+		ostream.seekp(0);
+#else
+		ostream.str("");
+#endif /* STRSTREAM */
+	}
+	value_type = DEVVAR_LONG64ARRAY;
+	value_size = datum.size();
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator >>(vector<DevLong64> &) - extract a vector<DevLong64> from DbDatum
+//
+//-----------------------------------------------------------------------------
+
+bool DbDatum::operator >> (vector<DevLong64>& datum)
+{
+	bool ret = true;
+	if (value_string.size() == 0)
+	{
+		if (exceptions_flags.test(isempty_flag))
+		{
+			ApiDataExcept::throw_exception((const char*)"API_EmptyDbDatum",
+					     (const char*)"cannot extract unsigned long vector, no data in DbDatum object ",
 					     (const char*)"DbDatum::operator >>(vector<unsigned long>)");
+		}
+		datum.resize(0);
+		ret = false;
+	}
+	else
+	{
+#ifdef STRSTREAM
+		strstream iostream;
+#else
+		stringstream iostream;
+#endif /* STRSTREAM */
+		datum.resize(value_string.size());
+		for (unsigned int i=0; i<value_string.size(); i++)
+		{
+			iostream.seekp (0); iostream.seekg(0); iostream.clear(); 
+			iostream << value_string[i] << ends;
+			iostream >> datum[i];
+			if (!iostream)
+			{
+				if (exceptions_flags.test(wrongtype_flag))
+				{
+					TangoSys_OMemStream desc;
+					desc << "Cannot extract unsigned long vector, elt number ";
+					desc << i+1 << " is not a DevLong64 (long 64 bits)" << ends;
+
+					ApiDataExcept::throw_exception((const char*)"API_IncompatibleArgumentType",
+					     desc.str(),
+					     (const char*)"DbDatum::operator >>(vector<DevLong64>)");
+				}
+				ret = false;
+				break;
+			}
+		}
+	}
+	
+	return ret;
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator <<(vector<DevULong64> &) - insert a vector<DevULong64> into DbDatum
+//
+//-----------------------------------------------------------------------------
+
+void DbDatum::operator << (vector<DevULong64>& datum)
+{
+#ifdef STRSTREAM
+	ostrstream ostream;
+#else
+	ostringstream ostream;
+#endif /* STRSTREAM */
+	value_string.resize(datum.size());
+	for (unsigned int i=0; i<datum.size(); i++)
+	{
+		ostream << datum[i];
+#ifdef STRSTREAM
+		ostream << ends;
+#endif
+		value_string[i] = ostream.str();
+#ifdef STRSTREAM
+		ostream.rdbuf()->freeze(false);
+		ostream.seekp(0);
+#else
+		ostream.str("");
+#endif /* STRSTREAM */
+	}
+	value_type = DEVVAR_ULONG64ARRAY;
+	value_size = datum.size();
+}
+
+//-----------------------------------------------------------------------------
+//
+// DbDatum::operator >>(vector<DevULong64> &) - extract a vector<DevULong64> from DbDatum
+//
+//-----------------------------------------------------------------------------
+
+bool DbDatum::operator >> (vector<DevULong64>& datum)
+{
+	bool ret = true;
+	if (value_string.size() == 0)
+	{
+		if (exceptions_flags.test(isempty_flag))
+		{
+			ApiDataExcept::throw_exception((const char*)"API_EmptyDbDatum",
+					     (const char*)"cannot extract unsigned long vector, no data in DbDatum object ",
+					     (const char*)"DbDatum::operator >>(vector<unsigned long>)");
+		}
+		datum.resize(0);
+		ret = false;
+	}
+	else
+	{
+#ifdef STRSTREAM
+		strstream iostream;
+#else
+		stringstream iostream;
+#endif /* STRSTREAM */
+		datum.resize(value_string.size());
+		for (unsigned int i=0; i<value_string.size(); i++)
+		{
+			iostream.seekp (0); iostream.seekg(0); iostream.clear(); 
+			iostream << value_string[i] << ends;
+			iostream >> datum[i];
+			if (!iostream)
+			{
+				if (exceptions_flags.test(wrongtype_flag))
+				{
+					TangoSys_OMemStream desc;
+					desc << "Cannot extract unsigned long vector, elt number ";
+					desc << i+1 << " is not a DevULong64 (unsigned long 64 bits)" << ends;
+
+					ApiDataExcept::throw_exception((const char*)"API_IncompatibleArgumentType",
+					     desc.str(),
+					     (const char*)"DbDatum::operator >>(vector<DevULong64>)");
 				}
 				ret = false;
 				break;
