@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: devapi_datahist.cpp,v 3.9 2005/05/10 13:56:46 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: devapi_datahist.cpp,v 3.10 2007/03/06 08:20:45 taurel Exp $\n$Name:  $";
 
 //
 // devapi_datahist.cpp 	- C++ source code file for TANGO devapi class 
@@ -9,6 +9,9 @@ static const char *RcsId = "$Id: devapi_datahist.cpp,v 3.9 2005/05/10 13:56:46 t
 // original 		- June 2002
 //
 // $Log: devapi_datahist.cpp,v $
+// Revision 3.10  2007/03/06 08:20:45  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.9  2005/05/10 13:56:46  taurel
 // - Error in the Windows change !!
 //
@@ -357,10 +360,8 @@ DeviceAttributeHistory::DeviceAttributeHistory(int n,DevAttrHistoryList_var &seq
 	
 	const DevVarLongArray *tmp_seq_lo;
 	CORBA::Long *tmp_lo;
-#ifdef TANGO_LONG64
-	const DevVarLongLongArray *tmp_seq_lolo;
+	const DevVarLong64Array *tmp_seq_lolo;
 	CORBA::LongLong *tmp_lolo;
-#endif
 	const DevVarShortArray *tmp_seq_sh;
 	CORBA::Short *tmp_sh;
 	const DevVarDoubleArray *tmp_seq_db;
@@ -386,23 +387,19 @@ DeviceAttributeHistory::DeviceAttributeHistory(int n,DevAttrHistoryList_var &seq
 		switch (ty_seq->kind())
 		{
 		case tk_long:
-#ifndef TANGO_LONG64		
 			seq[n].value.value >>= tmp_seq_lo;
 			max = tmp_seq_lo->maximum();
 			len = tmp_seq_lo->length();
 			tmp_lo = (const_cast<DevVarLongArray *>(tmp_seq_lo))->get_buffer((CORBA::Boolean)true);
 			LongSeq = new DevVarLongArray(max,len,tmp_lo,true);
-#endif
 			break;
 		
 		case tk_longlong:
-#ifndef TANGO_LONG32		
 			seq[n].value.value >>= tmp_seq_lolo;
 			max = tmp_seq_lolo->maximum();
 			len = tmp_seq_lolo->length();
-			tmp_lolo = (const_cast<DevVarLongLongArray *>(tmp_seq_lolo))->get_buffer((CORBA::Boolean)true);
-			LongSeq = new DevVarLongArray(max,len,tmp_lolo,true);
-#endif
+			tmp_lolo = (const_cast<DevVarLong64Array *>(tmp_seq_lolo))->get_buffer((CORBA::Boolean)true);
+			ext->Long64Seq = new DevVarLong64Array(max,len,tmp_lolo,true);
 			break;
 			
 		case tk_short:
@@ -481,10 +478,8 @@ DeviceAttributeHistory::DeviceAttributeHistory(int n,DevAttrHistoryList_3_var &s
 
 	const DevVarLongArray *tmp_seq_lo;
 	CORBA::Long *tmp_lo;
-#ifdef TANGO_LONG64
-	const DevVarLongLongArray *tmp_seq_lolo;
+	const DevVarLong64Array *tmp_seq_lolo;
 	CORBA::LongLong *tmp_lolo;
-#endif
 	const DevVarShortArray *tmp_seq_sh;
 	CORBA::Short *tmp_sh;
 	const DevVarDoubleArray *tmp_seq_db;
@@ -510,23 +505,19 @@ DeviceAttributeHistory::DeviceAttributeHistory(int n,DevAttrHistoryList_3_var &s
 		switch (ty_seq->kind())
 		{
 		case tk_long:
-#ifndef TANGO_LONG64		
 			seq[n].value.value >>= tmp_seq_lo;
 			max = tmp_seq_lo->maximum();
 			len = tmp_seq_lo->length();
 			tmp_lo = (const_cast<DevVarLongArray *>(tmp_seq_lo))->get_buffer((CORBA::Boolean)true);
 			LongSeq = new DevVarLongArray(max,len,tmp_lo,true);
-#endif
 			break;
 		
 		case tk_longlong:
-#ifndef TANGO_LONG32		
 			seq[n].value.value >>= tmp_seq_lolo;
 			max = tmp_seq_lolo->maximum();
 			len = tmp_seq_lolo->length();
-			tmp_lolo = (const_cast<DevVarLongLongArray *>(tmp_seq_lolo))->get_buffer((CORBA::Boolean)true);
-			LongSeq = new DevVarLongArray(max,len,tmp_lolo,true);
-#endif
+			tmp_lolo = (const_cast<DevVarLong64Array *>(tmp_seq_lolo))->get_buffer((CORBA::Boolean)true);
+			ext->Long64Seq = new DevVarLong64Array(max,len,tmp_lolo,true);
 			break;
 			
 		case tk_short:
@@ -812,6 +803,8 @@ ostream &operator<<(ostream &o_str,DeviceAttributeHistory &dah)
 					o_str << *(dah.UShortSeq.operator->());
 				else if (dah.UCharSeq.operator->() != NULL)
 					o_str << *(dah.UCharSeq.operator->());
+				else if (dah.ext->Long64Seq.operator->() != NULL)
+					o_str << *(dah.ext->Long64Seq.operator->());
 				else
 					o_str << *(dah.StringSeq.operator->());
 			}	
