@@ -30,7 +30,7 @@
 
 // DbGetDeviceMemberList
 // DbGetDeviceExportedList
-// DbGetDeviceFamilyList
+// DbGetDeviceDomainList
 // DbGetDeviceFamilyList
 // DbGetProperty
 // DbPutProperty
@@ -1921,7 +1921,8 @@ CORBA::Any*  FileDatabase :: DbGetDeviceList(CORBA::Any& send)
 			unsigned int i;		
 			for(i = 0; i < m_server.classes.size(); i++)
 			{
-				if ( strcmp((*data_in)[1], m_server.classes[i]->name.c_str()) == 0 )
+				//if ( strcmp((*data_in)[1], m_server.classes[i]->name.c_str()) == 0 )
+				if ( equalsIgnoreCase ((*data_in)[1].in(), m_server.classes[i]->name))
 				{
 					data_out->length(m_server.classes[i]->devices.size());
 					for (unsigned int j = 0; j < m_server.classes[i]->devices.size(); j++)
@@ -2002,7 +2003,7 @@ CORBA::Any*  FileDatabase :: DbInfo(CORBA::Any&){
 
 CORBA::Any*  FileDatabase :: DbImportDevice(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
@@ -2014,7 +2015,7 @@ CORBA::Any*  FileDatabase :: DbImportDevice(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbExportDevice(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbExportDevice");
@@ -2025,7 +2026,7 @@ CORBA::Any*  FileDatabase :: DbExportDevice(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbUnExportDevice(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbUnExportDevice");
@@ -2036,7 +2037,7 @@ CORBA::Any*  FileDatabase :: DbUnExportDevice(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbAddDevice(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbAddDevice");
@@ -2047,7 +2048,7 @@ CORBA::Any*  FileDatabase :: DbAddDevice(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbDeleteDevice(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbDeleteDevice");
@@ -2058,7 +2059,7 @@ CORBA::Any*  FileDatabase :: DbDeleteDevice(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbAddServer(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbAddServer");
@@ -2069,7 +2070,7 @@ CORBA::Any*  FileDatabase :: DbAddServer(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbDeleteServer(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbDeleteServer");
@@ -2080,7 +2081,7 @@ CORBA::Any*  FileDatabase :: DbDeleteServer(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbExportServer(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbExportServer");
@@ -2091,7 +2092,7 @@ CORBA::Any*  FileDatabase :: DbExportServer(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbUnExportServer(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbExportDevice");
@@ -2102,7 +2103,7 @@ CORBA::Any*  FileDatabase :: DbUnExportServer(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetServerInfo(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"Filedatabase::DbGetServerInfo");
@@ -2127,7 +2128,7 @@ CORBA::Any*  FileDatabase :: DbGetDeviceMemberList(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetDeviceExportedList(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"DbGetDeviceExportedList");
@@ -2148,6 +2149,17 @@ CORBA::Any*  FileDatabase :: DbGetDeviceFamilyList(CORBA::Any&)
 	return any_ptr;
 };
 
+CORBA::Any*  FileDatabase :: DbGetDeviceDomainList(CORBA::Any&)
+{ 
+	CORBA::Any* any_ptr = new CORBA::Any();	
+	
+	Tango::DevVarStringArray* argout = new Tango::DevVarStringArray();
+	argout->length(1);
+	(*argout)[0] = CORBA::string_dup("NoDevice");
+	(*any_ptr) <<= argout;
+	
+	return any_ptr;
+};
 
 /** At the moment we have no information about general properties
  * so I put nothing out
@@ -2179,7 +2191,7 @@ CORBA::Any*  FileDatabase :: DbGetProperty(CORBA::Any& send)
 
 CORBA::Any*  FileDatabase :: DbPutProperty(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
 					                       (const char *)"DbPutProperty");
@@ -2189,7 +2201,7 @@ CORBA::Any*  FileDatabase :: DbPutProperty(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbDeleteProperty(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
@@ -2200,7 +2212,7 @@ CORBA::Any*  FileDatabase :: DbDeleteProperty(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetAliasDevice(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
@@ -2211,7 +2223,7 @@ CORBA::Any*  FileDatabase :: DbGetAliasDevice(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetDeviceAlias(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
@@ -2222,7 +2234,7 @@ CORBA::Any*  FileDatabase :: DbGetDeviceAlias(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetAttributeAlias(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
@@ -2233,7 +2245,7 @@ CORBA::Any*  FileDatabase :: DbGetAttributeAlias(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetDeviceAliasList(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
@@ -2244,7 +2256,7 @@ CORBA::Any*  FileDatabase :: DbGetDeviceAliasList(CORBA::Any&)
 
 CORBA::Any*  FileDatabase :: DbGetAttributeAliasList(CORBA::Any&)
 { 
-	CORBA::Any* ret = new CORBA::Any;
+	CORBA::Any* ret = NULL;
 	
 	Tango::Except::throw_exception((const char *)"Method not implemented.",
 	   				                     (const char *)"Call to a Filedatabase not implemented.",
