@@ -8,6 +8,22 @@
 //
 // author(s) :          N.Leclercq
 //
+// $Revision: 3.14 $
+//
+// $Log: group.cpp,v $
+// Revision 3.14  2007/03/06 08:20:45  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.13  2006/10/24 13:11:41  bourtemb
+// Add the possibility to set the timeout for each DeviceProxy of the group.
+// Add set_timeout_millis() methods in Group, GroupElement and GroupDeviceElement
+// classes.
+// Add an optional timeout parameter to GroupElementFactory::instanciate() method
+// Add an optional timeout parameter to GroupElement::Add(), Group::Add() and
+// GroupDeviceElement methods.
+// Add a constructor to GroupDeviceElement class with a timeout parameter.
+//
+//
 // copyleft :           European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
 //                      FRANCE
@@ -31,7 +47,7 @@ bool GroupReply::exception_enabled = false;
 //=============================================================================
 // class GroupElementFactory
 //=============================================================================
-GroupElements GroupElementFactory::instanciate (const std::string& p)
+GroupElements GroupElementFactory::instanciate (const std::string& p, int timeout_ms)
 {
 #if defined(_LOCAL_DEBUGGING)
   cerr << "GroupElementFactory::instanciate::pattern [" << p << "]" << endl;
@@ -63,10 +79,10 @@ GroupElements GroupElementFactory::instanciate (const std::string& p)
   GroupElements tel(0);
   GroupDeviceElement* tde;
   for (unsigned int i = 0; i < dnl.size(); i++) {
-    tde = new GroupDeviceElement(dnl[i]);
+    tde = new GroupDeviceElement(dnl[i],timeout_ms);
     if (tde) {
       tel.push_back(tde);
-    } 
+    }
   }
 #if defined(_LOCAL_DEBUGGING)
     cerr << "\t|- GroupElementList contains " << tel.size() << " elements" << endl;
@@ -213,7 +229,7 @@ GroupCmdReply::~GroupCmdReply ()
   return data_m; 
 }
 //-----------------------------------------------------------------------------
-bool GroupCmdReply::extract (std::vector<long>& vl, std::vector<std::string>& vs) 
+bool GroupCmdReply::extract (std::vector<DevLong>& vl, std::vector<std::string>& vs) 
 {
   bool result = true;
   if (group_element_enabled_m == false) 
@@ -437,12 +453,12 @@ GroupElement::~GroupElement()
   //- noop dtor
 }
 //-----------------------------------------------------------------------------
-void GroupElement::add (const std::string& s)
+void GroupElement::add (const std::string& s, int timeout_ms)
 {
   //- noop default impl
 }
 //-----------------------------------------------------------------------------
-void GroupElement::add (const std::vector<std::string>& sl)
+void GroupElement::add (const std::vector<std::string>& sl, int timeout_ms)
 {
   //- noop default impl
 }
@@ -660,7 +676,7 @@ long Group::get_size_i (bool fwd)
   return size;
 }
 //-----------------------------------------------------------------------------
-void Group::add (Group* g)
+void Group::add (Group* g, int timeout_ms)
 {
 #ifdef TANGO_GROUP_HAS_THREAD_SAFE_IMPL
   omni_mutex_lock guard(elements_mutex);
@@ -672,34 +688,52 @@ void Group::add (Group* g)
 #if defined(_LOCAL_DEBUGGING)
     cerr << "Group::add::failed to add group" << (g ? (g->get_name() + " (self ref. or already in group)") : "NULL") << endl;
 #endif
+    if(g != 0) {
+      g->set_timeout_millis(timeout_ms);
+    }
     return;
   }
   add_i(g, false);
+  g->set_timeout_millis(timeout_ms);
 }
 //-----------------------------------------------------------------------------
-void Group::add (const std::string& p)
+void Group::add (const std::string& p, int timeout_ms)
 {
 #ifdef TANGO_GROUP_HAS_THREAD_SAFE_IMPL
   omni_mutex_lock guard(elements_mutex);
 #endif  
-  GroupElements el = GroupElementFactory::instanciate(p);
+  GroupElements el = GroupElementFactory::instanciate(p,timeout_ms);
   for (unsigned int e = 0; e < el.size(); e++) {
-    if (el[e] && add_i(el[e]) == false) {
+    if (el[e] && (add_i(el[e]) == false)) {
+      GroupElement* te = find_i(el[e]->get_name());
+      try {
+        te->set_timeout_millis(timeout_ms);
+      }
+      catch(...) {
+        // ignore errors
+      }
       delete el[e];
     }
   }
 }
 //-----------------------------------------------------------------------------
-void Group::add (const std::vector<std::string>& pl)
+void Group::add (const std::vector<std::string>& pl, int timeout_ms)
 {
 #ifdef TANGO_GROUP_HAS_THREAD_SAFE_IMPL
   omni_mutex_lock guard(elements_mutex);
 #endif
   for (unsigned int p = 0; p < pl.size(); p++) {
-    GroupElements el = GroupElementFactory::instanciate(pl[p]);
+    GroupElements el = GroupElementFactory::instanciate(pl[p],timeout_ms);
     for (unsigned int e = 0; e < el.size(); e++) {
-      if (el[e] && add_i(el[e]) == false) {
-        delete el[e];
+      if (el[e] && (add_i(el[e]) == false)) {
+        GroupElement* te = find_i(el[e]->get_name());
+        try {
+          te->set_timeout_millis(timeout_ms);
+        }
+	catch(...) {
+	  // ignore errors
+	}
+	delete el[e];
       }
     }
   }
@@ -950,6 +984,20 @@ bool Group::ping (bool fwd)
   return result;
 }
 //-----------------------------------------------------------------------------
+void Group::set_timeout_millis(int timeout_ms)
+{
+  GroupElementsIterator it = elements.begin();
+  GroupElementsIterator end = elements.end();
+  for (; it != end ; ++it) {
+    try {
+      (*it)->set_timeout_millis(timeout_ms); 
+    }
+    catch(...) {
+      // ignore errors
+    }
+  }
+}
+//-----------------------------------------------------------------------------
 GroupCmdReplyList Group::command_inout (const std::string& c, bool fwd)
 {
   long id = command_inout_asynch(c, false, fwd);
@@ -1316,6 +1364,18 @@ GroupDeviceElement::GroupDeviceElement (const std::string& name)
   } 
 }
 //-----------------------------------------------------------------------------
+GroupDeviceElement::GroupDeviceElement (const std::string& name, int timeout_ms) 
+  : GroupElement(name), dp(0)
+{
+  try {
+    connect();
+    set_timeout_millis(timeout_ms);
+  }
+  catch (...) {
+    //- ignore error
+  } 
+}
+//-----------------------------------------------------------------------------
 GroupDeviceElement::~GroupDeviceElement () 
 {
   disconnect();
@@ -1914,6 +1974,13 @@ long GroupDeviceElement::get_size (bool fwd)
 {
   return 1;
 }
+//-----------------------------------------------------------------------------
+// the following function could throw an exception, see DeviceProxy::set_timeout_millis()
+void GroupDeviceElement::set_timeout_millis (int timeout)
+{
+    if(timeout >= 0)
+      dev_proxy()->set_timeout_millis(timeout);
+}
 
 } // namespace Tango
 
