@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.7 2005/01/13 08:36:37 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.10 2007/04/20 14:38:33 taurel Exp $\n$Name:  $";
 
 //
 // cpp 	- C++ source code file for TANGO api Connection and DeviceProxy
@@ -9,6 +9,18 @@ static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.7 2005/01/13 08:36:37 taurel
 // original 	- August 2002
 //
 // $Log: proxy_asyn.cpp,v $
+// Revision 3.10  2007/04/20 14:38:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.9  2007/04/16 14:55:16  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.8  2007/02/14 14:12:17  taurel
+// - Fix bug in asynchronous calls reconnection
+//
 // Revision 3.7  2005/01/13 08:36:37  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -111,7 +123,7 @@ static const char *RcsId = "$Id: proxy_asyn.cpp,v 3.7 2005/01/13 08:36:37 taurel
 #include <tango.h>
 
 #define _dyn_attr
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#include <sys/timeb.h>
 #else
 	#include <sys/time.h>
@@ -190,7 +202,6 @@ long Connection::command_inout_asynch(const char *command, DeviceData &data_in,
 	{
 		request->send_oneway();
 		CORBA::release(request);
-//		connection_state = CONNECTION_NOTOK;
 	}
 	
 	return id;
@@ -386,7 +397,7 @@ DeviceData Connection::command_inout_reply(long id)
 				connection_state = CONNECTION_NOTOK;
 			
 //
-// Re-throw all CORBA system exceptions
+// Re-throw nearly all CORBA system exceptions
 //
 
 				CORBA::NVList_ptr req_arg = req.request->arguments();
@@ -396,6 +407,42 @@ DeviceData Connection::command_inout_reply(long id)
 				char *tmp = CORBA::string_dup(cmd);
 
 				char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(sys_ex);
+				
+//
+// Check if the exception was a connection exception
+// In this case, try to ping the device.
+// If successfull, just returns otherwise, throw the first exception
+//
+
+				string ex(cb_excep_mess);
+				string::size_type pos = ex.find("TRANSIENT_ConnectFailed");
+				if (pos != string::npos)
+				{
+					try
+					{
+						DeviceProxy *dev_ptr = static_cast<DeviceProxy *>(this);
+						dev_ptr->ping();
+						CORBA::string_free(tmp);
+
+//
+// If we arrive here, this means we have to do a re-try with a synchronous
+// request
+//
+
+						DeviceData dd_out = redo_synch_cmd(req);
+
+//
+// Remove request from request global table.
+//
+
+						ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+						pasyn_ctr--;
+					
+						return dd_out;
+						
+					}
+					catch (Tango::DevFailed &e) {}
+				}
 
 				TangoSys_OMemStream desc;
 				desc << "Failed to execute command_inout_asynch on device " << dev_name();
@@ -479,7 +526,7 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 	else
 	{
 		long nb = call_timeout / 20;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		struct timespec to_wait,inter;
 		to_wait.tv_sec = 0;
 		to_wait.tv_nsec = 20000000;
@@ -488,7 +535,7 @@ DeviceData Connection::command_inout_reply(long id,long call_timeout)
 		
 		for (i = 0;i < nb;i++)
 		{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			Sleep(20);
 #else
 			nanosleep(&to_wait,&inter);
@@ -796,6 +843,24 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 		if (!CORBA::is_nil(env) && (env->exception() != NULL))
 		{
 			read_attr_except(req.request,id,MULTIPLE);
+			
+//
+// If we arrive here, this means the exception was there due to a server
+// shutdown but it is now back into operation
+// Try to redo the call synchronously
+//
+
+			vector<DeviceAttribute> *a_ptr;
+			a_ptr = redo_synch_reads_call(req);
+			
+//
+// Remove request from request global table.
+//
+
+			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+			pasyn_ctr--;	
+		
+			return a_ptr;
 		}
 		
 		vector<DeviceAttribute> *dev_attr = new(vector<DeviceAttribute>);	
@@ -845,7 +910,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id)
 					(*dev_attr)[i].ext->err_list.inout().length(nb_except + 1);
 					(*dev_attr)[i].ext->err_list[nb_except].reason = CORBA::string_dup("API_AttributeFailed");
 					(*dev_attr)[i].ext->err_list[nb_except].origin = CORBA::string_dup("DeviceProxy::read_attribute()");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
 					string st = desc.str();
 					(*dev_attr)[i].ext->err_list[nb_except].desc = CORBA::string_dup(st.c_str());
 #else
@@ -933,7 +998,25 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 		CORBA::Environment_ptr env = req.request->env();
 		if (!CORBA::is_nil(env) && (env->exception() != NULL))
 		{
-			read_attr_except(req.request,id,SIMPLE);	
+			read_attr_except(req.request,id,SIMPLE);
+			
+//
+// If we arrive here, this means the exception was there due to a server
+// shutdown but it is now back into operation
+// Try to redo the call synchronously
+//
+
+			DeviceAttribute *a_ptr;
+			a_ptr = redo_synch_read_call(req);
+			
+//
+// Remove request from request global table.
+//
+
+			ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+			pasyn_ctr--;	
+		
+			return a_ptr;
 		}
 		
 		DeviceAttribute *dev_attr = new DeviceAttribute;	
@@ -957,9 +1040,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 			ApiUtil::attr_to_device(NULL,
 						&((*received_3)[0]),
 						version,dev_attr);				
-	
-
-			
+				
 //
 // Add an error in the error stack in case there is one
 //
@@ -974,7 +1055,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id)
 				dev_attr->ext->err_list.inout().length(nb_except + 1);
 				dev_attr->ext->err_list[nb_except].reason = CORBA::string_dup("API_AttributeFailed");
 				dev_attr->ext->err_list[nb_except].origin = CORBA::string_dup("DeviceProxy::read_attribute_reply()");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
 				string st = desc.str();
 				dev_attr->ext->err_list[nb_except].desc = CORBA::string_dup(st.c_str());
 #else
@@ -1058,7 +1139,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 	else
 	{	
 		long nb = call_timeout / 20;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		struct timespec to_wait,inter;
 		to_wait.tv_sec = 0;
 		to_wait.tv_nsec = 20000000;
@@ -1067,7 +1148,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 		
 		for (i = 0;i < nb;i++)
 		{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			Sleep(20);
 #else
 			nanosleep(&to_wait,&inter);
@@ -1098,6 +1179,24 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 	if (!CORBA::is_nil(env) && (env->exception() != NULL))
 	{
 		read_attr_except(req.request,id,MULTIPLE);
+		
+//
+// If we arrive here, this means the exception was there due to a server
+// shutdown but it is now back into operation
+// Try to redo the call synchronously
+//
+
+		vector<DeviceAttribute> *a_ptr;
+		a_ptr = redo_synch_reads_call(req);
+			
+//
+// Remove request from request global table.
+//
+
+		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+		pasyn_ctr--;	
+		
+		return a_ptr;
 	}
 
 
@@ -1149,7 +1248,7 @@ vector<DeviceAttribute> *DeviceProxy::read_attributes_reply(long id,long call_ti
 				(*dev_attr)[i].ext->err_list.inout().length(nb_except + 1);
 				(*dev_attr)[i].ext->err_list[nb_except].reason = CORBA::string_dup("API_AttributeFailed");
 				(*dev_attr)[i].ext->err_list[nb_except].origin = CORBA::string_dup("DeviceProxy::read_attributes_reply()");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
 				string st = desc.str();
 				(*dev_attr)[i].ext->err_list[nb_except].desc = CORBA::string_dup(st.c_str());
 #else
@@ -1233,7 +1332,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 	else
 	{
 		long nb = call_timeout / 20;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		struct timespec to_wait,inter;
 		to_wait.tv_sec = 0;
 		to_wait.tv_nsec = 20000000;
@@ -1242,7 +1341,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 		
 		for (i = 0;i < nb;i++)
 		{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			Sleep(20);
 #else
 			nanosleep(&to_wait,&inter);
@@ -1273,6 +1372,24 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 	if (!CORBA::is_nil(env) && (env->exception() != NULL))
 	{
 		read_attr_except(req.request,id,SIMPLE);
+		
+//
+// If we arrive here, this means the exception was there due to a server
+// shutdown but it is now back into operation
+// Try to redo the call synchronously
+//
+
+		DeviceAttribute *a_ptr;
+		a_ptr = redo_synch_read_call(req);
+			
+//
+// Remove request from request global table.
+//
+
+		ApiUtil::instance()->get_pasyn_table()->remove_request(id);
+		pasyn_ctr--;	
+		
+		return a_ptr;
 	}
 
 
@@ -1310,7 +1427,7 @@ DeviceAttribute *DeviceProxy::read_attribute_reply(long id,long call_timeout)
 			dev_attr->ext->err_list.inout().length(nb_except + 1);
 			dev_attr->ext->err_list[nb_except].reason = CORBA::string_dup("API_AttributeFailed");
 			dev_attr->ext->err_list[nb_except].origin = CORBA::string_dup("DeviceProxy::read_attribute_reply()");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
 			string st = desc.str();
 			dev_attr->ext->err_list[nb_except].desc = CORBA::string_dup(st.c_str());
 #else
@@ -1456,7 +1573,7 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 		connection_state = CONNECTION_NOTOK;
 			
 //
-// Re-throw all CORBA system exceptions
+// Re-throw nearly (but not all) all CORBA system exceptions
 //
 
 		CORBA::NVList_ptr req_arg = req->arguments();
@@ -1466,6 +1583,24 @@ void DeviceProxy::read_attr_except(CORBA::Request_ptr req,long id,read_attr_type
 
 		char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(sys_ex);
 
+//
+// Check if the exception was a connection exception
+// In this case, try to ping the device.
+// If successfull, just returns otherwise, throw the first exception
+//
+
+		string ex(cb_excep_mess);
+		string::size_type pos = ex.find("TRANSIENT_ConnectFailed");
+		if (pos != string::npos)
+		{
+			try
+			{
+				ping();
+				return;
+			}
+			catch (Tango::DevFailed &e) {}
+		}
+
 		TangoSys_OMemStream desc;
 		desc << "Failed to execute read_attributes_asynch on device " << device_name;
 		desc << "\nAttribute(s): ";
@@ -1584,6 +1719,26 @@ long DeviceProxy::write_attributes_asynch(vector<DeviceAttribute> &attr_list)
 			att[i].value  <<= attr_list[i].UCharSeq.in();
 			continue;
 		}
+		if (attr_list[i].ext->Long64Seq.operator->() != NULL)
+		{	
+			att[i].value  <<= attr_list[i].ext->Long64Seq.in();
+			continue;
+		}
+		if (attr_list[i].ext->ULongSeq.operator->() != NULL)
+		{	
+			att[i].value  <<= attr_list[i].ext->ULongSeq.in();
+			continue;
+		}
+		if (attr_list[i].ext->ULong64Seq.operator->() != NULL)
+		{	
+			att[i].value  <<= attr_list[i].ext->ULong64Seq.in();
+			continue;
+		}
+		if (attr_list[i].ext->StateSeq.operator->() != NULL)
+		{	
+			att[i].value  <<= attr_list[i].ext->StateSeq.in();
+			continue;
+		}
 	}
 		
 	CORBA::Request_ptr request;
@@ -1677,6 +1832,22 @@ long DeviceProxy::write_attribute_asynch(DeviceAttribute &attr)
 	{	
 		att[0].value  <<= attr.UCharSeq.in();
 	}
+	else if (attr.ext->Long64Seq.operator->() != NULL)
+	{	
+		att[0].value  <<= attr.ext->Long64Seq.in();
+	}
+	else if (attr.ext->ULongSeq.operator->() != NULL)
+	{	
+		att[0].value  <<= attr.ext->ULongSeq.in();
+	}
+	else if (attr.ext->ULong64Seq.operator->() != NULL)
+	{	
+		att[0].value  <<= attr.ext->ULong64Seq.in();
+	}
+	else if (attr.ext->StateSeq.operator->() != NULL)
+	{	
+		att[0].value  <<= attr.ext->StateSeq.in();
+	}
 
 	CORBA::Request_ptr request;
 	if (version < 3)
@@ -1757,7 +1928,7 @@ void DeviceProxy::write_attributes_reply(long id,long call_timeout)
 	else
 	{
 		long nb = call_timeout / 20;
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		struct timespec to_wait,inter;
 		to_wait.tv_sec = 0;
 		to_wait.tv_nsec = 20000000;
@@ -1766,7 +1937,7 @@ void DeviceProxy::write_attributes_reply(long id,long call_timeout)
 		
 		for (i = 0;i < nb;i++)
 		{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			Sleep(20);
 #else
 			nanosleep(&to_wait,&inter);
@@ -1797,6 +1968,15 @@ void DeviceProxy::write_attributes_reply(long id,long call_timeout)
 	if (!CORBA::is_nil(env) && (env->exception() != NULL))
 	{
 		write_attr_except(req.request,id,req.req_type);
+		
+			
+//
+// If we arrive here, this means the exception was there due to a server
+// shutdown but it is now back into operation
+// Try to redo the call synchronously
+//
+
+		redo_synch_write_call(req);
 	}
 				
 //
@@ -1865,6 +2045,14 @@ void DeviceProxy::write_attributes_reply(long id)
 		if (!CORBA::is_nil(env) && (env->exception() != NULL))
 		{
 			write_attr_except(req.request,id,req.req_type);
+			
+//
+// If we arrive here, this means the exception was there due to a server
+// shutdown but it is now back into operation
+// Try to redo the call synchronously
+//
+
+			redo_synch_write_call(req);
 		}
 	}
 				
@@ -2013,7 +2201,7 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		connection_state = CONNECTION_NOTOK;
 			
 //
-// Re-throw all CORBA system exceptions
+// Re-throw nearly all CORBA system exceptions
 //
 
 		CORBA::NVList_ptr req_arg = req->arguments();
@@ -2022,6 +2210,24 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 		*(nv->value()) >>= att;
 
 		char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(sys_ex);
+		
+//
+// Check if the exception was a connection exception
+// In this case, try to ping the device.
+// If successfull, just returns otherwise, throw the first exception
+//
+
+		string ex(cb_excep_mess);
+		string::size_type pos = ex.find("TRANSIENT_ConnectFailed");
+		if (pos != string::npos)
+		{
+			try
+			{
+				ping();
+				return;
+			}
+			catch (Tango::DevFailed &e) {}
+		}
 
 		TangoSys_OMemStream desc;
 		desc << "Failed to execute write_attributes_asynch on device " << device_name;
@@ -2044,4 +2250,219 @@ void DeviceProxy::write_attr_except(CORBA::Request_ptr req,long id,TgRequest::Re
 	}	
 }
 
+//-----------------------------------------------------------------------------
+//
+// method : 		DeviceProxy::retreve_read_args()
+// 
+// description : 	Retrieve the read_attribute asynchrouns call from the
+//					Tango request object which itself contains the CORBA
+//					request object
+//
+// argin(s) :		req : The Tango request object
+//					att_list : reference to a string vector whihc will be filled
+//							   in with attribute(s) name
+//
+//-----------------------------------------------------------------------------
+
+void DeviceProxy::retrieve_read_args(TgRequest &req,vector<string> &att_list)
+{			
+
+	att_list.clear();
+	
+//
+// Retrieve which attribute was read
+//
+
+	const Tango::DevVarStringArray *att_names = NULL;
+	try
+	{
+		CORBA::NVList_ptr args_ptr = req.request->arguments();
+		CORBA::NamedValue_ptr arg_ptr = args_ptr->item(0);
+		CORBA::Any *arg_val = arg_ptr->value();
+		(*arg_val) >>= att_names;
+		
+		for (unsigned loop = 0;loop < att_names->length();loop++)
+			att_list.push_back(string((*att_names)[loop]));
+	}
+	catch (CORBA::SystemException &e)
+	{
+		char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(&e);
+
+		TangoSys_OMemStream desc;
+		desc << "Failed to redo the call synchronously on device " << device_name;
+		if (att_names != NULL)
+		{
+			desc << "\nAttribute(s): ";
+			for (unsigned int i = 0;i < att_names->length();i++)
+			{
+				desc << (*att_names)[i];
+				if (i != (att_names->length() - 1))
+					desc << ", ";
+			}
+		}
+		desc << ends;
+					
+		ApiCommExcept::re_throw_exception(cb_excep_mess,
+							  			  (const char*)"API_CommunicationFailed",
+                        			  	  desc.str(),
+							  			  (const char*)"DeviceProxy::redo_simpl_call()");
+	}
+}
+
+
+//-----------------------------------------------------------------------------
+//
+// method : 		DeviceProxy::redo_synch_read_call()
+// 
+// description : 	Redo a synchronous read_attribute() call
+//					This is needed for re-connection with asynchronous call
+//
+// argin(s) :		req : The Tango request object
+//
+//-----------------------------------------------------------------------------
+
+DeviceAttribute *DeviceProxy::redo_synch_read_call(TgRequest &req)
+{			
+
+//
+// Retrieve which attribute was read
+//
+
+	vector<string> att_list;
+	retrieve_read_args(req,att_list);
+
+//
+// Redo the read_attribute but synchronously
+//
+		
+	DeviceAttribute attrib = read_attribute(att_list[0]);
+	DeviceAttribute *attr_ptr = new DeviceAttribute(attrib);
+	return attr_ptr;
+}
+
+//-----------------------------------------------------------------------------
+//
+// method : 		DeviceProxy::redo_synch_reads_call()
+// 
+// description : 	Redo a read-attributes() call but synchronously
+//					This is needed for re-connection with asynchronous call
+//
+// argin(s) :		req : The Tango request object
+//
+//-----------------------------------------------------------------------------
+
+vector<DeviceAttribute> *DeviceProxy::redo_synch_reads_call(TgRequest &req)
+{			
+
+//
+// Retrieve which attributes was read
+//
+
+	vector<string> att_list;
+	retrieve_read_args(req,att_list);		
+
+//
+// Redo the read_attributes but synchronously
+//
+		
+	return read_attributes(att_list);
+}
+
+//-----------------------------------------------------------------------------
+//
+// method : 		DeviceProxy::redo_synch_write_call()
+// 
+// description : 	Redo a read-attributes() call but synchronously
+//					This is needed for re-connection with asynchronous call
+//
+// argin(s) :		req : The Tango request object
+//
+//-----------------------------------------------------------------------------
+
+void DeviceProxy::redo_synch_write_call(TgRequest &req)
+{			
+
+//
+// Retrieve which attributes was written
+//
+
+	const Tango::AttributeValueList *att;
+	try
+	{
+		CORBA::NVList_ptr args_ptr = req.request->arguments();
+		CORBA::NamedValue_ptr arg_ptr = args_ptr->item(0);
+		CORBA::Any *arg_val = arg_ptr->value();
+		(*arg_val) >>= att;
+	}
+	catch (CORBA::SystemException &e)
+	{
+		char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(&e);
+
+		TangoSys_OMemStream desc;
+		desc << "Failed to redo the call synchronously on device " << device_name << ends;
+					
+		ApiCommExcept::re_throw_exception(cb_excep_mess,
+							  			  (const char*)"API_CommunicationFailed",
+                        			  	  desc.str(),
+							  			  (const char*)"DeviceProxy::redo_synch_write_call()");
+	}
+
+//
+// Redo the write_attributes but synchronously
+//
+		
+	return write_attribute(*att);
+}
+
+//-----------------------------------------------------------------------------
+//
+// method : 		DeviceProxy::redo_synch_cmd()
+// 
+// description : 	Redo a command_inout() call but synchronously
+//					This is needed for re-connection with asynchronous call
+//
+// argin(s) :		req : The Tango request object
+//
+//-----------------------------------------------------------------------------
+
+DeviceData Connection::redo_synch_cmd(TgRequest &req)
+{
+	const char *cmd_name;
+	const CORBA::Any *a_ptr;
+
+	try
+	{
+		CORBA::NVList_ptr args_ptr = req.request->arguments();
+		CORBA::NamedValue_ptr arg_ptr = args_ptr->item(0);
+		
+		*(arg_ptr->value()) >>= cmd_name;
+					
+		arg_ptr = args_ptr->item(1);
+		*(arg_ptr->value()) >>= a_ptr;
+	}
+	catch (CORBA::SystemException &e)
+	{
+		char *cb_excep_mess = Tango::Except::print_CORBA_SystemException(&e);
+
+		TangoSys_OMemStream desc;
+		desc << "Failed to redo the call synchronously on device " << dev_name() << ends;
+					
+		ApiCommExcept::re_throw_exception(cb_excep_mess,
+							  			  (const char*)"API_CommunicationFailed",
+                        			  	  desc.str(),
+							  			  (const char*)"DeviceProxy::redo_synch_write_call()");
+	}
+	
+//
+// Redo the call synchronously
+//
+
+	DeviceData dd;
+	CORBA::Any *tmp_any_ptr = new CORBA::Any(*a_ptr);
+	dd.any = tmp_any_ptr;
+	
+	return command_inout(cmd_name,dd);
+}
+
+		
 } // End of tango namespace
