@@ -11,9 +11,44 @@
 //
 // author(s) :		A.Gotz + E.Taurel
 //
-// $Revision: 3.24 $
+// $Revision: 3.34 $
 //
 // $Log: attribute.h,v $
+// Revision 3.34  2007/05/04 13:02:18  jensmeyer
+// Added the removal of the attribute polling and event configuration when
+// executing the remove_attribute() method.
+//
+// Revision 3.33  2007/04/20 14:40:24  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.32  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.31  2007/03/28 07:53:31  taurel
+// - Change some data member data type for 64 bits portability
+//
+// Revision 3.30  2007/03/06 08:18:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.29  2007/03/02 09:40:51  jensmeyer
+// Added quality change to archive event and added the field except to the
+// last_attr_value to store the last exception.
+//
+// Revision 3.28  2007/02/16 10:40:09  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
+// Revision 3.27  2007/02/08 16:15:35  taurel
+// - Add a set_attr_idx() method
+//
+// Revision 3.26  2006/12/28 15:39:11  taurel
+// - Added WAttribute::set_min_value() and WAttribute::set_max_value() method families
+//
+// Revision 3.25  2006/11/09 16:25:49  taurel
+// - Remove the fire_quality_event() method
+//
 // Revision 3.24  2006/06/29 11:51:12  jensmeyer
 // Moved init_poll_ext_trig() and init_poll_period() from Attribute to
 // DeviceImpl.
@@ -148,7 +183,7 @@
 #include <functional>
 #include <time.h>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#include <sys/types.h>
 	#include <sys/timeb.h>
 #endif
@@ -192,12 +227,16 @@ class AttrProperty;
 
 typedef union _Attr_CheckVal
 {
-	short		sh;
-	long		lg;
-	double		db;
-	float 		fl;
+	short			sh;
+	DevLong			lg;
+	double			db;
+	float 			fl;
 	unsigned short	ush;
 	unsigned char	uch;
+	DevLong64		lg64;
+	DevULong		ulg;
+	DevULong64		ulg64;
+	DevState		d_sta;
 }Attr_CheckVal;
 
 typedef union _Attr_Value
@@ -210,14 +249,19 @@ typedef union _Attr_Value
 	DevVarUShortArray 	*ush_seq;
 	DevVarBooleanArray	*boo_seq;
 	DevVarCharArray		*cha_seq;
+	DevVarLong64Array	*lg64_seq;
+	DevVarULongArray	*ulg_seq;
+	DevVarULong64Array	*ulg64_seq;
+	DevVarStateArray	*state_seq;
 }Attr_Value;
 
 
 typedef struct last_attr_value {
-	bool 			inited;
+	bool 				inited;
 	Tango::AttrQuality 	quality;
-	CORBA::Any 		value;
-	bool 			err;
+	CORBA::Any 			value;
+	bool 				err;
+	DevFailed 			except;
 } LastAttrValue;
 
 class EventSupplier;
@@ -238,40 +282,46 @@ class AttributeExt
 public:
 	AttributeExt() {}
 	
- 	Tango::DispLevel 	disp_level;			// Display level
- 	long			poll_period;			// Polling period
-	double			rel_change[2];			// Delta for relative change events in %
-	double			abs_change[2];			// Delta for absolute change events
-	double			archive_rel_change[2];		// Delta for relative archive change events in %
-	double			archive_abs_change[2];		// Delta for absolute change events
-	int			event_period;			// Delta for periodic events in ms
-	int			archive_period;			// Delta for archive periodic events in ms
-	double			last_periodic;			// Last time a periodic event was detected
-	double			archive_last_periodic;		// Last time an archive periodic event was detected
-	long			periodic_counter;		// Number of periodic events sent so far
-	long			archive_periodic_counter;	// Number of periodic events sent so far
-	LastAttrValue		prev_change_event;		// Last change attribute
-	LastAttrValue		prev_quality_event;		// Last quality attribute
-	LastAttrValue		prev_archive_event;		// Last archive attribute
-	int			event_change_subscription;	// Last time() a subscription was made
-	int			event_quality_subscription;	// Last time() a subscription was made
-	int			event_periodic_subscription;	// Last time() a subscription was made
-	int			event_archive_subscription; 	// Last time() a subscription was made
-	int			event_user_subscription; 	// Last time() a subscription was made
-	long			idx_in_attr;			// Index in MultiClassAttribute vector
-	string			d_name;				// The device name
-	DeviceImpl 		*dev;				// The device object
-	bool			change_event_implmented;	// Flag true if a manual fire change event is implemented.
-	bool			archive_event_implmented;	// Flag true if a manual fire archive event is implemented.	
-	bool			check_change_event_criteria;	// True if change event criteria should be checked when sending the event
-	bool			check_archive_event_criteria;	// True if change event criteria should be checked when sending the event		
+ 	Tango::DispLevel 	disp_level;						// Display level
+ 	long				poll_period;					// Polling period
+	double				rel_change[2];					// Delta for relative change events in %
+	double				abs_change[2];					// Delta for absolute change events
+	double				archive_rel_change[2];			// Delta for relative archive change events in %
+	double				archive_abs_change[2];			// Delta for absolute change events
+	int					event_period;					// Delta for periodic events in ms
+	int					archive_period;					// Delta for archive periodic events in ms
+	double				last_periodic;					// Last time a periodic event was detected
+	double				archive_last_periodic;			// Last time an archive periodic event was detected
+	long				periodic_counter;				// Number of periodic events sent so far
+	long				archive_periodic_counter;		// Number of periodic events sent so far
+	LastAttrValue		prev_change_event;				// Last change attribute
+	LastAttrValue		prev_quality_event;				// Last quality attribute
+	LastAttrValue		prev_archive_event;				// Last archive attribute
+	time_t				event_change_subscription;		// Last time() a subscription was made
+	time_t				event_quality_subscription;		// Last time() a subscription was made
+	time_t				event_periodic_subscription;	// Last time() a subscription was made
+	time_t				event_archive_subscription; 	// Last time() a subscription was made
+	time_t				event_user_subscription; 		// Last time() a subscription was made
+	long				idx_in_attr;					// Index in MultiClassAttribute vector
+	string				d_name;							// The device name
+	DeviceImpl 			*dev;							// The device object
+	bool				change_event_implmented;		// Flag true if a manual fire change event is implemented.
+	bool				archive_event_implmented;		// Flag true if a manual fire archive event is implemented.	
+	bool				check_change_event_criteria;	// True if change event criteria should be checked when sending the event
+	bool				check_archive_event_criteria;	// True if change event criteria should be checked when sending the event		
+	time_t				event_attr_conf_subscription;	// Last time() a subsription was made
+	Tango::DevLong64	tmp_lo64[2];
+	Tango::DevULong		tmp_ulo[2];
+	Tango::DevULong64	tmp_ulo64[2];
+	Tango::DevState		tmp_state[2];
+	double				archive_last_event;				// Last time an archive event was detected (periodic or not)
 	};
 
 /**
  * This class represents a Tango attribute.
  *
  * $Author: jensmeyer $
- * $Revision: 3.24 $
+ * $Revision: 3.34 $
  */
  
 class Attribute
@@ -449,7 +499,7 @@ public:
  * @param The attribute date
  */
 	void set_date(Tango::TimeVal &new_date) {when = new_date;}
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 /**
  * Set attribute date
  *
@@ -597,6 +647,25 @@ public:
  * <b>DevFailed</b> exception specification
  */
 	void set_value(Tango::DevLong *p_data,long x = 1, long y = 0,bool release = false);
+	
+/**
+ * Set internal attribute value (for Tango::DevLong64 attribute data type).
+ *
+ * This method stores the attribute read value inside the object. This data will be
+ * returned to the caller. This method also stores the date when it is called
+ * and initialise the attribute quality factor.
+ *
+ * @param p_data The attribute read value
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */	
+	void set_value(Tango::DevLong64 *p_data,long x = 1,long y = 0,bool release = false);
+	
 /**
  * Set internal attribute value (for Tango::DevFloat attribute data type).
  *
@@ -700,6 +769,57 @@ public:
  * <b>DevFailed</b> exception specification
  */
 	void set_value(Tango::DevUChar *p_data,long x = 1,long y = 0,bool release = false);
+/**
+ * Set internal attribute value (for Tango::DevULong attribute data type).
+ *
+ * This method stores the attribute read value inside the object. This data will be
+ * returned to the caller. This method also stores the date when it is called
+ * and initialise the attribute quality factor.
+ *
+ * @param p_data The attribute read value
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value(Tango::DevULong *p_data,long x = 1,long y = 0,bool release = false);
+/**
+ * Set internal attribute value (for Tango::DevULong64 attribute data type).
+ *
+ * This method stores the attribute read value inside the object. This data will be
+ * returned to the caller. This method also stores the date when it is called
+ * and initialise the attribute quality factor.
+ *
+ * @param p_data The attribute read value
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value(Tango::DevULong64 *p_data,long x = 1,long y = 0,bool release = false);
+/**
+ * Set internal attribute value (for Tango::DevState attribute data type).
+ *
+ * This method stores the attribute read value inside the object. This data will be
+ * returned to the caller. This method also stores the date when it is called
+ * and initialise the attribute quality factor.
+ *
+ * @param p_data The attribute read value
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value(Tango::DevState *p_data,long x = 1,long y = 0,bool release = false);
 
 //---------------------------------------------------------------------------
 
@@ -728,7 +848,7 @@ public:
 				    long x = 1,long y = 0,
 				    bool release = false);
 
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevShort *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -788,7 +908,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevLong *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -825,6 +945,66 @@ public:
 
 //-----------------------------------------------------------------------
 
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevLong64 attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */	
+	void set_value_date_quality(Tango::DevLong64 *p_data,
+				    time_t t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#ifdef _TG_WINDOWS_				    
+	void set_value_date_quality(Tango::DevLong64 *p_data,
+				    struct _timeb &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#else
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevLong64 attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * Please note that for Win32 user, the same method is defined using a 
+ * "_timeb" structure instead of a "timeval" structure to set date. 
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value_date_quality(Tango::DevLong64 *p_data,
+				    struct timeval &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#endif
+
+//-----------------------------------------------------------------------
+
 /**
  * Set internal attribute value, date and quality factor (for Tango::DevFloat attribute data type).
  *
@@ -849,7 +1029,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevFloat *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -911,7 +1091,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevDouble *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -971,7 +1151,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevString *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -1032,7 +1212,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevBoolean *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -1093,7 +1273,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevUShort *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -1154,7 +1334,7 @@ public:
 				    Tango::AttrQuality qual,
 				    long x = 1,long y = 0,
 				    bool release = false);
-#ifdef WIN32				    
+#ifdef _TG_WINDOWS_				    
 	void set_value_date_quality(Tango::DevUChar *p_data,
 				    struct _timeb &t,
 				    Tango::AttrQuality qual,
@@ -1189,6 +1369,186 @@ public:
 				    bool release = false);
 #endif
 
+//-----------------------------------------------------------------------
+
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevULong attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */	
+	void set_value_date_quality(Tango::DevULong *p_data,
+				    time_t t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#ifdef _TG_WINDOWS_				    
+	void set_value_date_quality(Tango::DevULong *p_data,
+				    struct _timeb &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#else
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevULong attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * Please note that for Win32 user, the same method is defined using a 
+ * "_timeb" structure instead of a "timeval" structure to set date. 
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value_date_quality(Tango::DevULong *p_data,
+				    struct timeval &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#endif
+
+//-----------------------------------------------------------------------
+
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevULong64 attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */	
+	void set_value_date_quality(Tango::DevULong64 *p_data,
+				    time_t t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#ifdef _TG_WINDOWS_				    
+	void set_value_date_quality(Tango::DevULong64 *p_data,
+				    struct _timeb &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#else
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevULong64 attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * Please note that for Win32 user, the same method is defined using a 
+ * "_timeb" structure instead of a "timeval" structure to set date. 
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value_date_quality(Tango::DevULong64 *p_data,
+				    struct timeval &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#endif
+
+//-----------------------------------------------------------------------
+
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevState attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */	
+	void set_value_date_quality(Tango::DevState *p_data,
+				    time_t t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#ifdef _TG_WINDOWS_				    
+	void set_value_date_quality(Tango::DevState *p_data,
+				    struct _timeb &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#else
+/**
+ * Set internal attribute value, date and quality factor (for Tango::DevState attribute data type).
+ *
+ * This method stores the attribute read value, the date and the attribute
+ * quality factor inside the object. This data will be
+ * returned to the caller.
+ *
+ * Please note that for Win32 user, the same method is defined using a 
+ * "_timeb" structure instead of a "timeval" structure to set date. 
+ *
+ * @param p_data The attribute read value
+ * @param t The date
+ * @param qual The attribute quality factor
+ * @param x The attribute x length. Default value is 1
+ * @param y The attribute y length. Default value is 0
+ * @param release The release flag. If true, memory pointed to by p_data will be 
+ * 		  freed after being send to the client. Default value is false.
+ * @exception DevFailed If the attribute data type is not coherent.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void set_value_date_quality(Tango::DevState *p_data,
+				    struct timeval &t,
+				    Tango::AttrQuality qual,
+				    long x = 1,long y = 0,
+				    bool release = false);
+#endif
+
 /**
  * Fire a change event for the attribute value. The event is pushed to the notification
  * daemon.
@@ -1328,6 +1688,20 @@ public:
  * @param except A pointer to a DevFailed exception to be thrown as archive event.
  */	
 	void fire_event(vector<string> &filt_names,vector<double> &filt_vals,DevFailed *except = NULL);		
+
+/**
+ * Remove the attribute configuration from the database.
+ * This method can be used to clean-up all the configuration of an attribute to come back to
+ * its default values or the remove all configuration of a dynamic attribute before deleting it.
+ *
+ * The method removes all configured attribute properties and removes the attribute from the 
+ * list of polled attributes.
+ *
+ * @exception DevFailed In case of database access problems.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ */
+	void remove_configuration();
 //@}
 
 //
@@ -1348,6 +1722,10 @@ public:
 	Tango::DevVarBooleanArray *get_boolean_value() {return value.boo_seq;}
 	Tango::DevVarUShortArray *get_ushort_value() {return value.ush_seq;}
 	Tango::DevVarCharArray *get_uchar_value() {return value.cha_seq;}
+	Tango::DevVarLong64Array *get_long64_value() {return value.lg64_seq;}
+	Tango::DevVarULongArray *get_ulong_value() {return value.ulg_seq;}
+	Tango::DevVarULong64Array *get_ulong64_value() {return value.ulg64_seq;}
+	Tango::DevVarStateArray *get_state_value() {return value.state_seq;}
 	
 	void add_write_value(Tango::DevVarShortArray *);
 	void add_write_value(Tango::DevVarLongArray *);
@@ -1357,11 +1735,16 @@ public:
 	void add_write_value(Tango::DevVarBooleanArray *);
 	void add_write_value(Tango::DevVarUShortArray *);
 	void add_write_value(Tango::DevVarCharArray *);
+	void add_write_value(Tango::DevVarLong64Array *);
+	void add_write_value(Tango::DevVarULongArray *);
+	void add_write_value(Tango::DevVarULong64Array *);
+	void add_write_value(Tango::DevVarStateArray *);
 	
 	unsigned long get_name_size() {return name_size;}
 	string &get_name_lower() {return name_lower;}
 	void set_value_flag(bool val) {value_flag = val;}
 	bool get_value_flag() {return value_flag;}
+	DispLevel get_disp_level() {return ext->disp_level;}
 	
 	void set_properties(const Tango::AttributeConfig &,string &);
 	void set_properties(const Tango::AttributeConfig_3 &,string &);
@@ -1374,13 +1757,15 @@ public:
 	bool quality_event_subscribed() {if (ext->event_quality_subscription != 0)return true;else return false;}
 	bool user_event_subscribed() {if (ext->event_user_subscription != 0)return true;else return false;}
 
-	void set_change_event_sub() {ext->event_change_subscription=(long)time(NULL);}
-	void set_periodic_event_sub() {ext->event_periodic_subscription=(long)time(NULL);}
-	void set_archive_event_sub() {ext->event_archive_subscription=(long)time(NULL);}
-	void set_quality_event_sub() {ext->event_quality_subscription=(long)time(NULL);}
-	void set_user_event_sub() {ext->event_user_subscription=(long)time(NULL);}		
+	void set_change_event_sub() {ext->event_change_subscription=time(NULL);}
+	void set_periodic_event_sub() {ext->event_periodic_subscription=time(NULL);}
+	void set_archive_event_sub() {ext->event_archive_subscription=time(NULL);}
+	void set_quality_event_sub() {ext->event_quality_subscription=time(NULL);}
+	void set_user_event_sub() {ext->event_user_subscription=time(NULL);}		
 
 	long get_attr_idx() {return ext->idx_in_attr;}
+	void set_attr_idx(long new_idx) {ext->idx_in_attr=new_idx;}
+	DeviceImpl *get_att_device();
 
 	void Attribute_2_AttributeValue(Tango::AttributeValue_3 *,DeviceImpl *);		
 
@@ -1396,7 +1781,7 @@ private:
 	void check_str_prop(const AttributeConfig &,DbData &,long &,DbData &,long &);	
 
 	unsigned long 		name_size;
-	string 			name_lower;
+	string 				name_lower;
 		
 	AttributeExt		*ext;
 			
@@ -1408,7 +1793,7 @@ protected:
 	virtual bool check_rds_alarm() {return false;}
 	bool check_level_alarm();
 	bool check_warn_alarm();
-	void fire_quality_event();
+	void upd_att_prop_db(Tango::Attr_CheckVal &,const char *);
 
 /**@name Class data members */
 //@{
@@ -1586,8 +1971,8 @@ protected:
 	bitset<numFlags>	alarm_conf;
 	bitset<numFlags>	alarm;	
 
-	long			dim_x;
-	long			dim_y;
+	long				dim_x;
+	long				dim_y;
 	
 	Tango::DevShort		tmp_sh[2];
 	Tango::DevLong		tmp_lo[2];
@@ -1596,7 +1981,8 @@ protected:
 	Tango::DevString	tmp_str[2];
 	Tango::DevUShort	tmp_ush[2];
 	Tango::DevBoolean	tmp_boo[2];
-	Tango::DevUChar		tmp_cha[2];		
+	Tango::DevUChar		tmp_cha[2];
+	
 	vector<AttrProperty>::iterator pos_end;	
 };
 
@@ -1604,7 +1990,7 @@ protected:
 // Macro to help coding
 //
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 #define MEM_STREAM_2_CORBA(A,B) NEW_MEM_STREAM_2_CORBA(A,B)
 #else
 #define MEM_STREAM_2_CORBA(A,B) OLD_MEM_STREAM_2_CORBA(A,B)
@@ -1639,14 +2025,18 @@ protected:
 	    (strcmp(A,NotANumber) != 0)) \
 	{ \
 		if ((data_type != Tango::DEV_STRING) && \
-		    (data_type != Tango::DEV_BOOLEAN)) \
+		    (data_type != Tango::DEV_BOOLEAN) && \
+		    (data_type != Tango::DEV_STATE)) \
 		{ \
 			short sh; \
-			long lg; \
+			DevLong lg; \
 			double db; \
 			float fl; \
 			unsigned short ush; \
 			unsigned char uch; \
+			DevLong64 lg64; \
+			DevULong ulg; \
+			DevULong64 ulg64; \
 \
 			B.seekp(0); \
 			B.seekg(0); \
@@ -1663,6 +2053,11 @@ protected:
 				if (!(B >> lg)) \
 					throw_err_format(H,C); \
 				break;\
+\
+			case Tango::DEV_LONG64: \
+				if (!(B >> lg64)) \
+					throw_err_format(H,C); \
+				break;\
 \
 			case Tango::DEV_DOUBLE: \
 				if (!(B >> db)) \
@@ -1683,6 +2078,16 @@ protected:
 				if (!(B >> uch)) \
 					throw_err_format(H,C); \
 				break; \
+\
+			case Tango::DEV_ULONG: \
+				if (!(B >> ulg)) \
+					throw_err_format(H,C); \
+				break; \
+\
+			case Tango::DEV_ULONG64: \
+				if (!(B >> ulg64)) \
+					throw_err_format(H,C); \
+				break; \
 			} \
 		} \
 		DbDatum max_val(H); \
