@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: blackbox.cpp,v 3.10 2006/05/18 08:52:37 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: blackbox.cpp,v 3.11 2007/04/20 14:40:24 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -16,9 +16,12 @@ static const char *RcsId = "$Id: blackbox.cpp,v 3.10 2006/05/18 08:52:37 taurel
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.10 $
+// $Revision: 3.11 $
 //
 // $Log: blackbox.cpp,v $
+// Revision 3.11  2007/04/20 14:40:24  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
 // Revision 3.10  2006/05/18 08:52:37  taurel
 // - Miscellaneous changes due to Python device server ported to Windows
 // - Fix some bugs discovered by Windows VC8 using the test suite
@@ -239,7 +242,7 @@ static const char *RcsId = "$Id: blackbox.cpp,v 3.10 2006/05/18 08:52:37 taurel
 
 #include <stdio.h>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 #include <sys/types.h>
 #include <sys/timeb.h>
 #else
@@ -247,7 +250,7 @@ static const char *RcsId = "$Id: blackbox.cpp,v 3.10 2006/05/18 08:52:37 taurel
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 #include <omniORB4/omniInterceptors.h>
 #include <omniORB4/internal/giopStrand.h>
@@ -356,7 +359,7 @@ void BlackBox::insert_corba_attr(BlackBoxElt_AttrType attr)
 	box[insert_elt].attr_type = attr;
 	box[insert_elt].op_type = Op_Unknown;
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //
 // Note that the exact conversion between milli-sec and u-sec will be done
 // only when data is send back to user. This save some times in unnecessary
@@ -424,7 +427,7 @@ void BlackBox::insert_cmd(const char *cmd,long vers,DevSource sour)
 		box[insert_elt].op_type = Op_Command_inout_2;
 	box[insert_elt].cmd_name = cmd;
 	box[insert_elt].source = sour;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //
 // Note that the exact conversion between milli-sec and u-sec will be done
 // only when data is send back to user. This save some times in unnecessary
@@ -488,7 +491,7 @@ void BlackBox::insert_op(BlackBoxElt_OpType op)
 	box[insert_elt].req_type = Req_Operation;
 	box[insert_elt].attr_type = Attr_Unknown;
 	box[insert_elt].op_type = op;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //
 // Note that the exact conversion between milli-sec and u-sec will be done
 // only when data is send back to user. This save some times in unnecessary
@@ -575,7 +578,7 @@ void BlackBox::insert_attr(const Tango::DevVarStringArray &names,long vers,DevSo
 		box[insert_elt].attr_names.push_back(tmp_str);
 	}
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //
 // Note that the exact conversion between milli-sec and u-sec will be done
 // only when data is send back to user. This save some times in unnecessary
@@ -637,7 +640,7 @@ void BlackBox::insert_attr(const Tango::AttributeValueList &att_list, long vers)
 		box[insert_elt].attr_names.push_back(tmp_str);
 	}
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //
 // Note that the exact conversion between milli-sec and u-sec will be done
 // only when data is send back to user. This save some times in unnecessary
@@ -1064,7 +1067,7 @@ void BlackBox::build_info_as_str(long index)
 
         	struct hostent *ho;
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 		struct in_addr ad;
 		ad.s_addr = (ip1 << 24) + (ip2 << 16) + (ip3 << 8) + ip4;
 		unsigned long ip_net = htonl(ad.s_addr);
@@ -1077,7 +1080,7 @@ void BlackBox::build_info_as_str(long index)
 #else
 		ho = gethostbyaddr((char *)(&ip),sizeof(ip),AF_INET);
 #endif /* __linux */
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 		
 		if (ho != NULL)
 		{
@@ -1299,7 +1302,7 @@ void BlackBox::date_ux_to_str(timeval &ux_date,char *str_date)
 	
 /* Add milliseconds */
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	sprintf(&(str_date[19]),":%.2d",(int)(ux_date.tv_usec/10));
 #else
 	sprintf(&(str_date[19]),":%.2d",(int)(ux_date.tv_usec/10000));
