@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dev_event.cpp,v 3.2 2006/07/04 13:12:17 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dev_event.cpp,v 3.5 2007/04/20 14:40:28 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -11,9 +11,21 @@ static const char *RcsId = "$Id: dev_event.cpp,v 3.2 2006/07/04 13:12:17 jensmey
 //
 // author(s) :		J.Meyer + E.Taurel
 //
-// $Revision: 3.2 $
+// $Revision: 3.5 $
 //
 // $Log: dev_event.cpp,v $
+// Revision 3.5  2007/04/20 14:40:28  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.4  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.3  2007/03/06 08:18:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.2  2006/07/04 13:12:17  jensmeyer
 // Corrected time include files for Windows.
 //
@@ -35,7 +47,7 @@ static const char *RcsId = "$Id: dev_event.cpp,v 3.2 2006/07/04 13:12:17 jensmey
 
 #include <tango.h>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 #include <sys/timeb.h>
 #else
 #include <sys/time.h>
@@ -128,6 +140,22 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevLong64 *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
+
 
 void DeviceImpl::push_event(string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			    Tango::DevFloat *p_data, long x, long y , bool release)
@@ -235,6 +263,53 @@ void DeviceImpl::push_event(string attr_name,vector<string> &filt_names,vector<d
 	attr.fire_event(filt_names,filt_vals);
 }
 
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevULong *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
+
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevULong64 *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
+
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevState *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
 
 //+-------------------------------------------------------------------------
 //
@@ -254,7 +329,7 @@ void DeviceImpl::push_event(string attr_name,vector<string> &filt_names,vector<d
 //		 	release : The release flag. If true, memory pointed to by p_data will be 
 // 				  freed after being send to the client. Default value is false.
 //--------------------------------------------------------------------------
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevShort *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -277,7 +352,7 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevLong *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -300,7 +375,30 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevLong64 *p_data,struct _timeb &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#else
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevLong64 *p_data,struct timeval &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
+
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevFloat *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -323,7 +421,7 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevDouble *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -346,7 +444,7 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevString *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -369,7 +467,7 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevBoolean *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -392,7 +490,7 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevUShort *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -415,7 +513,7 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
 			     Tango::DevUChar *p_data,struct _timeb &t, Tango::AttrQuality qual,
 			     long x,long y ,bool release)
@@ -438,6 +536,74 @@ void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<
 	attr.fire_event(filt_names,filt_vals);
 }
 
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevULong *p_data,struct _timeb &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#else
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevULong *p_data,struct timeval &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevULong64 *p_data,struct _timeb &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#else
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevULong64 *p_data,struct timeval &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevState *p_data,struct _timeb &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#else
+void DeviceImpl::push_event (string attr_name,vector<string> &filt_names,vector<double> &filt_vals,
+			     Tango::DevState *p_data,struct timeval &t, Tango::AttrQuality qual,
+			     long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_event(filt_names,filt_vals);
+}
 
 //////////////////// Push change event methods!!! //////////////////////////////////////
 
@@ -538,6 +704,20 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevLong *p_data, lo
 	attr.fire_change_event();
 }
 
+void DeviceImpl::push_change_event (string attr_name, Tango::DevLong64 *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
 
 void DeviceImpl::push_change_event(string attr_name, Tango::DevFloat *p_data, long x, long y , bool release)
 {
@@ -639,6 +819,50 @@ void DeviceImpl::push_change_event(string attr_name, Tango::DevUChar *p_data, lo
 	attr.fire_change_event();
 }
 
+void DeviceImpl::push_change_event (string attr_name, Tango::DevULong *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
+
+void DeviceImpl::push_change_event (string attr_name, Tango::DevULong64 *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
+
+void DeviceImpl::push_change_event (string attr_name, Tango::DevState *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
 
 //+-------------------------------------------------------------------------
 //
@@ -656,7 +880,7 @@ void DeviceImpl::push_change_event(string attr_name, Tango::DevUChar *p_data, lo
 //					   release   : The release flag. If true, memory pointed to by p_data will be 
 // 									freed after being send to the client. Default value is false.
 //--------------------------------------------------------------------------
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevShort *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -679,7 +903,7 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevShort *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevLong *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -702,7 +926,31 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevLong *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_change_event (string attr_name, Tango::DevLong64 *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_change_event (string attr_name, Tango::DevLong64 *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
+
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevFloat *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -725,7 +973,7 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevFloat *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevDouble *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -748,7 +996,7 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevDouble *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevString *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -771,7 +1019,7 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevString *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevBoolean *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -794,7 +1042,7 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevBoolean *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevUShort *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -817,7 +1065,7 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevUShort *p_data,
 	attr.fire_change_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_change_event (string attr_name, Tango::DevUChar *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -840,6 +1088,76 @@ void DeviceImpl::push_change_event (string attr_name, Tango::DevUChar *p_data,
 	attr.fire_change_event();
 }
 
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_change_event (string attr_name, Tango::DevULong *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_change_event (string attr_name, Tango::DevULong *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_change_event (string attr_name, Tango::DevULong64 *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_change_event (string attr_name, Tango::DevULong64 *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_change_event (string attr_name, Tango::DevState *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_change_event (string attr_name, Tango::DevState *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif												
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_change_event();
+}
+
+
 //////////////////// Push change archive methods!!! //////////////////////////////////////
 
 
@@ -939,6 +1257,21 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevLong *p_data, l
 	attr.fire_archive_event();
 }
 
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevLong64 *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+
 
 void DeviceImpl::push_archive_event(string attr_name, Tango::DevFloat *p_data, long x, long y , bool release)
 {
@@ -1040,6 +1373,49 @@ void DeviceImpl::push_archive_event(string attr_name, Tango::DevUChar *p_data, l
 	attr.fire_archive_event();
 }
 
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevULong *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevULong64 *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevState *p_data, long x, long y , bool release)
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value (p_data, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+
 
 //+-------------------------------------------------------------------------
 //
@@ -1057,7 +1433,7 @@ void DeviceImpl::push_archive_event(string attr_name, Tango::DevUChar *p_data, l
 //					   release   : The release flag. If true, memory pointed to by p_data will be 
 // 									freed after being send to the client. Default value is false.
 //--------------------------------------------------------------------------
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevShort *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1080,7 +1456,7 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevShort *p_data,
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevLong *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1103,7 +1479,31 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevLong *p_data,
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevLong64 *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevLong64 *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+
+
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevFloat *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1126,7 +1526,7 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevFloat *p_data,
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevDouble *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1149,7 +1549,7 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevDouble *p_data,
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevString *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1172,7 +1572,7 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevString *p_data,
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevBoolean *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1195,7 +1595,7 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevBoolean *p_data
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevUShort *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1218,7 +1618,7 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevUShort *p_data,
 	attr.fire_archive_event();
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 void DeviceImpl::push_archive_event (string attr_name, Tango::DevUChar *p_data, 
 											  struct _timeb &t, Tango::AttrQuality qual,
 											  long x,long y ,bool release)
@@ -1241,5 +1641,74 @@ void DeviceImpl::push_archive_event (string attr_name, Tango::DevUChar *p_data,
 	attr.fire_archive_event();
 }
 
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevULong *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevULong *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevULong64 *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevULong64 *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+
+#ifdef _TG_WINDOWS_
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevState *p_data, 
+											  struct _timeb &t, Tango::AttrQuality qual,
+											  long x,long y ,bool release)
+#else
+void DeviceImpl::push_archive_event (string attr_name, Tango::DevState *p_data, 
+												struct timeval &t, Tango::AttrQuality qual,
+												long x,long y ,bool release)
+#endif
+{
+	// get the tango synchroisation monitor
+	Tango::AutoTangoMonitor synch(this);
+	
+	// search the attribute from the attribute list		
+	Tango::MultiAttribute *attr_list = get_device_attr();
+	Tango::Attribute &attr           = attr_list->get_attr_by_name (attr_name.c_str());
+
+	// set the attribute value
+	attr.set_value_date_quality (p_data, t, qual, x, y, release);
+	// push the event
+	attr.fire_archive_event();
+}
+
 
 } // End of Tango namespace
