@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_2.cpp,v 3.12 2006/06/20 08:00:04 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: device_2.cpp,v 3.15 2007/04/20 14:40:28 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -15,9 +15,21 @@ static const char *RcsId = "$Id: device_2.cpp,v 3.12 2006/06/20 08:00:04 jensmey
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.12 $
+// $Revision: 3.15 $
 //
 // $Log: device_2.cpp,v $
+// Revision 3.15  2007/04/20 14:40:28  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.14  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.13  2007/03/06 08:18:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.12  2006/06/20 08:00:04  jensmeyer
 // After changing the the init value of the attribute polling period to -1 insted of 0 I detected
 // a campatibility problem with the old versions. I changed back the init value to 0 and
@@ -247,11 +259,11 @@ static const char *RcsId = "$Id: device_2.cpp,v 3.12 2006/06/20 08:00:04 jensmey
 #include <device_2.h>
 #include <new>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 #include <sys/timeb.h>
 #else
 #include <sys/time.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 
 namespace Tango
@@ -462,7 +474,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 // to give it to the polling thread
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 						Sleep((DWORD)600);
 						get_poll_monitor().get_monitor();
 #else
@@ -512,7 +524,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 			{
 				double last = polled_cmd->get_last_insert_date();
 				struct timeval now;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				struct _timeb now_win;
 				_ftime(&now_win);
 				now.tv_sec = (unsigned long)now_win.time;
@@ -695,7 +707,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				{
 					for (j = 0;j < nb_poll;j++)
 					{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 						if (_stricmp(poll_list[j]->get_name().c_str(),real_names[i]) == 0)
 #else
 						if (strcasecmp(poll_list[j]->get_name().c_str(),real_names[i]) == 0)
@@ -743,7 +755,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 						vector<string> &napa = get_non_auto_polled_attr();
 						for (j = 0;j < napa.size();j++)
 						{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 							if (_stricmp(napa[j].c_str(),real_names[non_polled[i]]) == 0)
 #else
 							if (strcasecmp(napa[j].c_str(),real_names[non_polled[i]]) == 0)
@@ -789,7 +801,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 // Wait for first polling
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				get_poll_monitor().rel_monitor();
 				Sleep((DWORD)600);
 				get_poll_monitor().get_monitor();
@@ -834,7 +846,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				unsigned long j;
 				for (j = 0;j < poll_list.size();j++)
 				{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 					if ((poll_list[j]->get_type() == Tango::POLL_ATTR) &&
 			    	    	    (_stricmp(poll_list[j]->get_name().c_str(),real_names[i]) == 0))
 #else
@@ -873,7 +885,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				{
 					double last = polled_attr->get_last_insert_date();
 					struct timeval now;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 					struct _timeb now_win;
 					_ftime(&now_win);
 					now.tv_sec = (unsigned long)now_win.time;
@@ -922,7 +934,6 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				{
 					if (w_type != Tango::READ)
 					{
-	
 						delete back;
 						back = NULL;
 				
@@ -964,6 +975,8 @@ throw (Tango::DevFailed, CORBA::SystemException)
 								Tango::DevVarShortArray *new_tmp_sh;
 								const Tango::DevVarLongArray *tmp_lg;
 								Tango::DevVarLongArray *new_tmp_lg;
+								const Tango::DevVarLong64Array *tmp_lg64;
+								Tango::DevVarLong64Array *new_tmp_lg64;
 								const Tango::DevVarStringArray *tmp_str;
 								Tango::DevVarStringArray *new_tmp_str;
 								const Tango::DevVarFloatArray *tmp_fl;
@@ -974,6 +987,12 @@ throw (Tango::DevFailed, CORBA::SystemException)
 								Tango::DevVarUShortArray *new_tmp_ush;
 								const Tango::DevVarCharArray *tmp_uch;
 								Tango::DevVarCharArray *new_tmp_uch;
+								const Tango::DevVarULongArray *tmp_ulg;
+								Tango::DevVarULongArray *new_tmp_ulg;
+								const Tango::DevVarULong64Array *tmp_ulg64;
+								Tango::DevVarULong64Array *new_tmp_ulg64;
+								const Tango::DevVarStateArray *tmp_state;
+								Tango::DevVarStateArray *new_tmp_state;
 								
 								switch (type)
 								{
@@ -999,13 +1018,17 @@ throw (Tango::DevFailed, CORBA::SystemException)
 
 								case Tango::DEV_LONG :
 									att_val_3.value >>= tmp_lg;
-									new_tmp_lg = new DevVarLongArray(
-										tmp_lg->length(),
-										tmp_lg->length(),
-										const_cast<long *>(tmp_lg->get_buffer()),
-										false);
+									new_tmp_lg = new DevVarLongArray(tmp_lg->length(),tmp_lg->length(),
+																const_cast<DevLong *>(tmp_lg->get_buffer()),false);
 									(*back)[i].value <<= new_tmp_lg;
 									break;
+									
+								case Tango::DEV_LONG64 :
+									att_val_3.value >>= tmp_lg64;
+									new_tmp_lg64 = new DevVarLong64Array(tmp_lg64->length(),tmp_lg64->length(),
+																const_cast<DevLong64 *>(tmp_lg64->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_lg64;
+									break;
 
 								case Tango::DEV_STRING :
 									att_val_3.value >>= tmp_str;
@@ -1056,6 +1079,27 @@ throw (Tango::DevFailed, CORBA::SystemException)
 										false);
 									(*back)[i].value <<= new_tmp_uch;
 									break;
+									
+								case Tango::DEV_ULONG :
+									att_val_3.value >>= tmp_ulg;
+									new_tmp_ulg = new DevVarULongArray(tmp_ulg->length(),tmp_ulg->length(),
+																const_cast<DevULong *>(tmp_ulg->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_ulg;
+									break;
+									
+								case Tango::DEV_ULONG64 :
+									att_val_3.value >>= tmp_ulg64;
+									new_tmp_ulg64 = new DevVarULong64Array(tmp_ulg64->length(),tmp_ulg64->length(),
+																const_cast<DevULong64 *>(tmp_ulg64->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_ulg64;
+									break;
+									
+								case Tango::DEV_STATE :
+									att_val_3.value >>= tmp_state;
+									new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
+																const_cast<DevState *>(tmp_state->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_state;
+									break;
 								}
 							}
 
@@ -1077,6 +1121,8 @@ throw (Tango::DevFailed, CORBA::SystemException)
 								Tango::DevVarShortArray *new_tmp_sh;
 								const Tango::DevVarLongArray *tmp_lg;
 								Tango::DevVarLongArray *new_tmp_lg;
+								const Tango::DevVarLong64Array *tmp_lg64;
+								Tango::DevVarLong64Array *new_tmp_lg64;
 								const Tango::DevVarStringArray *tmp_str;
 								Tango::DevVarStringArray *new_tmp_str;
 								const Tango::DevVarFloatArray *tmp_fl;
@@ -1087,6 +1133,12 @@ throw (Tango::DevFailed, CORBA::SystemException)
 								Tango::DevVarUShortArray *new_tmp_ush;
 								const Tango::DevVarCharArray *tmp_uch;
 								Tango::DevVarCharArray *new_tmp_uch;
+								const Tango::DevVarULongArray *tmp_ulg;
+								Tango::DevVarULongArray *new_tmp_ulg;
+								const Tango::DevVarULong64Array *tmp_ulg64;
+								Tango::DevVarULong64Array *new_tmp_ulg64;
+								const Tango::DevVarStateArray *tmp_state;
+								Tango::DevVarStateArray *new_tmp_state;
 								
 								switch (type)
 								{
@@ -1112,13 +1164,17 @@ throw (Tango::DevFailed, CORBA::SystemException)
 
 								case Tango::DEV_LONG :
 									att_val.value >>= tmp_lg;
-									new_tmp_lg = new DevVarLongArray(
-										tmp_lg->length(),
-										tmp_lg->length(),
-										const_cast<long *>(tmp_lg->get_buffer()),
-										false);
+									new_tmp_lg = new DevVarLongArray(tmp_lg->length(),tmp_lg->length(),
+												const_cast<DevLong *>(tmp_lg->get_buffer()),false);
 									(*back)[i].value <<= new_tmp_lg;
 									break;
+									
+								case Tango::DEV_LONG64 :
+									att_val.value >>= tmp_lg64;
+									new_tmp_lg64 = new DevVarLong64Array(tmp_lg64->length(),tmp_lg64->length(),
+												const_cast<DevLong64 *>(tmp_lg64->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_lg64;
+									break;
 
 								case Tango::DEV_STRING :
 									att_val.value >>= tmp_str;
@@ -1169,6 +1225,27 @@ throw (Tango::DevFailed, CORBA::SystemException)
 										false);
 									(*back)[i].value <<= new_tmp_uch;
 									break;
+									
+								case Tango::DEV_ULONG :
+									att_val.value >>= tmp_ulg;
+									new_tmp_ulg = new DevVarULongArray(tmp_ulg->length(),tmp_ulg->length(),
+												const_cast<DevULong *>(tmp_ulg->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_ulg;
+									break;
+									
+								case Tango::DEV_ULONG64 :
+									att_val.value >>= tmp_ulg64;
+									new_tmp_ulg64 = new DevVarULong64Array(tmp_ulg64->length(),tmp_ulg64->length(),
+												const_cast<DevULong64 *>(tmp_ulg64->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_ulg64;
+									break;
+									
+								case Tango::DEV_STATE :
+									att_val.value >>= tmp_state;
+									new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
+												const_cast<DevState *>(tmp_state->get_buffer()),false);
+									(*back)[i].value <<= new_tmp_state;
+									break;
 								}
 							}
 							
@@ -1673,7 +1750,7 @@ throw(Tango::DevFailed, CORBA::SystemException)
 		try
 		{
 			back_attr = new Tango::DevAttrHistoryList_3(n);
-                	back_attr->length(n);
+			back_attr->length(n);
 		}	
 		catch (bad_alloc)
 		{
