@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_3.cpp,v 3.18 2006/06/20 08:00:04 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: device_3.cpp,v 3.25 2007/06/05 15:52:29 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -15,9 +15,33 @@ static const char *RcsId = "$Id: device_3.cpp,v 3.18 2006/06/20 08:00:04 jensmey
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.18 $
+// $Revision: 3.25 $
 //
 // $Log: device_3.cpp,v $
+// Revision 3.25  2007/06/05 15:52:29  taurel
+// - Clarify one error message
+//
+// Revision 3.24  2007/04/20 14:40:28  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.23  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.22  2007/03/06 08:18:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.21  2007/02/16 10:40:09  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
+// Revision 3.20  2007/02/08 16:32:56  taurel
+// - Remove an old already commented line
+//
+// Revision 3.19  2007/01/19 10:25:00  taurel
+// - Change a logging message to be more precise
+//
 // Revision 3.18  2006/06/20 08:00:04  jensmeyer
 // After changing the the init value of the attribute polling period to -1 insted of 0 I detected
 // a campatibility problem with the old versions. I changed back the init value to 0 and
@@ -179,11 +203,13 @@ static const char *RcsId = "$Id: device_3.cpp,v 3.18 2006/06/20 08:00:04 jensmey
 #include <device_3.h>
 #include <new>
 
-#ifdef WIN32
+#include <eventsupplier.h>
+
+#ifdef _TG_WINDOWS_
 #include <sys/timeb.h>
 #else
 #include <sys/time.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 
 namespace Tango
@@ -261,7 +287,8 @@ Tango::AttributeValueList_3* Device_3Impl::read_attributes_3(const Tango::DevVar
 					     Tango::DevSource source)
 throw (Tango::DevFailed, CORBA::SystemException)
 {
-	cout4 << "Device_3Impl::read_attributes_3 arrived" << endl;
+//	cout4 << "Device_3Impl::read_attributes_3 arrived" << endl;
+	cout4 << "Device_3Impl::read_attributes_3 arrived for dev " << get_name() << ", att[0] = " << names[0] << endl;
 
 //
 // Record operation request in black box
@@ -597,7 +624,6 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 					        		(const char *)"Device_3Impl::read_attributes_no_except");
 				}
 				attr_vect[att.get_attr_idx()]->read(this,att);
-//				read_attr(dev_attr->get_attr_by_ind(wanted_attr[i].idx_in_multi_attr));
 
 			}
 			catch (Tango::DevFailed &e)
@@ -775,7 +801,7 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 					(*back)[index].err_list[0].severity = Tango::ERR;
 					(*back)[index].err_list[0].reason = CORBA::string_dup("API_AttrValueNotSet");
 					(*back)[index].err_list[0].origin = CORBA::string_dup("Device_3Impl::read_attributes_no_except");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 					string s = o.str();
 					(*back)[index].err_list[0].desc = CORBA::string_dup(s.c_str());
 #else
@@ -796,11 +822,9 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 			    		    	    (w_type == Tango::READ_WRITE) ||
 			    		    	    (w_type == Tango::READ_WITH_WRITE))
 						{
-							if ((w_type == Tango::READ_WRITE) ||
-				    		    	    (w_type == Tango::READ_WITH_WRITE))
+							if ((w_type == Tango::READ_WRITE) || (w_type == Tango::READ_WITH_WRITE))
 								dev_attr->add_write_value(att);
-				
-								
+									
 							if ((att.is_alarmed().any() == true) && (qual != Tango::ATTR_INVALID))
 								att.check_alarm();
 
@@ -810,37 +834,101 @@ void Device_3Impl::read_attributes_no_except(const Tango::DevVarStringArray& nam
 						switch (att.get_data_type())
 						{
 						case Tango::DEV_SHORT :
-							a <<= att.get_short_value();
+						{
+							Tango::DevVarShortArray *ptr = att.get_short_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
 				
 						case Tango::DEV_LONG :
-							a <<= att.get_long_value();
+						{
+							Tango::DevVarLongArray *ptr = att.get_long_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
+							
+						case Tango::DEV_LONG64 :
+						{
+							Tango::DevVarLong64Array *ptr = att.get_long64_value();
+							a <<= *ptr;
+							delete ptr;
+							break;
+						}
 				
 						case Tango::DEV_DOUBLE :
-							a <<= att.get_double_value();
+						{
+							Tango::DevVarDoubleArray *ptr = att.get_double_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
 				
 						case Tango::DEV_STRING :
-							a <<= att.get_string_value();
+						{
+							Tango::DevVarStringArray *ptr = att.get_string_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
 							
 						case Tango::DEV_FLOAT :
-							a <<= att.get_float_value();
+						{
+							Tango::DevVarFloatArray *ptr = att.get_float_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
 				
 						case Tango::DEV_BOOLEAN :
-							a <<= att.get_boolean_value();
+						{
+							Tango::DevVarBooleanArray *ptr = att.get_boolean_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
 				
 						case Tango::DEV_USHORT :
-							a <<= att.get_ushort_value();
+						{
+							Tango::DevVarUShortArray *ptr = att.get_ushort_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
+						}
 				
 						case Tango::DEV_UCHAR :
-							a <<= att.get_uchar_value();
+						{
+							Tango::DevVarUCharArray *ptr = att.get_uchar_value();
+							a <<= *ptr;
+							delete ptr;
 							break;
 						}
+							
+						case Tango::DEV_ULONG :
+						{
+							Tango::DevVarULongArray *ptr = att.get_ulong_value();
+							a <<= *ptr;
+							delete ptr;
+							break;
+						}
+							
+						case Tango::DEV_ULONG64 :
+						{
+							Tango::DevVarULong64Array *ptr = att.get_ulong64_value();
+							a <<= *ptr;
+							delete ptr;
+							break;
+						}
+							
+						case Tango::DEV_STATE :
+						{
+							Tango::DevVarStateArray *ptr = att.get_state_value();
+							a <<= *ptr;
+							delete ptr;
+							break;
+						}
+						}
 			
 						if (att.get_when().tv_sec == 0)
 							att.set_time();	
@@ -932,7 +1020,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 			dev_attr->get_attr_ind_by_name(names[i]);
 			for (j = 0;j < nb_poll;j++)
 			{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				if (_stricmp(poll_list[j]->get_name().c_str(),names[i]) == 0)
 #else
 				if (strcasecmp(poll_list[j]->get_name().c_str(),names[i]) == 0)
@@ -985,7 +1073,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 				(*back)[non_polled[i]].err_list[0].severity = Tango::ERR;
 				(*back)[non_polled[i]].err_list[0].reason = CORBA::string_dup("API_AttrNotPolled");
 				(*back)[non_polled[i]].err_list[0].origin = CORBA::string_dup("Device_3Impl::read_attributes_from_cache");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 				string s = o.str();
 				(*back)[non_polled[i]].err_list[0].desc = CORBA::string_dup(s.c_str());
 #else
@@ -1006,7 +1094,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 				vector<string> &napa = get_non_auto_polled_attr();
 				for (j = 0;j < napa.size();j++)
 				{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 					if (_stricmp(napa[j].c_str(),names[non_polled[i]]) == 0)
 #else
 					if (strcasecmp(napa[j].c_str(),names[non_polled[i]]) == 0)
@@ -1022,7 +1110,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 					(*back)[non_polled[i]].err_list[0].severity = Tango::ERR;
 					(*back)[non_polled[i]].err_list[0].reason = CORBA::string_dup("API_AttrNotPolled");
 					(*back)[non_polled[i]].err_list[0].origin = CORBA::string_dup("Device_3Impl::read_attributes_from_cache");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 					string s = o.str();
 					(*back)[non_polled[i]].err_list[0].desc = CORBA::string_dup(s.c_str());
 #else
@@ -1097,7 +1185,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 
 		if (start_polling_nb != 0)
 		{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			get_poll_monitor().rel_monitor();
 			Sleep((DWORD)500);
 			get_poll_monitor().get_monitor();
@@ -1128,7 +1216,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 		unsigned long j;
 		for (j = 0;j < poll_list.size();j++)
 		{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			if ((poll_list[j]->get_type() == Tango::POLL_ATTR) &&
 	   	    	    (_stricmp(poll_list[j]->get_name().c_str(),names[i]) == 0))
 #else
@@ -1154,7 +1242,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 			(*back)[i].err_list[0].severity = Tango::ERR;
 			(*back)[i].err_list[0].reason = CORBA::string_dup("API_NoDataYet");
 			(*back)[i].err_list[0].origin = CORBA::string_dup("Device_3Impl::read_attributes_from_cache");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			string s = o.str();
 			(*back)[i].err_list[0].desc = CORBA::string_dup(s.c_str());
 #else
@@ -1179,7 +1267,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 		{
 			double last = polled_attr->get_last_insert_date();
 			struct timeval now;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			struct _timeb now_win;
 			_ftime(&now_win);
 			now.tv_sec = (unsigned long)now_win.time;
@@ -1200,7 +1288,7 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 				(*back)[i].err_list[0].severity = Tango::ERR;
 				(*back)[i].err_list[0].reason = CORBA::string_dup("API_NotUpdatedAnyMore");
 				(*back)[i].err_list[0].origin = CORBA::string_dup("Device_3Impl::read_attributes_from_cache");
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 				string s = o.str();
 				(*back)[i].err_list[0].desc = CORBA::string_dup(s.c_str());
 #else
@@ -1248,6 +1336,8 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 					Tango::DevVarShortArray *new_tmp_sh;
 					const Tango::DevVarLongArray *tmp_lg;
 					Tango::DevVarLongArray *new_tmp_lg;
+					const Tango::DevVarLong64Array *tmp_lg64;
+					Tango::DevVarLong64Array *new_tmp_lg64;
 					const Tango::DevVarStringArray *tmp_str;
 					Tango::DevVarStringArray *new_tmp_str;
 					const Tango::DevVarFloatArray *tmp_fl;
@@ -1258,6 +1348,12 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 					Tango::DevVarUShortArray *new_tmp_ush;
 					const Tango::DevVarCharArray *tmp_uch;
 					Tango::DevVarCharArray *new_tmp_uch;
+					const Tango::DevVarULongArray *tmp_ulg;
+					Tango::DevVarULongArray *new_tmp_ulg;
+					const Tango::DevVarULong64Array *tmp_ulg64;
+					Tango::DevVarULong64Array *new_tmp_ulg64;
+					const Tango::DevVarStateArray *tmp_state;
+					Tango::DevVarStateArray *new_tmp_state;
 					Tango::DevState sta;
 																														
 					switch (type)
@@ -1282,12 +1378,17 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 
 					case Tango::DEV_LONG :
 						att_val.value >>= tmp_lg;
-						new_tmp_lg = new DevVarLongArray(tmp_lg->length(),
-										 tmp_lg->length(),
-										 const_cast<long *>(tmp_lg->get_buffer()),
-										 false);
+						new_tmp_lg = new DevVarLongArray(tmp_lg->length(),tmp_lg->length(),
+										 const_cast<DevLong *>(tmp_lg->get_buffer()),false);
 						(*back)[i].value <<= new_tmp_lg;
 						break;
+						
+					case Tango::DEV_LONG64 :
+						att_val.value >>= tmp_lg64;
+						new_tmp_lg64 = new DevVarLong64Array(tmp_lg64->length(),tmp_lg64->length(),
+										 const_cast<DevLong64 *>(tmp_lg64->get_buffer()),false);
+						(*back)[i].value <<= new_tmp_lg64;
+						break;
 
 					case Tango::DEV_STRING :
 						att_val.value >>= tmp_str;
@@ -1334,9 +1435,37 @@ void Device_3Impl::read_attributes_from_cache(const Tango::DevVarStringArray& na
 						(*back)[i].value <<= new_tmp_uch;
 						break;
 						
+					case Tango::DEV_ULONG :
+						att_val.value >>= tmp_ulg;
+						new_tmp_ulg = new DevVarULongArray(tmp_ulg->length(),tmp_ulg->length(),
+										 const_cast<DevULong *>(tmp_ulg->get_buffer()),false);
+						(*back)[i].value <<= new_tmp_ulg;
+						break;
+						
+					case Tango::DEV_ULONG64 :
+						att_val.value >>= tmp_ulg64;
+						new_tmp_ulg64 = new DevVarULong64Array(tmp_ulg64->length(),tmp_ulg64->length(),
+										 const_cast<DevULong64 *>(tmp_ulg64->get_buffer()),false);
+						(*back)[i].value <<= new_tmp_ulg64;
+						break;
+						
 					case Tango::DEV_STATE :
-						att_val.value >>= sta;
-						(*back)[i].value <<= sta;
+						CORBA::TypeCode_var ty;
+						ty = att_val.value.type();
+			
+						if (ty->kind() == CORBA::tk_enum)
+						{
+							att_val.value >>= sta;
+							(*back)[i].value <<= sta;
+						}
+						else
+						{
+							att_val.value >>= tmp_state;
+							new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
+											 const_cast<DevState *>(tmp_state->get_buffer()),false);
+							(*back)[i].value <<= new_tmp_state;
+						}
+						break;
 					}
 				}
 				(*back)[i].quality= att_val.quality;
@@ -1539,7 +1668,8 @@ throw (Tango::MultiDevFailed, Tango::DevFailed, CORBA::SystemException)
 					TangoSys_OMemStream o;
 		
 					o << "It is actually not allowed to write attribute ";
-					o << att.get_name() << ends;
+					o << att.get_name();
+					o << ". The device state is " << Tango::DevStateName[get_state()] << ends;
 
 
 					Except::throw_exception((const char *)"API_AttrNotAllowed",
@@ -1960,7 +2090,13 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				        (const char *)"Device_3Impl::set_attribute_config_3");
 	}
 	
-
+//
+// Get some event related data
+//
+	
+	Tango::Util *tg = Tango::Util::instance();
+	EventSupplier *ev_supply = tg->get_event_supplier();
+	
 //
 // Update attribute config first in database, then locally
 //
@@ -1995,6 +2131,16 @@ throw (Tango::DevFailed, CORBA::SystemException)
 			    (old_alarm == true) &&
 			    (attr.is_alarmed().any() == false))
 				attr.set_quality(Tango::ATTR_VALID);
+				
+//
+// Send the event
+//
+				
+			if (ev_supply != NULL)
+			{
+				string tmp_name(new_conf[i].name);
+				ev_supply->push_att_conf_events(this,new_conf[i],(Tango::DevFailed *)NULL,tmp_name);
+			}
 		}
 
 	}
@@ -2029,7 +2175,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 			o << "\nAll remaining attribute(s) have not been updated";
 		o << ends;
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 		string s = o.str();
 		e.errors[0].reason = CORBA::string_dup(s.c_str());
 #else		
@@ -2055,8 +2201,7 @@ throw (Tango::DevFailed, CORBA::SystemException)
 				dev_attr->get_alarm_list().push_back(i);
 		}
 	}
-
-
+	
 //
 // Return to caller
 //
@@ -2121,6 +2266,10 @@ void Device_3Impl::write_attributes_in_db(vector<long> &att_in_db,vector<AttIdx>
 			tmp_db << (*att.get_last_written_lg())[0];
 			break;
 			
+		case Tango::DEV_LONG64 :
+			tmp_db << (*att.get_last_written_lg64())[0];
+			break;
+			
 		case Tango::DEV_DOUBLE :
 			tmp_db << (*att.get_last_written_db())[0];
 			break;
@@ -2144,7 +2293,22 @@ void Device_3Impl::write_attributes_in_db(vector<long> &att_in_db,vector<AttIdx>
 			
 		case Tango::DEV_UCHAR :
 			tmp_db << (*att.get_last_written_uch())[0];
-			break;						
+			break;
+			
+		case Tango::DEV_ULONG :
+			tmp_db << (*att.get_last_written_ulg())[0];
+			break;
+			
+		case Tango::DEV_ULONG64 :
+			tmp_db << (*att.get_last_written_ulg64())[0];
+			break;		
+			
+		case Tango::DEV_STATE :
+			{
+			Tango::DevState tmp_state = (*att.get_last_written_state())[0];
+			tmp_db << (short)tmp_state;
+			}
+			break;
 		}
 		db_data.push_back(tmp_db);			
 	}
@@ -2352,7 +2516,7 @@ void Device_3Impl::alarmed_not_read(vector<AttIdx> &wanted_attr)
 void Device_3Impl::state2attr(Tango::DevState state,Tango::AttributeValue_3 &back)
 {
 	back.value <<= state;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	struct _timeb after_win;
 
 	_ftime(&after_win);
@@ -2394,7 +2558,7 @@ void Device_3Impl::status2attr(Tango::ConstDevString status,Tango::AttributeValu
 	str_seq[0] = CORBA::string_dup(status);
 		
 	back.value <<= str_seq;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	struct _timeb after_win;
 
 	_ftime(&after_win);
