@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: deviceclass.cpp,v 1.14 2006/06/21 14:52:19 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: deviceclass.cpp,v 1.21 2007/05/22 12:48:40 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,37 @@ static const char *RcsId = "$Id: deviceclass.cpp,v 1.14 2006/06/21 14:52:19 taur
 //
 // author(s) :		E.Taurel
 //
-// $Revision: 1.14 $
+// $Revision: 1.21 $
 //
 // $Log: deviceclass.cpp,v $
+// Revision 1.21  2007/05/22 12:48:40  taurel
+// - Clarified an error message
+//
+// Revision 1.20  2007/05/17 07:59:07  taurel
+// - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
+// Add device name in monitor print message
+// Add device_destroyer method in DeviceClass class
+//
+// Revision 1.19  2007/05/11 07:29:29  taurel
+// - Added device_destroyer() methods. These method are doing the opposite of
+// the device_factory() one. They remove (and delete) a device from the list
+// of available device in the class. They do not mmodify the DB
+//
+// Revision 1.18  2007/04/20 14:40:29  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 1.17  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 1.16  2007/03/06 08:18:04  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 1.15  2006/11/20 16:26:37  taurel
+// - Small changes when removing servant in case the device is not exported
+//
 // Revision 1.14  2006/06/21 14:52:19  taurel
 // - Just fixing a logging print message
 //
@@ -279,7 +307,7 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 	Tango::DevVarShortArray sh_seq(1);
 	sh_seq.length(1);
 						
-	long lg;
+	Tango::DevLong lg;
 	Tango::DevVarLongArray lg_seq(1);
 	lg_seq.length(1);
 		
@@ -419,21 +447,18 @@ void DeviceClass::set_memorized_values(bool all,long idx)
 							break;
 
 						case Tango::DEV_BOOLEAN:
-	#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 							if (!(str >> boolalpha >> boo))
 								throw_mem_value(device_list[i],att);
-	#else
-							transform(mem_value.begin(),
-								  mem_value.end(),
-								  mem_value.begin(),
-								  ::tolower);
+#else
+							transform(mem_value.begin(),mem_value.end(),mem_value.begin(),::tolower);
 							if (mem_value == "true")
 								boo = true;
 							else if (mem_value == "false")
 								boo = false;
 							else
 								throw_mem_value(device_list[i],att);
-	#endif
+#endif
 							att.set_write_value(boo);
 							
 							boo_seq[0] = boo;
@@ -661,7 +686,7 @@ DeviceClass::~DeviceClass()
 void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_poa)
 {
 	cout4 << "Entering DeviceClas::delete_dev method for device with index " << idx << endl;
-		
+
 //
 // If the polling thread is alive and if device is polled,
 // ask polling thread to stop polling
@@ -694,7 +719,7 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 	omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
 	PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;		
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	try
 	{
 		lock_ptr->Release();
@@ -722,7 +747,7 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 
 		lock_ptr->Get();
 	}
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 	catch (Tango::DevFailed &)
 	{
 	}
@@ -732,7 +757,15 @@ void DeviceClass::delete_dev(long idx,Tango::Util *tg,PortableServer::POA_ptr r_
 //
 
 	if (device_list[idx]->is_py_device() == false)
-		device_list[idx]->_remove_ref();
+	{
+		if (device_list[idx]->get_exported_flag() == true)
+			device_list[idx]->_remove_ref();
+		else
+		{
+			Device_3Impl *dev_3 = static_cast<Device_3Impl *>(device_list[idx]);
+			dev_3->delete_dev();
+		}
+	}		
 	else
 	{
 		Device_3Impl *dev_3 = static_cast<Device_3Impl *>(device_list[idx]);
@@ -1208,5 +1241,69 @@ void DeviceClass::add_wiz_class_prop(string &p_name,string &desc)
 	add_wiz_class_prop(p_name,desc,def);
 }
 
+//+----------------------------------------------------------------------------
+//
+// method :		DeviceClass::device_destroyer
+// 
+// description :	Method to remove and delete a device from the running devices
+//					belonging to the Tango class
+//
+// in : 	dev_name : The device name
+//
+//-----------------------------------------------------------------------------
+
+void DeviceClass::device_destroyer(const string &dev_name)
+{
+	
+//
+// Check that the class know this device
+//
+  
+  	unsigned long k;				
+  	for (k = 0;k < device_list.size();k++)
+  	{
+  		if (device_list[k]->get_name() == dev_name)
+  			break;
+  	}
+  
+  	if (k == device_list.size())
+  	{
+		TangoSys_OMemStream o;
+		o << "Device " << dev_name << " not in Tango class device list!" << ends;
+		
+  		Tango::Except::throw_exception((const char *)"API_CantDestroyDevice",o.str(),
+  					       (const char *)"DeviceClass::device_destroyer");
+  	}
+	
+//
+// Check if the device is polled
+// If yes, ask polling thread to stop polling it
+//
+
+	if (device_list[k]->is_polled() == true)
+	{
+		device_list[k]->stop_polling();
+	}
+	
+			
+//
+// Delete the device
+//
+ 
+	Tango::Util *tg = Tango::Util::instance();
+	 			
+	PortableServer::POA_ptr r_poa = tg->get_poa();
+	delete_dev(k,tg,r_poa);
+	vector<DeviceImpl *>::iterator it = device_list.begin();
+	it += k;
+	device_list.erase(it);
+	CORBA::release(r_poa);  	  				
+}
+
+void DeviceClass::device_destroyer(const char *dev_name)
+{
+	string name_str(dev_name);
+	return device_destroyer(name_str);
+}
 
 } // End of Tango namespace
