@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserver.cpp,v 3.21 2006/06/28 09:21:28 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserver.cpp,v 3.23 2007/04/20 14:40:32 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,15 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.21 2006/06/28 09:21:28 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.21 $
+// $Revision: 3.23 $
 //
 // $Log: dserver.cpp,v $
+// Revision 3.23  2007/04/20 14:40:32  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.22  2007/01/19 10:24:04  taurel
+// - Fix bug in the delete_devices() method which makes the Util::get_device_list_by_class() method generating core in some cases
+//
 // Revision 3.21  2006/06/28 09:21:28  taurel
 // - Remove a 1 second sleep at the beginning of the thread executing the Kill command
 //
@@ -288,11 +294,11 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.21 2006/06/28 09:21:28 taurel E
 #include <algorithm>
 #include <math.h>
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 #include <unistd.h>
 #include <signal.h>
 #include <dlfcn.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 #include <stdlib.h>
 
@@ -728,6 +734,7 @@ void DServer::delete_devices()
 				CORBA::release(r_poa);
 				
 				delete class_list[i];
+				class_list.pop_back();
 			}
 			else
 			{
@@ -1437,7 +1444,7 @@ void DServer::create_cpp_class(const char *cl_name,const char *par_name)
 	string class_name(cl_name);
 	string lib_name = class_name;
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	HMODULE mod;
 
 	if ((mod = LoadLibrary(lib_name.c_str())) == NULL)
@@ -1515,7 +1522,7 @@ void DServer::create_cpp_class(const char *cl_name,const char *par_name)
 	cout4 << "dlsym is a success" << endl;
 	
 	Cpp_creator_ptr mt = (Cpp_creator_ptr)sym;
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 	Tango::DeviceClass *dc = (*mt)(par_name);
 	add_class(dc);
 }
