@@ -11,9 +11,19 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.6 $
+// $Revision: 3.9 $
 //
 // $Log: dserver.h,v $
+// Revision 3.9  2007/05/15 07:46:59  taurel
+// - The polling thread is not configured by a separate thread any more.
+// The Add_obj_polling command now support a delta_t to start the first polling
+//
+// Revision 3.8  2007/04/20 14:40:32  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.7  2007/03/29 07:04:05  taurel
+// - Change some data types for 64 bits compatibility
+//
 // Revision 3.6  2006/05/18 08:52:37  taurel
 // - Miscellaneous changes due to Python device server ported to Windows
 // - Fix some bugs discovered by Windows VC8 using the test suite
@@ -212,7 +222,7 @@ public :
 
 	Tango::DevVarStringArray *polled_device();
 	Tango::DevVarStringArray *dev_poll_status(string &);
-	void add_obj_polling(const Tango::DevVarLongStringArray *,bool with_db_upd = true);
+	void add_obj_polling(const Tango::DevVarLongStringArray *,bool with_db_upd = true,int delta_ms = 0);
 	void upd_obj_polling_period(const Tango::DevVarLongStringArray *,bool with_db_upd = true);
 	void rem_obj_polling(const Tango::DevVarStringArray *,bool with_db_upd = true);
 	void stop_polling();
@@ -251,10 +261,10 @@ protected :
 	
 	vector<DeviceClass *>	class_list;
 	
-	unsigned int		last_heartbeat;
+	time_t			last_heartbeat;
 	bool			heartbeat_started;
 private:
-#if ((defined WIN32) && (defined TANGO_HAS_DLL) && !(defined _TANGO_LIB))
+#if ((defined _TG_WINDOWS_) && (defined TANGO_HAS_DLL) && !(defined _TANGO_LIB))
 	__declspec(dllexport) void class_factory();
 #else
 	void class_factory();
