@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverclass.cpp,v 3.9 2005/08/09 15:53:34 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverclass.cpp,v 3.12 2007/04/20 14:40:33 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,21 @@ static const char *RcsId = "$Id: dserverclass.cpp,v 3.9 2005/08/09 15:53:34 jens
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.9 $
+// $Revision: 3.12 $
 //
 // $Log: dserverclass.cpp,v $
+// Revision 3.12  2007/04/20 14:40:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.11  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.10  2007/03/29 07:04:05  taurel
+// - Change some data types for 64 bits compatibility
+//
 // Revision 3.9  2005/08/09 15:53:34  jensmeyer
 // Changed DServerClass::instance() to return an exception
 // in the case of a not initialised object. The exit() killed the server
@@ -537,7 +549,7 @@ CORBA::Any *DevSetTraceLevelCmd::execute(DeviceImpl *device,const CORBA::Any &in
 // Get new level
 //
 
-	long new_level;
+	int new_level;
 	if ((in_any >>= new_level) == false)
 	{
 		cout3 << "DevSetTraceLevelCmd::execute() --> Wrong argument type" << endl;
@@ -612,7 +624,7 @@ CORBA::Any *DevGetTraceLevelCmd::execute(DeviceImpl *device,const CORBA::Any &in
 // Get level
 //
 
-	long level = Tango::Util::instance()->get_trace_level();
+	int level = Tango::Util::instance()->get_trace_level();
 	
 //
 // return data to the caller
@@ -754,7 +766,7 @@ CORBA::Any *DevSetTraceOutputCmd::execute(DeviceImpl *device,const CORBA::Any &i
 	{
 		delete(Tango::Util::instance()->get_trace_output_stream());
 		Tango::Util::instance()->set_trace_output_stream((ofstream *)NULL);
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _WINDOWS) || (defined __SUNPRO_CC) || (defined GCC_STD)
 		ostream &tmp_stream = Tango::Util::instance()->get_out();
 
 //
@@ -776,7 +788,7 @@ CORBA::Any *DevSetTraceOutputCmd::execute(DeviceImpl *device,const CORBA::Any &i
 		ofstream *ofp = new ofstream(in_file_ptr);
 		if (ofp->good())
 		{
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD)
 			cout.copyfmt(*ofp);
 			cout.clear(ofp->rdstate());
 			cout.rdbuf(ofp->rdbuf());
