@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserverpoll.cpp,v 3.10 2006/04/13 13:31:44 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserverpoll.cpp,v 3.16 2007/05/17 07:59:07 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,34 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.10 2006/04/13 13:31:44 taur
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.10 $
+// $Revision: 3.16 $
 //
 // $Log: dserverpoll.cpp,v $
+// Revision 3.16  2007/05/17 07:59:07  taurel
+// - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
+// Add device name in monitor print message
+// Add device_destroyer method in DeviceClass class
+//
+// Revision 3.15  2007/05/15 07:46:59  taurel
+// - The polling thread is not configured by a separate thread any more.
+// The Add_obj_polling command now support a delta_t to start the first polling
+//
+// Revision 3.14  2007/05/04 13:02:18  jensmeyer
+// Added the removal of the attribute polling and event configuration when
+// executing the remove_attribute() method.
+//
+// Revision 3.13  2007/04/20 14:40:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.12  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.11  2007/03/29 07:00:28  taurel
+// - Fix some data type for 64 bits compatibility
+//
 // Revision 3.10  2006/04/13 13:31:44  taurel
 // Several changes:
 // - A new DeviceImpl::push_event() method to push USER event with State as data
@@ -230,13 +255,13 @@ static const char *RcsId = "$Id: dserverpoll.cpp,v 3.10 2006/04/13 13:31:44 taur
 #include <tango.h>
 #include <math.h>
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#include <sys/timeb.h>
 #else
 	#include <sys/time.h>
 #endif
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	#include <iomanip>
 #else
 	#include <iomanip.h>
@@ -532,7 +557,7 @@ Tango::DevVarStringArray *DServer::dev_poll_status(string &dev_name)
 					returned_info = returned_info + "\nData not updated since ";
 					double since = poll_list[i]->get_last_insert_date_i();
 					struct timeval now;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 					struct _timeb now_win;
 					_ftime(&now_win);
 					now.tv_sec = (unsigned long)now_win.time;
@@ -704,7 +729,7 @@ Tango::DevVarStringArray *DServer::dev_poll_status(string &dev_name)
 //-----------------------------------------------------------------------------
 
 void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
-			      bool with_db_upd)
+			      bool with_db_upd,int delta_ms)
 {
 	NoSyncModelTangoMonitor nosyn_mon(this);
 
@@ -826,7 +851,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 // Check that the update period is not to small
 //
 
-	long upd = (argin->lvalue)[0];
+	int upd = (argin->lvalue)[0];
 	if ((upd != 0) && (upd < MIN_POLL_PERIOD))
 	{
 		TangoSys_OMemStream o;
@@ -884,6 +909,7 @@ void DServer::add_obj_polling(const Tango::DevVarLongStringArray *argin,
 		shared_cmd.cmd_code = POLL_ADD_OBJ;
 		shared_cmd.dev = dev;
 		shared_cmd.index = poll_list.size() - 1;
+		shared_cmd.new_upd = delta_ms;
 
 		mon.signal();
 		
@@ -1147,7 +1173,7 @@ void DServer::upd_obj_polling_period(const Tango::DevVarLongStringArray *argin,
 // Check that the update period is not to small
 //
 
-	long upd = (argin->lvalue)[0];
+	int upd = (argin->lvalue)[0];
 	if ((upd != 0) && (upd < MIN_POLL_PERIOD))
 	{
 		TangoSys_OMemStream o;
@@ -1360,55 +1386,66 @@ void DServer::rem_obj_polling(const Tango::DevVarStringArray *argin,
 	long tmp_upd = (*ite)->get_upd();
 
 //
+// Test whether the polling thread is still running!
+//
+
+	if ( tg->get_polling_thread_object() != NULL )
+	{
+//
 // Send command to the polling thread
 //
 
-	cout4 << "Sending cmd to polling thread" << endl;
-	int interupted;
-	TangoMonitor &mon = tg->get_poll_monitor();
-	PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
-	{	
-		omni_mutex_lock sync(mon);
-		if (shared_cmd.cmd_pending == true)
-		{
-			mon.signal();
-		}
-		shared_cmd.cmd_pending = true;
-		if (tmp_upd == 0)
-			shared_cmd.cmd_code = POLL_REM_EXT_TRIG_OBJ;
-		else
-			shared_cmd.cmd_code = POLL_REM_OBJ;
-		shared_cmd.dev = dev;
-		shared_cmd.name = obj_name;
-		shared_cmd.type = type;
-	
-		mon.signal();
-		
-		cout4 << "Cmd sent to polling thread" << endl;
-	
-//
-// Wait for thread to execute command except if the command is
-// requested by the polling thread itself
-//
-
-		omni_thread *th = omni_thread::self();
-		int th_id = th->id();
-		if (th_id != tg->get_polling_thread_id())
-		{
-			while (shared_cmd.cmd_pending == true)
+		cout4 << "Sending cmd to polling thread" << endl;
+		int interupted;
+		TangoMonitor &mon = tg->get_poll_monitor();
+		PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
+		{	
+			omni_mutex_lock sync(mon);
+			if (shared_cmd.cmd_pending == true)
 			{
-				interupted = mon.wait(DEFAULT_TIMEOUT);
-				if ((shared_cmd.cmd_pending == true) && (interupted == false))
+				mon.signal();
+			}
+			shared_cmd.cmd_pending = true;
+			if (tmp_upd == 0)
+				shared_cmd.cmd_code = POLL_REM_EXT_TRIG_OBJ;
+			else
+				shared_cmd.cmd_code = POLL_REM_OBJ;
+			shared_cmd.dev = dev;
+			shared_cmd.name = obj_name;
+			shared_cmd.type = type;
+
+			mon.signal();
+
+			cout4 << "Cmd sent to polling thread" << endl;
+
+	//
+	// Wait for thread to execute command except if the command is
+	// requested by the polling thread itself
+	//
+
+			omni_thread *th = omni_thread::self();
+			int th_id = th->id();
+			if (th_id != tg->get_polling_thread_id())
+			{
+				while (shared_cmd.cmd_pending == true)
 				{
-					cout4 << "TIME OUT" << endl;
-					Except::throw_exception((const char *)"API_CommandTimedOut",
-					        	(const char *)"Polling thread blocked !!!",
-					        	(const char *)"DServer::rem_obj_polling");
+					interupted = mon.wait(DEFAULT_TIMEOUT);
+					if ((shared_cmd.cmd_pending == true) && (interupted == false))
+					{
+						cout4 << "TIME OUT" << endl;
+						Except::throw_exception((const char *)"API_CommandTimedOut",
+					        		(const char *)"Polling thread blocked !!!",
+					        		(const char *)"DServer::rem_obj_polling");
+					}
 				}
 			}
 		}
+		cout4 << "Thread cmd normally executed" << endl;
 	}
-	cout4 << "Thread cmd normally executed" << endl;
+	else
+		{
+		cout4 << "Polling thread is no longer running!!!!" << endl;
+		}
 
 //
 // Remove the object from the polled object list
@@ -1551,7 +1588,7 @@ void DServer::stop_polling()
 		}
 		shared_cmd.cmd_pending = true;
 		shared_cmd.cmd_code = POLL_STOP;
-	
+
 		mon.signal();
 		
 		while (shared_cmd.cmd_pending == true)
