@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserversignal.cpp,v 3.7 2005/05/09 15:34:11 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserversignal.cpp,v 3.9 2007/04/20 14:40:33 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,18 @@ static const char *RcsId = "$Id: dserversignal.cpp,v 3.7 2005/05/09 15:34:11 tau
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.7 $
+// $Revision: 3.9 $
 //
 // $Log: dserversignal.cpp,v $
+// Revision 3.9  2007/04/20 14:40:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.8  2007/04/16 14:56:36  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
 // Revision 3.7  2005/05/09 15:34:11  taurel
 // - Fix deadlock when read_attribute_no_cache try to start attribute polling
 // - Signal management for Sun
@@ -212,7 +221,7 @@ namespace Tango
 DServerSignal *DServerSignal::_instance = NULL;
 DevSigAction DServerSignal::reg_sig[_NSIG];
 string DServerSignal::sig_name[_NSIG];
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 HANDLE DServerSignal::win_ev = NULL;
 int DServerSignal::win_signo = 0;
 #endif
@@ -263,7 +272,7 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 // Set array of signal name
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	sig_name[SIGINT] = "SIGINT";
 	sig_name[SIGILL] = "SIGILL";
 	sig_name[SIGFPE] = "SIGFPE";
@@ -312,7 +321,7 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 	sig_name[SIGLOST] = "SIGLOST";
 #endif /* sun */
 #endif /* linux */
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 	TangoSys_OMemStream o;	
 	for (long i = 0;i < _NSIG;i++)
@@ -321,7 +330,7 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 		{
 			o << i << ends;
 			sig_name[i] = o.str();
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 			o.seekp(0);
 			o.clear();
 #else
@@ -345,7 +354,7 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 	sigset(SIGQUIT, SIG_DFL);
 #endif
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	
 //
 // Block signals in thread other than the thread dedicated to signal
@@ -378,7 +387,7 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 	sigprocmask(SIG_BLOCK,&sigs_to_block,NULL);
 #endif
 	sigprocmask(SIG_BLOCK,&sigs_to_block,NULL);
-#else /* WIN32 */
+#else /* _TG_WINDOWS_ */
 	win_ev = CreateEvent(NULL,FALSE,FALSE,NULL);
 
 	register_handler(SIGINT);
@@ -386,7 +395,7 @@ DServerSignal::DServerSignal():TangoMonitor("signal")
 	register_handler(SIGABRT);
 	if (Util::_service == false)
 		register_handler(SIGBREAK);
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 //
 // Start the thread dedicated to signal
@@ -840,7 +849,7 @@ void DServerSignal::register_handler(long signo,bool handler)
 {
 	cout4 << "Installing OS signal handler for signal " << sig_name[signo] << endl;
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (::signal(signo, DServerSignal::main_sig_handler) == SIG_ERR)
 	{
 		TangoSys_OMemStream o;
@@ -947,7 +956,7 @@ void DServerSignal::unregister_handler(long signo)
 	cout4 << "Removing OS signal handler for signal " << sig_name[signo] << endl;
 
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (::signal(signo, SIG_DFL) == SIG_ERR)
 	{
 		TangoSys_OMemStream o;
@@ -1042,7 +1051,7 @@ pid_t DServerSignal::get_sig_thread_pid()
 //
 //-----------------------------------------------------------------------------
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	#if (defined __linux)
 void DServerSignal::main_sig_handler(int signo)
 {
