@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: eventcmds.cpp,v 1.11 2006/06/20 08:00:04 jensmeyer Exp $";
+static const char *RcsId = "$Id: eventcmds.cpp,v 1.14 2007/03/06 08:19:43 taurel Exp $";
 //+=============================================================================
 //
 // file :        eventcmds.cpp
@@ -8,11 +8,21 @@ static const char *RcsId = "$Id: eventcmds.cpp,v 1.11 2006/06/20 08:00:04 jensme
 //
 // project :     TANGO events
 //
-// $Author: jensmeyer $
+// $Author: taurel $
 //
-// $Revision: 1.11 $
+// $Revision: 1.14 $
 //
 // $Log: eventcmds.cpp,v $
+// Revision 1.14  2007/03/06 08:19:43  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 1.13  2007/02/16 10:40:57  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
+// Revision 1.12  2006/11/10 08:25:08  taurel
+// - Fix bug if an EventSubsriptionChange command arrives in the admin
+// device while the DS is in its shutdown sequence
+//
 // Revision 1.11  2006/06/20 08:00:04  jensmeyer
 // After changing the the init value of the attribute polling period to -1 insted of 0 I detected
 // a campatibility problem with the old versions. I changed back the init value to 0 and
@@ -218,29 +228,39 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 	const Tango::DevVarStringArray	*argin;
 	extract(in_any, argin);
 
-        if (argin->length() < 4)
-        {
+    if (argin->length() < 4)
+    {
 		TangoSys_OMemStream o;
 		o << "Not enough input arguments, needs 4 i.e. device name, attribute name, action, event name" << ends;
 
 		Except::throw_exception((const char *)"DServer_Events",
-					o.str(),
-					(const char *)"EventSubscriptionChangeCmd::execute");
-
-        }
+								o.str(),
+								(const char *)"EventSubscriptionChangeCmd::execute");
+	}
  
-        string dev_name, attr_name, action, event, attr_name_lower;
+	string dev_name, attr_name, action, event, attr_name_lower;
 	dev_name = (*argin)[0];
-        attr_name = (*argin)[1];
-        action = (*argin)[2];
-        event = (*argin)[3];
+	attr_name = (*argin)[1];
+	action = (*argin)[2];
+	event = (*argin)[3];
 
 	attr_name_lower = attr_name;
 	transform(attr_name_lower.begin(),attr_name_lower.end(),attr_name_lower.begin(),::tolower);
 	
 	cout4 << "EventSubscriptionChangeCmd: execute(): subscribtion for device " << dev_name << " attribute " << attr_name << " action " << action << " event " << event << endl;
-        Tango::Util *tg = Tango::Util::instance();
-	
+	Tango::Util *tg = Tango::Util::instance();
+
+//
+// If we receive this command while the DS is in its
+// shuting down sequence, do nothing
+//
+
+	if (tg->get_polling_thread_object() == NULL)
+	{
+        CORBA::Any *ret = return_empty_any("EventSubscriptionChange");
+		return ret;
+	}
+		
 //
 // If the EventSupplier object is not created, create it right now
 //
@@ -250,36 +270,39 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 		tg->create_event_supplier();
 	}
 	
-
 //
 // Get device reference
 //
 	
-        DeviceImpl *dev_impl;
-        try
-        {
-                dev_impl = tg->get_device_by_name((*argin)[0]);
-        }
-        catch (Tango::DevFailed &e)
-        {
-                TangoSys_OMemStream o;
-                o << "Device " << (*argin)[0] << " not found" << ends;
+	DeviceImpl *dev_impl;
+	try
+	{
+		dev_impl = tg->get_device_by_name((*argin)[0]);
+	}
+	catch (Tango::DevFailed &e)
+	{
+		TangoSys_OMemStream o;
+		o << "Device " << (*argin)[0] << " not found" << ends;
  
-                Except::re_throw_exception(e,(const char *)"API_DeviceNotFound",o.str(),
-                                           (const char *)"DServer::EventSubscriptionChangeCmd:");
-        }                                                                                 
+		Except::re_throw_exception(e,(const char *)"API_DeviceNotFound",o.str(),
+                                   (const char *)"DServer::EventSubscriptionChangeCmd:");
+	}                                                                                 
 
 	int attr_ind = dev_impl->dev_attr->get_attr_ind_by_name(attr_name.c_str());
 	Attribute &attribute = dev_impl->dev_attr->get_attr_by_ind(attr_ind);
-
 	
 	if (action == "subscribe")
 	{
 		if (event == "user_event")
 		{
 			cout4 << "EventSubscriptionChangeCmd::execute(): update user_event subscription\n";
-       			attribute.ext->event_user_subscription = time(NULL);
-      		}
+			attribute.ext->event_user_subscription = time(NULL);
+		}
+		else if (event == "attr_conf")
+		{
+			cout4 << "EventSubscriptionChangeCmd::execute(): update attr_conf subsription\n";
+			attribute.ext->event_attr_conf_subscription = time(NULL);
+		}
 		else
 		{		
 			// If the polling is necessary to send events, check whether the polling is
@@ -322,8 +345,8 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 			}
 			
 				
-       	if (event == "change")
-       	{
+       		if (event == "change")
+       		{
 				cout4 << "EventSubscriptionChangeCmd::execute(): update change subscription\n";
 
 				//
@@ -333,14 +356,14 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 				if (attr_name_lower != "state")
 				{
 					if ((attribute.get_data_type() != Tango::DEV_STRING) &&
-			            	    (attribute.get_data_type() != Tango::DEV_BOOLEAN))
+			            (attribute.get_data_type() != Tango::DEV_BOOLEAN))
 					{
 						if ( attribute.is_check_change_criteria() == true )
 						{
 							if ((attribute.ext->rel_change[0] == INT_MAX) &&
-					    		    (attribute.ext->rel_change[1] == INT_MAX) &&
-					    		    (attribute.ext->abs_change[0] == INT_MAX) &&
-					    		    (attribute.ext->abs_change[1] == INT_MAX))
+					    		(attribute.ext->rel_change[1] == INT_MAX) &&
+					    		(attribute.ext->abs_change[0] == INT_MAX) &&
+					    		(attribute.ext->abs_change[1] == INT_MAX))
 							{		
 								TangoSys_OMemStream o;
 								o << "Event properties (abs_change or rel_change) for attribute ";
@@ -354,23 +377,20 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 						}
 					}
 				}
-       				attribute.ext->event_change_subscription = time(NULL);
+       			attribute.ext->event_change_subscription = time(NULL);
 			}
- 
-      			else if (event == "quality")
-      			{
+      		else if (event == "quality")
+      		{
 				cout4 << "EventSubscriptionChangeCmd::execute(): update quality_change subscription\n";
-       				attribute.ext->event_quality_subscription = time(NULL);
-      			}
- 
-      			else if (event == "periodic")
-      			{
+       			attribute.ext->event_quality_subscription = time(NULL);
+      		}
+      		else if (event == "periodic")
+      		{
 				cout4 << "EventSubscriptionChangeCmd::execute(): update periodic subscription\n";
-       				attribute.ext->event_periodic_subscription = time(NULL);
-      			}                                                  
- 
-      			else if (event == "archive")
-      			{
+       			attribute.ext->event_periodic_subscription = time(NULL);
+      		}                                                  
+      		else if (event == "archive")
+      		{
 				//
 				// Check if the attribute has some of the archive properties defined
 				//
@@ -383,7 +403,7 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 						if ( attribute.is_check_archive_criteria() == true )
 						{
 							if ((attribute.ext->archive_abs_change[0] == INT_MAX) &&
-					      		    (attribute.ext->archive_abs_change[1] == INT_MAX) &&
+					      		(attribute.ext->archive_abs_change[1] == INT_MAX) &&
 						   	    (attribute.ext->archive_period        == INT_MAX))
 							{
 								TangoSys_OMemStream o;
@@ -400,8 +420,8 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 				}		
 		
 				cout4 << "EventSubscriptionChangeCmd::execute(): update archive subscription\n";
-       				attribute.ext->event_archive_subscription = time(NULL);
-      			}				 			
+       			attribute.ext->event_archive_subscription = time(NULL);
+      		}				 			
 		}		  
 	
 		//
@@ -441,14 +461,14 @@ CORBA::Any *EventSubscriptionChangeCmd::execute(Tango::DeviceImpl *device,const
 		}
 		catch (...)
 		{
-				//cout << "EventSubscriptionChangeCmd::execute(): failed to start polling !\n";
-				// nada
+			//cout << "EventSubscriptionChangeCmd::execute(): failed to start polling !\n";
+			// nada
 		}
-			//delete send;			
+		//delete send;			
 		//}
 	}
 
-        CORBA::Any *ret = return_empty_any("EventSubscriptionChange");
+	CORBA::Any *ret = return_empty_any("EventSubscriptionChange");
 	return ret;
 }
 
