@@ -8,9 +8,16 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.1 $
+// $Revision: 3.3 $
 //
 // $Log: except.h,v $
+// Revision 3.3  2007/03/06 08:19:43  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.2  2007/03/02 09:49:06  jensmeyer
+// Added the method compare_exception() to find out whether to DevFailed
+// exceptions are equal or different.
+//
 // Revision 3.1  2005/06/29 08:31:18  taurel
 // - Last commit before release 5.2 ?
 //
@@ -122,6 +129,7 @@
 #define _EXCEPT_H
 
 #include <string>
+
 #include <idl/tango.h>
 
 using namespace std;
@@ -146,7 +154,7 @@ namespace Tango
  * static methods
  *
  * $Author: taurel $
- * $Revision: 3.1 $
+ * $Revision: 3.3 $
  */
  
 class Except
@@ -1782,6 +1790,23 @@ public:
  * <b>DevFailed</b> exception specification
  */	
 	static void throw_exception(const CORBA::SystemException &ex,const string &origin);
+	
+	
+/**
+ * Compare two Tango DevFailed exceptions for equality
+ *
+ * The two DevFailed exceptions are verified by comparing the
+ * reason, origin, description and severity of all exceptions in the error stack.
+ * The strings reason, origin and description are compared literally.
+ * Click <a href="../../../tango_idl/idl_html/Tango.html#DevFailed">here</a> to read
+ * <b>DevFailed</b> exception specification
+ *
+ * @param ex1 The first DevFailed exception
+ * @param ex2 The second DevFailed exception
+ * @return A boolean set to true if the two exceptions are equal
+ */	
+	static bool compare_exception(Tango::DevFailed &ex1, Tango::DevFailed &ex2);	
+	
 		
 //@}
 
