@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: multiattribute.cpp,v 3.8 2005/06/29 08:31:18 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: multiattribute.cpp,v 3.12 2007/04/16 14:57:42 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,24 @@ static const char *RcsId = "$Id: multiattribute.cpp,v 3.8 2005/06/29 08:31:18 ta
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.12 $
 //
 // $Log: multiattribute.cpp,v $
+// Revision 3.12  2007/04/16 14:57:42  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.11  2007/03/06 08:19:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.10  2007/02/08 16:21:19  taurel
+// - Fix a bug in index management in the remove_attribute() method
+//
+// Revision 3.9  2006/11/09 16:29:44  taurel
+// - Fix bug in some index management of the remove_attribute() feature
+//
 // Revision 3.8  2005/06/29 08:31:18  taurel
 // - Last commit before release 5.2 ?
 //
@@ -884,7 +899,7 @@ void MultiAttribute::add_attribute(string &dev_name,
 
 void MultiAttribute::remove_attribute(string &attr_name)
 {
-	cout4 << "Entering MultiAttribute::add_attribute" << endl;
+	cout4 << "Entering MultiAttribute::remove_attribute" << endl;
 
 //
 // Get attribute index in vector
@@ -892,40 +907,6 @@ void MultiAttribute::remove_attribute(string &attr_name)
 
 	long att_index = get_attr_ind_by_name(attr_name.c_str());
 	
-//
-// If it is writable, remove it to the writable attribute list
-//
-
-	Tango::AttrWriteType w_type = attr_list[att_index]->get_writable();
-	if ((w_type == Tango::WRITE) ||
-	    (w_type == Tango::READ_WRITE))
-	{
-		vector<long>::iterator pos;
-		
-		pos = remove(writable_attr_list.begin(),
-		       	     writable_attr_list.end(),
-		             att_index);
-		writable_attr_list.erase(pos,writable_attr_list.end());
-	}
-		
-//
-// If one of the alarm properties is defined, remove it to the alarmed attribute
-// list
-//
-
-	if (attr_list[att_index]->is_alarmed().any() == true)
-	{
-		if (w_type != Tango::WRITE)
-		{
-			vector<long>::iterator pos;
-
-			pos = remove(alarm_attr_list.begin(),
-		       		     alarm_attr_list.end(),
-		        	     att_index);
-			alarm_attr_list.erase(pos,writable_attr_list.end());
-		}
-	}
-	
 //
 // Remove the attribute from the main vector
 //
@@ -934,8 +915,58 @@ void MultiAttribute::remove_attribute(string &attr_name)
 	delete att;
 	vector<Tango::Attribute *>::iterator pos = attr_list.begin();
 	advance(pos,att_index);
-	attr_list.erase(pos);
+	pos = attr_list.erase(pos);
+
+//
+// Update all the index for attribute following the one which has been deleted
+//
 	
+	for (;pos != attr_list.end();++pos)
+		(*pos)->set_attr_idx((*pos)->get_attr_idx() - 1);
+
+//
+// Clear the writable attribute index vector and rebuild it
+// This is necessary because this vector containd index in
+// the main attribute list which has been modified
+//
+
+	writable_attr_list.clear();
+	for (long i = 0;i < attr_list.size();i++)
+	{
+		Tango::AttrWriteType w_type = attr_list[i]->get_writable();
+		if ((w_type == Tango::WRITE) ||
+    		    (w_type == Tango::READ_WRITE))
+		{
+			writable_attr_list.push_back(i);
+		}
+	}
+	
+//
+// Do the same for the alarmed attribute for the same reason
+//
+
+	alarm_attr_list.clear();
+	for (long i = 0;i < attr_list.size();i++)
+	{
+		if (attr_list[i]->is_alarmed().any() == true)
+		{
+			Tango::AttrWriteType w_type = attr_list[i]->get_writable();
+			if (w_type != Tango::WRITE)
+				alarm_attr_list.push_back(i);
+		}
+	}
+			
+//
+// Check the associated attributes
+//
+			
+	string default_dev_name("a/b/c");
+	for (long i = 0;i < attr_list.size();i++)
+	{
+		check_associated(i,default_dev_name);
+	}
+	
+		
 	cout4 << "Leaving MultiAttribute::remove_attribute" << endl;
 }
 
@@ -1308,12 +1339,16 @@ void MultiAttribute::add_write_value(Attribute &att)
 
 	Tango::DevVarShortArray *sh_write_val;
 	Tango::DevVarLongArray *lg_write_val;
+	Tango::DevVarLong64Array *lg64_write_val;
 	Tango::DevVarDoubleArray *db_write_val;
 	Tango::DevVarStringArray *str_write_val;
 	Tango::DevVarFloatArray *fl_write_val;
 	Tango::DevVarBooleanArray *boo_write_val;
 	Tango::DevVarUShortArray *ush_write_val;
 	Tango::DevVarCharArray *uch_write_val;
+	Tango::DevVarULongArray *ulg_write_val;
+	Tango::DevVarULong64Array *ulg64_write_val;
+	Tango::DevVarStateArray *state_write_val;
 		
 	switch (att.get_data_type())
 	{
@@ -1327,13 +1362,18 @@ void MultiAttribute::add_write_value(Attribute &att)
 		att.add_write_value(lg_write_val);
 		break;
 		
+	case Tango::DEV_LONG64 :
+		lg64_write_val = assoc_att.get_last_written_lg64();
+		att.add_write_value(lg64_write_val);
+		break;
+		
 	case Tango::DEV_DOUBLE :
 		db_write_val = assoc_att.get_last_written_db();		
 		att.add_write_value(db_write_val);
 		break;
 		
 	case Tango::DEV_STRING :
-		str_write_val = assoc_att.get_last_written_str();		
+		str_write_val = assoc_att.get_last_written_str();
 		att.add_write_value(str_write_val);
 		break;
 		
@@ -1356,6 +1396,21 @@ void MultiAttribute::add_write_value(Attribute &att)
 		uch_write_val = assoc_att.get_last_written_uch();		
 		att.add_write_value(uch_write_val);
 		break;
+		
+	case Tango::DEV_ULONG :
+		ulg_write_val = assoc_att.get_last_written_ulg();
+		att.add_write_value(ulg_write_val);
+		break;
+		
+	case Tango::DEV_ULONG64 :
+		ulg64_write_val = assoc_att.get_last_written_ulg64();
+		att.add_write_value(ulg64_write_val);
+		break;
+		
+	case Tango::DEV_STATE :
+		state_write_val = assoc_att.get_last_written_state();
+		att.add_write_value(state_write_val);
+		break;
 	}
 }
 
