@@ -10,9 +10,12 @@
 //
 // author(s) :		A.Gotz + E.Taurel
 //
-// $Revision: 3.5 $
+// $Revision: 3.6 $
 //
 // $Log: multiattribute.h,v $
+// Revision 3.6  2007/04/20 14:41:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
 // Revision 3.5  2006/06/29 11:48:24  jensmeyer
 // Added get_attribute_list(0 method to extract the attribute vector.
 //
@@ -191,8 +194,8 @@ class MultiAttributeExt
  * an aggregate of Attribute or WAttribute objects. It eases management of 
  * multiple attributes
  *
- * $Author: jensmeyer $
- * $Revision: 3.5 $
+ * $Author: taurel $
+ * $Revision: 3.6 $
  */
  
 class MultiAttribute
@@ -299,7 +302,7 @@ public:
  *
  * @return  The attribute number
  */
-	unsigned long get_attr_nb() {return attr_list.size();}
+	unsigned long get_attr_nb() {return (unsigned long)attr_list.size();}
 
 /**
  * Check alarm for one attribute with a given name.
