@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollring.cpp,v 3.5 2005/01/13 09:28:32 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: pollring.cpp,v 3.8 2007/04/20 14:41:33 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,21 @@ static const char *RcsId = "$Id: pollring.cpp,v 3.5 2005/01/13 09:28:32 taurel E
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.5 $
+// $Revision: 3.8 $
 //
 // $Log: pollring.cpp,v $
+// Revision 3.8  2007/04/20 14:41:33  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.7  2007/04/16 14:57:42  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.6  2007/03/06 08:19:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
 // Revision 3.5  2005/01/13 09:28:32  taurel
 // Fix some bugs :
 // - R/W attribute : W value not returned when read if set by set_write_value
@@ -175,12 +187,12 @@ static const char *RcsId = "$Id: pollring.cpp,v 3.5 2005/01/13 09:28:32 taurel E
 #include <pollring.h>
 
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 #include <sys/types.h>
 #include <sys/timeb.h>
 #else
 #include <sys/time.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 namespace Tango
 {
@@ -807,6 +819,8 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList *ptr,long type)
 				Tango::DevVarShortArray *new_tmp_sh;
 				const Tango::DevVarLongArray *tmp_lg;
 				Tango::DevVarLongArray *new_tmp_lg;
+				const Tango::DevVarLong64Array *tmp_lg64;
+				Tango::DevVarLong64Array *new_tmp_lg64;
 				const Tango::DevVarStringArray *tmp_str;
 				Tango::DevVarStringArray *new_tmp_str;
 				const Tango::DevVarFloatArray *tmp_fl;
@@ -817,7 +831,12 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList *ptr,long type)
 				Tango::DevVarUShortArray *new_tmp_ush;
 				const Tango::DevVarCharArray *tmp_uch;
 				Tango::DevVarCharArray *new_tmp_uch;
-				Tango::DevState sta;
+				const Tango::DevVarULongArray *tmp_ulg;
+				Tango::DevVarULongArray *new_tmp_ulg;
+				const Tango::DevVarULong64Array *tmp_ulg64;
+				Tango::DevVarULong64Array *new_tmp_ulg64;
+				const Tango::DevVarStateArray *tmp_state;
+				Tango::DevVarStateArray *new_tmp_state;
 				
 				switch (type)
 				{
@@ -843,13 +862,17 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList *ptr,long type)
 					
 				case Tango::DEV_LONG :
 					(*ring[index].attr_value)[0].value >>= tmp_lg;
-					new_tmp_lg = new DevVarLongArray(
-							tmp_lg->length(),
-							tmp_lg->length(),
-							const_cast<long *>(tmp_lg->get_buffer()),
-							false);
+					new_tmp_lg = new DevVarLongArray(tmp_lg->length(),tmp_lg->length(),
+										const_cast<DevLong *>(tmp_lg->get_buffer()),false);
 					(*ptr)[seq_index].value.value <<= new_tmp_lg;
 					break;
+					
+				case Tango::DEV_LONG64 :
+					(*ring[index].attr_value)[0].value >>= tmp_lg64;
+					new_tmp_lg64 = new DevVarLong64Array(tmp_lg64->length(),tmp_lg64->length(),
+										const_cast<DevLong64 *>(tmp_lg64->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_lg64;
+					break;
 
 				case Tango::DEV_STRING :
 					(*ring[index].attr_value)[0].value >>= tmp_str;
@@ -901,9 +924,25 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList *ptr,long type)
 					(*ptr)[seq_index].value.value <<= new_tmp_uch;
 					break;
 					
+				case Tango::DEV_ULONG :
+					(*ring[index].attr_value)[0].value >>= tmp_ulg;
+					new_tmp_ulg = new DevVarULongArray(tmp_ulg->length(),tmp_ulg->length(),
+										const_cast<DevULong *>(tmp_ulg->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_ulg;
+					break;
+					
+				case Tango::DEV_ULONG64 :
+					(*ring[index].attr_value)[0].value >>= tmp_ulg64;
+					new_tmp_ulg64 = new DevVarULong64Array(tmp_ulg64->length(),tmp_ulg64->length(),
+										const_cast<DevULong64 *>(tmp_ulg64->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_ulg64;
+					break;
+					
 				case Tango::DEV_STATE :
-					(*ring[index].attr_value)[0].value >>= sta;
-					(*ptr)[seq_index].value.value <<= sta;
+					(*ring[index].attr_value)[0].value >>= tmp_state;
+					new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
+										const_cast<DevState *>(tmp_state->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_state;
 					break;
 				}
 			}
@@ -972,6 +1011,8 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList_3 *ptr,long typ
 				Tango::DevVarShortArray *new_tmp_sh;
 				const Tango::DevVarLongArray *tmp_lg;
 				Tango::DevVarLongArray *new_tmp_lg;
+				const Tango::DevVarLong64Array *tmp_lg64;
+				Tango::DevVarLong64Array *new_tmp_lg64;
 				const Tango::DevVarStringArray *tmp_str;
 				Tango::DevVarStringArray *new_tmp_str;
 				const Tango::DevVarFloatArray *tmp_fl;
@@ -982,7 +1023,12 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList_3 *ptr,long typ
 				Tango::DevVarUShortArray *new_tmp_ush;
 				const Tango::DevVarCharArray *tmp_uch;
 				Tango::DevVarCharArray *new_tmp_uch;
-				Tango::DevState sta;
+				const Tango::DevVarULongArray *tmp_ulg;
+				Tango::DevVarULongArray *new_tmp_ulg;
+				const Tango::DevVarULong64Array *tmp_ulg64;
+				Tango::DevVarULong64Array *new_tmp_ulg64;
+				const Tango::DevVarStateArray *tmp_state;
+				Tango::DevVarStateArray *new_tmp_state;
 				
 
 				switch (type)
@@ -1009,13 +1055,17 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList_3 *ptr,long typ
 
 				case Tango::DEV_LONG :
 					(*ring[index].attr_value_3)[0].value >>= tmp_lg;
-					new_tmp_lg = new DevVarLongArray(
-							tmp_lg->length(),
-							tmp_lg->length(),
-							const_cast<long *>(tmp_lg->get_buffer()),
-							false);
+					new_tmp_lg = new DevVarLongArray(tmp_lg->length(),tmp_lg->length(),
+							const_cast<DevLong *>(tmp_lg->get_buffer()),false);
 					(*ptr)[seq_index].value.value <<= new_tmp_lg;
 					break;
+					
+				case Tango::DEV_LONG64 :
+					(*ring[index].attr_value_3)[0].value >>= tmp_lg64;
+					new_tmp_lg64 = new DevVarLong64Array(tmp_lg64->length(),tmp_lg64->length(),
+							const_cast<DevLong64 *>(tmp_lg64->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_lg64;
+					break;
 
 				case Tango::DEV_STRING :
 					(*ring[index].attr_value_3)[0].value >>= tmp_str;
@@ -1066,10 +1116,26 @@ void PollRing::get_attr_history(long n,Tango::DevAttrHistoryList_3 *ptr,long typ
 							false);
 					(*ptr)[seq_index].value.value <<= new_tmp_uch;
 					break;
+
+				case Tango::DEV_ULONG :
+					(*ring[index].attr_value_3)[0].value >>= tmp_ulg;
+					new_tmp_ulg = new DevVarULongArray(tmp_ulg->length(),tmp_ulg->length(),
+							const_cast<DevULong *>(tmp_ulg->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_ulg;
+					break;
 					
+				case Tango::DEV_ULONG64 :
+					(*ring[index].attr_value_3)[0].value >>= tmp_ulg64;
+					new_tmp_ulg64 = new DevVarULong64Array(tmp_ulg64->length(),tmp_ulg64->length(),
+							const_cast<DevULong64 *>(tmp_ulg64->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_ulg64;
+					break;
+										
 				case Tango::DEV_STATE :
-					(*ring[index].attr_value_3)[0].value >>= sta;
-					(*ptr)[seq_index].value.value <<= sta;
+					(*ring[index].attr_value_3)[0].value >>= tmp_state;
+					new_tmp_state = new DevVarStateArray(tmp_state->length(),tmp_state->length(),
+							const_cast<DevState *>(tmp_state->get_buffer()),false);
+					(*ptr)[seq_index].value.value <<= new_tmp_state;
 					break;
 				}
 			}
