@@ -9,9 +9,21 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.11 $
 //
 // $Log: tango_config.h,v $
+// Revision 3.11  2007/04/20 14:41:34  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.10  2007/04/16 14:57:43  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.9  2007/03/28 11:13:25  taurel
+// - Added a WIN32_VC8 define
+//
 // Revision 3.8  2006/05/18 08:51:56  taurel
 // - Miscellaneous changes due to Python device server ported to Windows
 // - Fix some bugs discovered by Windows VC8 using the test suite
@@ -195,7 +207,7 @@
 // setting in the preprocessor definitions....
 //
 
-#ifdef WIN32
+#if ((defined WIN32) || (defined WIN64))
 	#define		__WIN32__
 	#define		__x86__
 	#ifndef _WIN32_WINNT
@@ -205,11 +217,25 @@
 	#define		__OSVERSION 4
 #endif
 
+//
+// Define a common preprocessor macros for all Windows (32 or 64 bits)
+//
+// Please note that on Windows, this file include Windows include file.
+// The Windows Windef.h file defines a Preprocessor WIN32 on top of the
+// _WIN32 one.
+// This measn that on Windows 64 bits, we will have BOTH WIN32 and WIN64
+// defined even if on the compiler command line we define only WIN64
+//
+
+#if ((defined WIN32) || (defined WIN64))
+	#define _TG_WINDOWS_
+#endif
+
 //
 // For Windows DLL (import and export nightmare)
 // 
 
-#ifdef  WIN32
+#ifdef  _TG_WINDOWS_
 	#if ((defined _USRDLL) || (defined TANGO_HAS_DLL))
 		#if (defined _TANGO_LIB)
 			#define TANGO_IMP_EXP	__declspec(dllexport)
@@ -225,13 +251,13 @@
 #else	 
 	#define TANGO_IMP_EXP
 	#define TANGO_IMP
-#endif /* WIN32 */
+#endif /* _WINDOWS_ */
 
 //
 // Some helper define
 //
 
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	#define 	TangoSys_OMemStream	ostringstream
 	#define		TangoSys_MemStream	stringstream
 	#define		TangoSys_Pid		int
@@ -245,12 +271,12 @@
 
 
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	#pragma warning(disable : 4355)
 	#pragma warning(disable : 4715)
-    #pragma warning(disable : 4786)
+    	#pragma warning(disable : 4786)
 	#if (_MSC_VER >= 1400)       // VC8+ 
-	#pragma warning(disable : 4996)    // disable all deprecation warnings
+		#pragma warning(disable : 4996)    // disable all deprecation warnings
 	#endif   // VC8+
 #endif
 
@@ -258,7 +284,7 @@
 // Define a common sleep call
 //
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 #define Tango_sleep(A) sleep(A);
 #else
 #define Tango_sleep(A) Sleep(A * 1000);
@@ -280,7 +306,7 @@
 // Is it a 32 or 64 bits computer
 //
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	#include <omniORB4/acconfig.h>
 
 	#if SIZEOF_LONG == 8
