@@ -8,9 +8,30 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.18 $
+// $Revision: 3.24 $
 //
 // $Log: tango_const.h,v $
+// Revision 3.24  2007/06/15 07:29:43  jensmeyer
+// Increase the tango library version number
+//
+// Revision 3.23  2007/04/16 14:57:45  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.22  2007/03/28 11:16:30  taurel
+// - Change Tango release number and a definition
+//
+// Revision 3.21  2007/03/06 08:19:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.20  2007/02/16 10:40:57  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
+// Revision 3.19  2007/01/19 15:45:53  taurel
+// - Added the Utils::get_tango_lib_release() method
+//
 // Revision 3.18  2006/09/20 07:48:47  jensmeyer
 // Increased MAX_TRANSFER_SIZE variable from 8 to 16 MBytes.
 // Necessary for transfer og big images
@@ -232,47 +253,48 @@ namespace Tango
 // Some general interest define
 //
 
-#define		TgLibVers			"5.5.0"
+#define		TgLibVers				"6.0.0"		// Please, always code this following format "X.Y.Z"
 
-#define		DevVersion			3
+#define		DevVersion				3
 #define		DefaultMaxSeq			20
-#define		DefaultBlackBoxDepth		25
-#define 	DefaultPollRingDepth		10
+#define		DefaultBlackBoxDepth	25
+#define 	DefaultPollRingDepth	10
 
 #define		InitialOutput			"Initial Output"
 #define		DSDeviceDomain  		"dserver"
 #define		DefaultDocUrl			"http://www.esrf.fr/tango"
-#define		EnvVariable			"TANGO_HOST"
-#define		DbObjName			"database"
-#define		DescNotSet			"Uninitialised"
+#define		EnvVariable				"TANGO_HOST"
+#define		DbObjName				"database"
+#define		DescNotSet				"Uninitialised"
 #define		ResNotDefined			"0"
-#define         MessBoxTitle			"Tango Device Server"
+#define		MessBoxTitle			"Tango Device Server"
 #define		StatusNotSet			"Not initialised"
 
 #define		DefaultWritAttrProp		false
-#define		AllAttr 			"All attributes"
-#define		AllAttr_3 			"All attributes_3"
+#define		AllAttr 				"All attributes"
+#define		AllAttr_3 				"All attributes_3"
 
-#define		PollCommand			"command"
+#define		PollCommand				"command"
 #define		PollAttribute			"attribute"
 
 #define		MIN_POLL_PERIOD			20
-#define		DELTA_T				1002000000
+#define		DELTA_T					1002000000
 #define		MIN_DELTA_WORK			20000
 #define		TIME_HEARTBEAT			2000
 #define		POLL_LOOP_NB			500
-#define		ONE_SECOND			1000000
+#define		ONE_SECOND				1000000
 #define		DISCARD_THRESHOLD		0.02
 
 #define		DEFAULT_TIMEOUT_SEC		4
 #define		DEFAULT_TIMEOUT			DEFAULT_TIMEOUT_SEC * 1000
-#define		DEFAULT_POLL_OLD_FACTOR		4
+#define		DEFAULT_POLL_OLD_FACTOR	4
 
 #define		TG_IMP_MINOR_TO			10
-#define		TG_IMP_MINOR_DEVFAILED		11
+#define		TG_IMP_MINOR_DEVFAILED	11
 #define		TG_IMP_MINOR_NON_DEVFAILED	12
 
 #define		TANGO_PY_MOD_NAME		"_PyTango.pyd"
+#define		DATABASE_CLASS			"DataBase"
 
 //
 // Event related define
@@ -282,8 +304,8 @@ namespace Tango
 #define 	EVENT_RESUBSCRIBE_PERIOD 	600
 #define		DEFAULT_EVENT_PERIOD		1000
 #define		DELTA_PERIODIC				0.98  // Using a delta of 2% only for times < 5000 ms
-#define     DELTA_PERIODIC_LONG		100   // For times > 5000ms only keep a delta of 100ms
-#define		HEARTBEAT			"Event heartbeat"
+#define     DELTA_PERIODIC_LONG			100   // For times > 5000ms only keep a delta of 100ms
+#define		HEARTBEAT					"Event heartbeat"
 
 //
 // Client timeout as defined by omniORB4.0.0
@@ -296,13 +318,13 @@ namespace Tango
 // Time to wait before trying to reconnect after
 // a connevtion failure
 //
-#define 		RECONNECTION_DELAY		1000   //ms. Only try to reconnect every second
+#define 	RECONNECTION_DELAY		1000   //ms. Only try to reconnect every second
 
 //
-// Max transfer size 16 MBytes (in byte). Needed by omniORB
+// Max transfer size 256 MBytes (in byte). Needed by omniORB
 //
 
-#define		MAX_TRANSFER_SIZE		"16777216"
+#define		MAX_TRANSFER_SIZE		"268435456"
 
 //
 // Tango name length
@@ -778,7 +800,12 @@ enum CmdArgType {
 	DEV_STATE,
 	CONST_DEV_STRING,
 	DEVVAR_BOOLEANARRAY,
-	DEV_UCHAR
+	DEV_UCHAR,
+	DEV_LONG64,
+	DEV_ULONG64,
+	DEVVAR_LONG64ARRAY,
+	DEVVAR_ULONG64ARRAY,
+	DEV_INT
 };
 
 enum MessBoxType {
@@ -905,7 +932,12 @@ const char * const CmdArgTypeName[] = {
 	"DevState",
 	"ConstDevString",
 	"DevVarBooleanArray",
-	"DevUChar"
+	"DevUChar",
+	"DevLong64",
+	"DevULong64",
+	"DevVarLong64Array",
+	"DevVarULong64Array",
+	"DevInt"
 };
 
 //
@@ -934,7 +966,8 @@ enum EventType {
 	QUALITY_EVENT,
 	PERIODIC_EVENT,
 	ARCHIVE_EVENT,
-	USER_EVENT
+	USER_EVENT,
+	ATTR_CONF_EVENT
 };
 
 enum KeepAliveCmdCode {
@@ -949,15 +982,15 @@ enum KeepAliveCmdCode {
 #define		AlrmValueNotSpec	"Not specified"
 #define		AssocWritNotSpec	"None"
 #define		LabelNotSpec	  	"No label"
-#define		DescNotSpec		"No description"
-#define		UnitNotSpec		"No unit"
+#define		DescNotSpec			"No description"
+#define		UnitNotSpec			"No unit"
 #define		StdUnitNotSpec		"No standard unit"
 #define		DispUnitNotSpec		"No display unit"
 #define		FormatNotSpec		"%6.2f"
 
-#define		NotANumber		"NaN"
+#define		NotANumber			"NaN"
 
-#define		MemNotUsed		"Not used yet"
+#define		MemNotUsed			"Not used yet"
 #define		MemAttrPropName		"__value"
 
 typedef struct _OptAttrProp
@@ -1014,6 +1047,8 @@ typedef struct _OptAttrProp
  * API_AttrWrongMemValue
  * API_WizardConfError
  * API_EventSupplierNotConstructed
+ * API_CantGetClientIdent
+ * API_DeviceLocked
  
  */
 
