@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: thsig.cpp,v 3.8 2006/03/02 08:03:40 jensmeyer Exp $\n$Name:  $";
+static const char *RcsId = "$Id: thsig.cpp,v 3.9 2007/04/20 14:41:34 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,7 +14,7 @@ static const char *RcsId = "$Id: thsig.cpp,v 3.8 2006/03/02 08:03:40 jensmeyer E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.9 $
 //
 // copyleft :           European Synchrotron Radiation Facility
 //                      BP 220, Grenoble 38043
@@ -36,7 +36,7 @@ namespace Tango
 void *DServerSignal::ThSig::run_undetached(void *ptr)
 {
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	sigset_t sigs_to_catch;
 
 //
@@ -70,7 +70,7 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 	
 	while(1)
 	{
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 		int ret = sigwait(&sigs_to_catch,&signo);
 		// sigwait() under linux might return an errno number without initialising the
 		// signo variable. Do a ckeck here to avoid problems!!!
@@ -101,7 +101,7 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 			th_data_created = true;
 		}
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 			
 //
 // Add a new signal to catch in the mask
@@ -228,7 +228,7 @@ void *DServerSignal::ThSig::run_undetached(void *ptr)
 				}
 				catch(...)
 				{
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 					raise(SIGKILL);
 #endif
 				}
