@@ -1,5 +1,5 @@
-static const char *RcsId = "$Id: utils.cpp,v 3.20 2006/10/06 06:46:53 jensmeyer Exp $\n$Name:  $";
- 
+static const char *RcsId = "$Id: utils.cpp,v 3.29 2007/05/17 07:59:07 taurel Exp $\n$Name:  $";
+
 //+=============================================================================
 //
 // file :               Tango_utils.cpp
@@ -9,11 +9,44 @@ static const char *RcsId = "$Id: utils.cpp,v 3.20 2006/10/06 06:46:53 jensmeyer
 //
 // project :            TANGO
 //
-// author(s) :          A.Gotz + E.Taurel 
+// author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.20 $
+// $Revision: 3.29 $
 //
 // $Log: utils.cpp,v $
+// Revision 3.29  2007/05/17 07:59:07  taurel
+// - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
+// Add device name in monitor print message
+// Add device_destroyer method in DeviceClass class
+//
+// Revision 3.28  2007/05/15 07:46:59  taurel
+// - The polling thread is not configured by a separate thread any more.
+// The Add_obj_polling command now support a delta_t to start the first polling
+//
+// Revision 3.27  2007/04/20 14:41:34  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.26  2007/04/16 14:57:45  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.25  2007/03/29 07:11:06  taurel
+// - Change some data types for 64 bits compatibility
+//
+// Revision 3.24  2007/03/12 15:16:57  taurel
+// - Change the way to return Tango lib release
+//
+// Revision 3.23  2007/03/06 08:19:03  taurel
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.22  2007/02/16 10:40:57  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
+// Revision 3.21  2007/01/19 15:45:53  taurel
+// - Added the Utils::get_tango_lib_release() method
+//
 // Revision 3.20  2006/10/06 06:46:53  jensmeyer
 // Corrected omnithread initialisation in server_init in the case of a windows service.
 //
@@ -258,7 +291,7 @@ static const char *RcsId = "$Id: utils.cpp,v 3.20 2006/10/06 06:46:53 jensmeyer
 #include <dserverclass.h>
 #include <eventsupplier.h>
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	#include <unistd.h>
 	#include <assert.h>
 	#include <sys/time.h>
@@ -268,7 +301,7 @@ static const char *RcsId = "$Id: utils.cpp,v 3.20 2006/10/06 06:46:53 jensmeyer
 	#include <process.h>
         #include <coutbuf.h>
 	#include <ntservice.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 #include <omniORB4/omniInterceptors.h>
 
@@ -278,13 +311,13 @@ namespace Tango
 {
 
 Util *Util::_instance = NULL;
-long Util::_tracelevel = 0;
+int Util::_tracelevel = 0;
 bool Util::_UseDb = true;
 bool Util::_FileDb = false;
 bool Util::_daemon = false;
 long Util::_sleep_between_connect = 60;
 bool Util::_constructed = false;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 bool Util::_win = false;
 bool Util::_service = false;
 #endif
@@ -321,7 +354,7 @@ Util *Util::init(int argc,char *argv[])
 	return _instance;
 }
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 Util *Util::init(HINSTANCE hi,int nCmd)
 {
 	if (_instance == NULL)
@@ -368,12 +401,12 @@ Util *Util::instance(bool exit)
 //
 //-----------------------------------------------------------------------------
 
-#if ((defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined __SUNPRO_CC) || (defined GCC_STD))
 Util::Util(int argc,char *argv[]):cl_list_ptr(NULL)
 # ifndef TANGO_HAS_LOG4TANGO
     ,cout_tmp(cout.rdbuf())
 # endif
-#elif (defined WIN32)
+#elif (defined _TG_WINDOWS_)
 Util::Util(int argc,char *argv[]):cl_list_ptr(NULL),mon("Windows startup")
 # ifndef TANGO_HAS_LOG4TANGO
     ,cout_tmp(cout.rdbuf())
@@ -455,7 +488,7 @@ void Util::effective_job(int argc,char *argv[])
 		else
 		{
 			const char *options[][2] = {
-			{"endPointPublishAllIFs","1"},
+			{"endPointPublish","all(addr)"},
 			{"clientCallTimeOutPeriod",CLNT_TIMEOUT_STR},
 			{"serverCallTimeOutPeriod","5000"},
 			{"maxServerThreadPoolSize","100"},
@@ -523,7 +556,7 @@ void Util::effective_job(int argc,char *argv[])
   		trace_output = InitialOutput;
 		file_stream = NULL;
 
-# if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+# if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 		cout_tmp.copyfmt(cout);
 		cout_tmp.clear(cout.rdstate());
 # else
@@ -544,7 +577,7 @@ void Util::effective_job(int argc,char *argv[])
 // connected to the database becaue a call to the database server is done in the
 // display_help_message() method
 //
-			
+
 			if (display_help == true)
 			{
 				display_help_message();
@@ -603,7 +636,7 @@ void Util::effective_job(int argc,char *argv[])
 }
 
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 //+----------------------------------------------------------------------------
 //
 // method : 		Util::Util()
@@ -702,7 +735,7 @@ void Util::check_args(int argc,char *argv[])
 	}
 	ds_instance_name = argv[1];
 	char *tmp;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if ((tmp = strrchr(argv[0],'\\')) == 0)
 	{
 		if ((tmp = strrchr(argv[0],'/')) == 0)
@@ -734,13 +767,13 @@ void Util::check_args(int argc,char *argv[])
 // For Windows only. Remove the .exe after the executable name
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	string::size_type pos;
 	if ((pos = ds_exec_name.find('.')) != string::npos)
 	{
 		ds_exec_name.erase(pos,ds_exec_name.size());
 	}
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 	if (argc > 2)
 	{
@@ -814,7 +847,7 @@ void Util::check_args(int argc,char *argv[])
 						_FileDb = true;
 						database_file_name = argv[ind]; 
 						database_file_name.erase(0,6);
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 						replace(database_file_name.begin(),
 							database_file_name.end(),
 							'\\','/');
@@ -1053,7 +1086,7 @@ void Util::connect_db()
 		{
 			try
 			{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 				if (_service == true)
 					db = new Database(orb.in(),
 							  ds_exec_name,
@@ -1097,7 +1130,7 @@ void Util::connect_db()
 	{
        		try
         	{
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 			if (_service == true)
 				db = new Database(orb.in(),
 						  ds_exec_name,
@@ -1190,8 +1223,8 @@ void Util::misc_init()
 
 	TangoSys_OMemStream o;
 	
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
-	#ifdef WIN32
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
+	#ifdef _TG_WINDOWS_
 	pid = _getpid();
 	#else
 		#ifdef __linux
@@ -1264,7 +1297,7 @@ void Util::misc_init()
 // Convert Tango version number to string (for device export)
 //
 		
-#if ((defined WIN32) || (defined __SUNPRO_CC) || (defined GCC_STD) || (defined __HP_aCC))
+#if ((defined _TG_WINDOWS_) || (defined __SUNPRO_CC) || (defined GCC_STD))
 	o.seekp(0,ios_base::beg);
 	o.clear();
 	o << DevVersion << ends;
@@ -1287,8 +1320,8 @@ void Util::misc_init()
 // Init text to be displayed on main window with a default value
 //
 
-#ifdef WIN32
-	main_win_text = "European Synchrotron Radiation Facility (ESRF)\n";
+#ifdef _TG_WINDOWS_
+	main_win_text = "TANGO collaboration (ALBA / ELETTRA / ESRF / SOLEIL)\n";
 	main_win_text = main_win_text + "CORBA based device server\n";
 	main_win_text = main_win_text + "Developped by Tango team";
 #endif
@@ -1342,7 +1375,7 @@ void Util::create_event_supplier()
 
 Util::~Util()
 {
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (ds_window != NULL)
 	{
 		TangoMonitor &mon = get_poll_monitor();
@@ -1396,14 +1429,14 @@ void Util::server_already_running()
 // from what has been implemented for the TACO control system
 //
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	pid = _getpid();
 #else
 	pid = getpid();
 #endif
 	
 	srand(pid);
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	DWORD backoff;
 	backoff = (DWORD)(1000. * (float)rand() / (float)RAND_MAX);
 	Sleep(backoff);
@@ -1493,7 +1526,7 @@ void Util::server_already_running()
 	}
 	catch (CORBA::NO_RESPONSE &)
 	{
-                print_err_message("This server is already running but is blocked!");
+		print_err_message("This server is already running but is blocked!");
 	}	
 	catch (CORBA::COMM_FAILURE &)
 	{
@@ -1570,7 +1603,7 @@ void Util::server_init(bool with_window)
 // defined in the binding
 //
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (Util::_service == true)
 	{
 		omni_thread::create_dummy();
@@ -1583,7 +1616,7 @@ void Util::server_init(bool with_window)
 		th->set_value(key_py_data,new Tango::PyData());
 	}
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (with_window == true)
 	{
 
@@ -1629,13 +1662,32 @@ void Util::server_init(bool with_window)
 		
 //
 // Configure polling from the polling properties
+// In case of python DS, we need to release the Python GIL
+// because the polling_configure method will send cmd to the
+// pollinh thread which will try to get the Python GIL
 //
-		
+
+		int th_id = th->id();
+		PyLock *lock_ptr;
+		bool py_ds_main_th = false;
+		if ((th_id == 0) && (is_py_ds() == true))
+		{
+			py_ds_main_th = true;
+			omni_thread::value_t *tmp_py_data = th->get_value(key_py_data);
+			lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+			lock_ptr->Release();
+		}
+			
 		polling_configure();
 		
-#ifdef WIN32
+		if (py_ds_main_th == true)
+		{
+			lock_ptr->Get();
+		}
+		
+#ifdef _TG_WINDOWS_
 	}
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 	
 }
 
@@ -1658,7 +1710,7 @@ void Util::server_run()
 // thread is used for windows management.
 //
 	
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 	if (_win == true)
 	{
 	
@@ -1765,7 +1817,7 @@ void Util::server_run()
 
 void Util::server_cleanup()
 {
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 //
 // Destroy the ORB
 //
@@ -1870,6 +1922,63 @@ vector<DeviceImpl *> &Util::get_device_list_by_class(const char *class_name)
 DeviceImpl *Util::get_device_by_name(const string &dev_name)
 {
 
+	string dev_name_lower(dev_name);
+	transform(dev_name_lower.begin(),dev_name_lower.end(),dev_name_lower.begin(),::tolower);
+
+	DeviceImpl *ret_ptr = find_device_name_core(dev_name_lower);	
+
+//
+// If the device is not found, may be the name we have received is an alias ?
+//
+
+	if (ret_ptr == NULL)
+	{
+		string d_name;
+		try
+		{
+			db->get_device_alias(dev_name_lower,d_name);
+		}
+		catch (Tango::DevFailed &) {}
+		
+		if (d_name.size() != 0)
+		{
+			transform(d_name.begin(),d_name.end(),d_name.begin(),::tolower);
+			
+			ret_ptr = find_device_name_core(d_name);
+			
+//
+// If the name given to this method is a valid alias name,
+// store the alias name in device object for possible
+// future call to this method (save some db calls)
+//
+
+			if (ret_ptr != NULL)
+			{
+				ret_ptr->set_alias_name_lower(dev_name_lower);
+			}
+			
+		}
+	}
+		
+//
+// Throw exception if the device is not found
+//
+
+	if (ret_ptr == NULL)
+	{
+		TangoSys_OMemStream o;
+		o << "Device " << dev_name << " not found" << ends;
+		Except::throw_exception((const char *)"API_DeviceNotFound",
+				        o.str(),
+				        (const char *)"Util::get_device_by_name()");
+	}
+	
+	return ret_ptr;
+}
+
+
+DeviceImpl *Util::find_device_name_core(string &dev_name)
+{
 //
 // Retrieve class list. Don't use the get_dserver_device() method followed by
 // the get_class_list(). In case of several classes embedded within
@@ -1880,9 +1989,6 @@ DeviceImpl *Util::get_device_by_name(const string &dev_name)
 
 	const vector<DeviceClass *> &tmp_cl_list = *cl_list_ptr;
 	DeviceImpl *ret_ptr = NULL;
-
-	string dev_name_lower(dev_name);
-	transform(dev_name_lower.begin(),dev_name_lower.end(),dev_name_lower.begin(),::tolower);
 	
 //
 // Check if the wanted device exists in each class
@@ -1900,12 +2006,22 @@ DeviceImpl *Util::get_device_by_name(const string &dev_name)
 		{
 			string name(dev_list[j]->get_name());
 			transform(name.begin(),name.end(),name.begin(),::tolower);
-			if (name == dev_name_lower)
+			if (name == dev_name)
 			{
 				found = true;
 				ret_ptr = dev_list[j];
 				break;
 			}
+			string &alias_name = dev_list[j]->get_alias_name_lower();
+			if (alias_name.size() != 0)
+			{
+				if (alias_name == dev_name)
+				{
+					found = true;
+					ret_ptr = dev_list[j];
+					break;
+				}
+			}
 		}
 		if (found == true)
 			break;
@@ -1921,26 +2037,17 @@ DeviceImpl *Util::get_device_by_name(const string &dev_name)
 		vector<DeviceImpl *> &devlist = ds_class->get_device_list();
 		string name(devlist[0]->get_name());
 		transform(name.begin(),name.end(),name.begin(),::tolower);
-		if (name == dev_name_lower)
+		if (name == dev_name)
 		{
 			found = true;
 			ret_ptr = devlist[0];
 			j--;
 		}
 	}
-
+		
 //
-// Throw exception if the device is not found
+// Return to caller. The returned value is NULL if the device is not found
 //
-
-	if ((i == nb_class) && (j == nb_dev))
-	{
-		TangoSys_OMemStream o;
-		o << "Device " << dev_name << " not found" << ends;
-		Except::throw_exception((const char *)"API_DeviceNotFound",
-				        o.str(),
-				        (const char *)"Util::get_device_by_name()");
-	}
 	
 	return ret_ptr;
 }
@@ -2163,7 +2270,7 @@ void Util::unregister_server()
 
 void Util::print_err_message(const char *err_mess,Tango::MessBoxType type)
 {
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
         if (_win == true)
         {
                 switch (type)
@@ -2209,171 +2316,146 @@ void Util::print_err_message(const char *err_mess,Tango::MessBoxType type)
 
 void Util::polling_configure()
 {
-	PollConfigureThread *pollconf_th = new PollConfigureThread();
-	pollconf_th->start();
-}
 
-void PollConfigureThread::run(void *ptr)
-{
-
-	Tango::Util *tg = Tango::Util::instance();
-#ifdef WIN32
-//
-// In case, some commands or attributes are polled, this method will send
-// command to the polling thread. These commands test the thread id of
-// the calling thread. In a Windows service, to be able to do this, we
-// have to make this thread known to omnithread
-//
-// In Tango V5, this code is now executed in a thread which is created
-// and therefore, this create_dummy call, is not necessary any more
-//
-
-/*	if (Util::_service == true)
-	{
-		omni_thread::create_dummy();
-	}*/
-#endif
 //
 // Send a stop polling command to thread in order not to poll devices
 //
 
-	try
-	{
-		DServer *adm_dev = tg->get_dserver_device();
-		adm_dev->stop_polling();
+	DServer *adm_dev = get_dserver_device();
+	adm_dev->stop_polling();
 
-		vector<DeviceClass *> &tmp_cl_list = adm_dev->get_class_list();
-		unsigned long i,j,k;
-		long upd;
-		TangoSys_MemStream s;
+	vector<DeviceClass *> &tmp_cl_list = adm_dev->get_class_list();
+	unsigned long i,j,k;
+	int upd;
+	TangoSys_MemStream s;
 
 //
 // Create the structure used to send data to the polling thread
 // and store them in a vector
 //
 
-		vector<DevVarLongStringArray *> v_poll_cmd;
+	vector<DevVarLongStringArray *> v_poll_cmd;
 
 //
 // A loop on each class and each device in class
 //
 
-		long smallest_upd;
-		bool first_loop = true;
+	int smallest_upd;
+	bool first_loop = true;
 
-		for (i = 0;i < tmp_cl_list.size();i++)
+	for (i = 0;i < tmp_cl_list.size();i++)
+	{
+		vector<DeviceImpl *> &dev_list = tmp_cl_list[i]->get_device_list();
+		for (j = 0;j < dev_list.size();j++)
 		{
-			vector<DeviceImpl *> &dev_list = tmp_cl_list[i]->get_device_list();
-			for (j = 0;j < dev_list.size();j++)
-			{
-				vector<string> &poll_cmd_list = dev_list[j]->get_polled_cmd();
-				vector<string> &poll_attr_list = dev_list[j]->get_polled_attr();
+			vector<string> &poll_cmd_list = dev_list[j]->get_polled_cmd();
+			vector<string> &poll_attr_list = dev_list[j]->get_polled_attr();
 
 //
 // Check that cmd_list and attr_list have a correct syntax
 //
 
-				if ((poll_cmd_list.size() % 2) == 1)
-				{
-					TangoSys_OMemStream o;
-					o << "System property polled_cmd for device " << dev_list[j]->get_name() << " has wrong syntax" << ends;
-					Except::throw_exception((const char *)"API_BadConfigurationProperty",
-				        			o.str(),
-				        			(const char *)"PollConfigureThread::run_undetached()");
-				}
-			
-				if ((poll_attr_list.size() % 2) == 1)
-				{
-					TangoSys_OMemStream o;
-					o << "System property polled_attr for device " << dev_list[j]->get_name() << " has wrong syntax" << ends;
-					Except::throw_exception((const char *)"API_BadConfigurationProperty",
-				        			o.str(),
-				        			(const char *)"PollConfigureThread::run_undetached()");
-				}
+			if ((poll_cmd_list.size() % 2) == 1)
+			{
+				TangoSys_OMemStream o;
+				o << "System property polled_cmd for device " << dev_list[j]->get_name() << " has wrong syntax" << ends;
+				Except::throw_exception((const char *)"API_BadConfigurationProperty",
+				        		o.str(),
+				        		(const char *)"Util::polling_configure");
+			}
+
+			if ((poll_attr_list.size() % 2) == 1)
+			{
+				TangoSys_OMemStream o;
+				o << "System property polled_attr for device " << dev_list[j]->get_name() << " has wrong syntax" << ends;
+				Except::throw_exception((const char *)"API_BadConfigurationProperty",
+				        		o.str(),
+				        		(const char *)"Util::polling_configure");
+			}
 
 
 //
 // A loop on each command
 //
 
-				for (k = 0;k < poll_cmd_list.size();k++)
-				{
+			for (k = 0;k < poll_cmd_list.size();k++)
+			{
 
-					DevVarLongStringArray *send = new DevVarLongStringArray();
-					send->lvalue.length(1);
-					send->svalue.length(3);
+				DevVarLongStringArray *send = new DevVarLongStringArray();
+				send->lvalue.length(1);
+				send->svalue.length(3);
 
-					send->svalue[0] = CORBA::string_dup(dev_list[j]->get_name().c_str());
-					send->svalue[1] = CORBA::string_dup("command");
-					send->svalue[2] = CORBA::string_dup(poll_cmd_list[k].c_str());
+				send->svalue[0] = CORBA::string_dup(dev_list[j]->get_name().c_str());
+				send->svalue[1] = CORBA::string_dup("command");
+				send->svalue[2] = CORBA::string_dup(poll_cmd_list[k].c_str());
 							
 //
 // Convert polling period to a number and store polling thread command params
 //
 
-					s << poll_cmd_list[k + 1];
-					s >> upd;
-					s.clear();
-					send->lvalue[0] = upd;
+				s << poll_cmd_list[k + 1];
+				s >> upd;
+				s.clear();
+				send->lvalue[0] = upd;
 
-					if (first_loop == true)
-					{
-						smallest_upd = upd;
-						first_loop = false;
-					}
-					else
-					{
-						if (upd < smallest_upd)
-							smallest_upd = upd;
-					}
-
-					v_poll_cmd.push_back(send);
-					k++;
+				if (first_loop == true)
+				{
+					smallest_upd = upd;
+					first_loop = false;
 				}
+				else
+				{
+					if (upd < smallest_upd)
+						smallest_upd = upd;
+				}
+
+				v_poll_cmd.push_back(send);
+				k++;
+			}
 
 //
 // A loop on each attribute
 //
 
-				for (k = 0;k < poll_attr_list.size();k++)
-				{
+			for (k = 0;k < poll_attr_list.size();k++)
+			{
 
-					DevVarLongStringArray *send = new DevVarLongStringArray();
-					send->lvalue.length(1);
-					send->svalue.length(3);
+				DevVarLongStringArray *send = new DevVarLongStringArray();
+				send->lvalue.length(1);
+				send->svalue.length(3);
 
-					send->svalue[0] = CORBA::string_dup(dev_list[j]->get_name().c_str());
-					send->svalue[1] = CORBA::string_dup("attribute");
-					send->svalue[2] = CORBA::string_dup(poll_attr_list[k].c_str());
+				send->svalue[0] = CORBA::string_dup(dev_list[j]->get_name().c_str());
+				send->svalue[1] = CORBA::string_dup("attribute");
+				send->svalue[2] = CORBA::string_dup(poll_attr_list[k].c_str());
 							
 //
 // Convert polling period to a number ans store polling thread command params
 //
 
-					s << poll_attr_list[k + 1];
-					s >> upd;
-					s.clear();
-					send->lvalue[0] = upd;
+				s << poll_attr_list[k + 1];
+				s >> upd;
+				s.clear();
+				send->lvalue[0] = upd;
 
-					if (first_loop == true)
-					{
-						smallest_upd = upd;
-						first_loop = false;
-					}
-					else
-					{
-						if (upd < smallest_upd)
-							smallest_upd = upd;
-					}
-
-					v_poll_cmd.push_back(send);
-					k++;
+				if (first_loop == true)
+				{
+					smallest_upd = upd;
+					first_loop = false;
 				}
+				else
+				{
+					if (upd < smallest_upd)
+						smallest_upd = upd;
+				}
+
+				v_poll_cmd.push_back(send);
+				k++;
 			}
 		}
+	}
 
 
-		unsigned long nb_cmd = v_poll_cmd.size();
+	unsigned long nb_cmd = v_poll_cmd.size();
 
 //
 // Send command to polling thread one by one and sleep between
@@ -2383,161 +2465,70 @@ void PollConfigureThread::run(void *ptr)
 // sleeping time due to thread wake-up time
 //
 
-		if (nb_cmd != 0)
-		{
-			long sleeping_time = smallest_upd / nb_cmd;
+	if (nb_cmd != 0)
+	{
+		int sleeping_time = smallest_upd / nb_cmd;
+		int delta_time = 0;
 	
-			long delta_os = 15;
-			if (delta_os < sleeping_time)
-				sleeping_time = sleeping_time - delta_os;
-		
-#ifndef WIN32
-			long nb_sec = 0;
-			long nb_nsec;
-			struct timespec to_wait,inter;
+		long delta_os = 15;
+		if (delta_os < sleeping_time)
+			sleeping_time = sleeping_time - delta_os;
 
-			if (sleeping_time >= 1000)
+		cout4 << "PollConfigureThread: smallest_upd = " << smallest_upd;
+		cout4 << ", delta_time = " << sleeping_time;
+		cout4 << ", nb_poll_objects = " << nb_cmd << endl;
+		
+		for (i = 0;i < nb_cmd;i++)
+		{
+			try
 			{
-				nb_sec = sleeping_time / 1000;
-				nb_nsec = (sleeping_time - (nb_sec * 1000)) * 1000000;		
+				adm_dev->add_obj_polling(v_poll_cmd[i],false,delta_time);
 			}
-			else
-				nb_nsec = sleeping_time * 1000000;
-		
-			to_wait.tv_sec = nb_sec;
-			to_wait.tv_nsec = nb_nsec;
-#endif
-
-			cout4 << "PollConfigureThread: smallest_upd = " << smallest_upd;
-			cout4 << ", sleeping_time = " << sleeping_time;
-			cout4 << ", nb_poll_objects = " << nb_cmd << endl;
-		
-			for (i = 0;i < nb_cmd;i++)
+			catch (Tango::DevFailed &e)
 			{
-				try
+				bool throw_ex = true;
+
+				if (::strcmp(e.errors[0].reason.in(),"API_AlreadyPolled") == 0)
 				{
-					adm_dev->add_obj_polling(v_poll_cmd[i],false);
-				}
-				catch (Tango::DevFailed &e)
-				{
-					bool throw_ex = true;
-				
-					if (::strcmp(e.errors[0].reason.in(),"API_AlreadyPolled") == 0)
+					try
 					{
-						try
-						{
-							adm_dev->upd_obj_polling_period(v_poll_cmd[i],false);
-							throw_ex = false;
-						}
-						catch (Tango::DevFailed &) {}
-					}
-						
-					if (throw_ex == true)
-					{
-						TangoSys_OMemStream o;
-						o << "System property polled_xxx for device " << v_poll_cmd[i]->svalue[0].in() << " has wrong syntax or not supported attribute/command" << ends;
-						Except::re_throw_exception(e,(const char *)"API_BadConfigurationProperty",
-									o.str(),
-									(const char *)"PollConfigureThread::run_undetached");
+						adm_dev->upd_obj_polling_period(v_poll_cmd[i],false);
+						throw_ex = false;
 					}
+					catch (Tango::DevFailed &) {}
 				}
 
-
-				if (nb_cmd > 1)
-				{		
-#ifdef WIN32
-					Sleep(sleeping_time);
-#else
-					nanosleep(&to_wait,&inter);
-#endif
+				if (throw_ex == true)
+				{
+					TangoSys_OMemStream o;
+					o << "Error when configuring polling for device " << v_poll_cmd[i]->svalue[0].in() << ends;
+					Except::re_throw_exception(e,(const char *)"API_BadConfigurationProperty",
+								o.str(),
+								(const char *)"Util::polling_configure");
 				}
 			}
+
+
+			if (nb_cmd > 1)
+			{		
+				delta_time = delta_time + sleeping_time;
+			}
 		}
+	}
 		
 //
 // Now, start the real polling
 //
 
-		adm_dev->start_polling();
+	adm_dev->start_polling();
 
 //
 // Delete allocated memory
 //
 
-		for (unsigned long l = 0;l < nb_cmd;l++)
-			delete v_poll_cmd[l];
+	for (unsigned long l = 0;l < nb_cmd;l++)
+		delete v_poll_cmd[l];
 
-	}
-	catch (Tango::DevFailed &e)
-	{
-		Tango::Except::print_exception(e);
-
-		if (tg != NULL)
-		{
-			try
-			{
-				
-//
-// Stopping a device server means :
-//  	- Send kill command to the polling thread
-//    	- Join with this polling thread
-//	- Unregister server in database
-//	- Delete devices (except the admin one)
-//	- Force writing file database in case of 
-//	- Shutdown the ORB
-//	- Cleanup Logging
-//
-
-				TangoMonitor &mon = tg->get_poll_monitor();
-				PollThCmd &shared_cmd = tg->get_poll_shared_cmd();
-
-				{	
-					omni_mutex_lock sync(mon);
-
-					shared_cmd.cmd_pending = true;
-					shared_cmd.cmd_code = POLL_EXIT;
-
-					mon.signal();
-
-				}
-				tg->get_polling_thread_object()->join(0);
-				tg->clr_poll_th_ptr();
-      	   			tg->unregister_server();
-
-				omni_thread::self()->set_value(key_py_data,new PyData());				
-				omni_thread::value_t *tmp_py_data = omni_thread::self()->get_value(key_py_data);
-				PyLock *lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
-				lock_ptr->Get();
-					
-				tg->get_dserver_device()->delete_devices();
-
-				lock_ptr->Release();
-				if (tg->_FileDb == true)
-				{
-					Database *db_ptr = tg->get_database();
-					delete db_ptr;
-					cout4 << "Database object deleted" << endl;
-				}
-				cout4 << "Going to shutdown ORB" << endl;
-				CORBA::ORB_ptr orb = tg->get_orb();
-				orb->shutdown(true);
-				CORBA::release(orb);
-				cout4 << "ORB shutdown" << endl;
-
-#ifdef TANGO_HAS_LOG4TANGO
-	  			Logging::cleanup();
-				cout4 << "Logging cleaned-up" << endl;
-#endif
-
-			}
-			catch(...)
-			{
-#ifndef WIN32
-				raise(SIGKILL);
-#endif
-			}
-		}
-	}
 }
 
 //+----------------------------------------------------------------------------
@@ -2810,7 +2801,22 @@ void Util::trigger_cmd_polling(Tango::DeviceImpl *dev,string &name)
 }
 
 
-#ifdef WIN32
+//+----------------------------------------------------------------------------
+//
+// method : 		Util::get_tango_lib_vers()
+// 
+// description : 	Return a number set to the Tango release number
+//			coded with 3 digits (550, 551,552,600)		
+//
+//-----------------------------------------------------------------------------
+
+long Util::get_tango_lib_release()
+{
+	return _convert_tango_lib_release(TgLibVers);
+}
+
+
+#ifdef _TG_WINDOWS_
 //+----------------------------------------------------------------------------
 //
 // method : 		Util::build_argc_argv()
@@ -2829,9 +2835,9 @@ void Util::build_argc_argv()
 // Get command line
 //
 
-        char *cmd_line = GetCommandLine();
+	char *cmd_line = GetCommandLine();
 
-        int cnt=0;
+	int cnt=0;
 	char *tmp;
 
 //
@@ -2839,53 +2845,53 @@ void Util::build_argc_argv()
 // we will have too many pointers allocates but it is not a problem
 //
 
-        int cmd_line_size = strlen(cmd_line);
-        for (int i = 0;i < cmd_line_size;i++)
-        {
-                if (cmd_line[i] == ' ')
-                        cnt++;
-        }
+    int cmd_line_size = strlen(cmd_line);
+    for (int i = 0;i < cmd_line_size;i++)
+    {
+        if (cmd_line[i] == ' ')
+                cnt++;
+    }
 
 //
 // Allocate memory for argv
 //
 
-        argv = new char *[cnt + 1];
+	argv = new char *[cnt + 1];
 
 //
 // If only one args, no parsing is necessary
 //
 
-        if (cnt == 0)
-        {
-                argv[0] = new char [cmd_line_size + 1];
-                strcpy(argv[0],cmd_line);
-                argc = 1;
-        }
-        else
-        {
+    if (cnt == 0)
+    {
+            argv[0] = new char [cmd_line_size + 1];
+            strcpy(argv[0],cmd_line);
+            argc = 1;
+    }
+    else
+    {
                 
 //
 // Get program name
 //
 
-	        tmp = strtok(cmd_line," ");
-	        argv[0] = new char [strlen(tmp) + 1];
-	        strcpy(argv[0],tmp);
+        tmp = strtok(cmd_line," ");
+        argv[0] = new char [strlen(tmp) + 1];
+        strcpy(argv[0],tmp);
 
 //
 // Get remaining args
 //
 
-                int i = 1;	        
-                while ((tmp = strtok(NULL," ")) != NULL)
-                {
-                        argv[i] = new char [strlen(tmp) + 1];
-                        strcpy(argv[i],tmp);
-                        i++;
-                }
-                argc = i;
+        int i = 1;	        
+        while ((tmp = strtok(NULL," ")) != NULL)
+        {
+            argv[i] = new char [strlen(tmp) + 1];
+            strcpy(argv[i],tmp);
+            i++;
         }
+        argc = i;
+    }
 }
 
 HWND Util::get_console_window()
@@ -2900,7 +2906,7 @@ HWND Util::get_ds_main_window()
 
 CoutBuf *Util::get_debug_object()
 {
-  return ds_window ? ds_window->get_output_buffer() : 0;
+	return ds_window ? ds_window->get_output_buffer() : 0;
 }
 
 
@@ -2979,7 +2985,7 @@ void Util::ORBWin32Loop::wait_for_go()
 		util->mon.wait();
 	}
 }
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 
 int TangoMonitor::wait(long nb_millis)
@@ -3010,12 +3016,17 @@ void clear_att_dim(Tango::AttributeValue_3 &att_val)
 void create_PyPerThData(omni::omniInterceptors::createThread_T::info_T &info)
 {
 	PyData *py_dat_ptr = new PyData();
+#ifdef _TG_WINDOWS_
+	omni_thread::ensure_self es;
+#endif
+
 	omni_thread::self()->set_value(key_py_data,py_dat_ptr);
-	
+
 	info.run();
 	
 	omni_thread::self()->remove_value(key_py_data);
 	delete py_dat_ptr;
+
 	return;
 }
 
@@ -3033,5 +3044,37 @@ AutoPyLock::~AutoPyLock()
 	lock_ptr->Release();
 }
 	
+//
+// A small function to convert Tango lib release string to a number
+// 
+// It is defined as a function to be used within the Utils and ApiUtil
+// classes (both client and server part)
+//
+
+long _convert_tango_lib_release(const char *tg_vers_str)
+{
+	long ret;
 	
+	TangoSys_MemStream str;
+	string::size_type pos;
+	string vers_str(tg_vers_str);
+	
+	pos = vers_str.find('.');
+	if (pos == string::npos)
+		return -1;
+	vers_str.erase(pos,1);
+	
+	pos = vers_str.find('.');
+	if (pos == string::npos)
+		return -1;
+	vers_str.erase(pos,1);
+	
+	str << vers_str << ends;
+	if (!(str >> ret))
+		ret = -1;
+		
+	return ret;
+}
+
+
 } // End of Tango namespace
