@@ -8,9 +8,50 @@
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.15 $
+// $Revision: 3.26 $
 //
 // $Log: utils.h,v $
+// Revision 3.26  2007/05/17 07:59:07  taurel
+// - The polling is not configured via a separate thread any more. The polling thread add_obj_polling method has been modified to support a parameter telling to the polling thread when it has to polled the object.
+// Add device name in monitor print message
+// Add device_destroyer method in DeviceClass class
+//
+// Revision 3.25  2007/05/15 07:46:59  taurel
+// - The polling thread is not configured by a separate thread any more.
+// The Add_obj_polling command now support a delta_t to start the first polling
+//
+// Revision 3.24  2007/04/20 14:41:35  taurel
+// - Ported to Windows 64 bits x64 architecture
+//
+// Revision 3.23  2007/04/16 14:57:46  taurel
+// - Added 3 new attributes data types (DevULong, DevULong64 and DevState)
+// - Ported to omniORB4.1
+// - Increased the MAX_TRANSFER_SIZE to 256 MBytes
+// - Added a new filterable field in the archive event
+//
+// Revision 3.22  2007/03/29 07:11:06  taurel
+// - Change some data types for 64 bits compatibility
+//
+// Revision 3.21  2007/03/12 15:17:00  taurel
+// - Change the way to return Tango lib release
+//
+// Revision 3.20  2007/03/08 11:42:46  tiagocoutinho
+// - Added 64 bits data types for 64 bits computer...
+//
+// Revision 3.19  2007/02/16 10:40:57  taurel
+// - Implementing a new types of event on the Attribute configuration
+//
+// Revision 3.18  2007/02/08 16:25:17  taurel
+// - Add some "unsigned" for local variable used in comparaison with vector
+// size(). This makes c++ -Wall happy
+//
+// Revision 3.17  2007/01/19 15:45:54  taurel
+// - Added the Utils::get_tango_lib_release() method
+//
+// Revision 3.16  2006/11/10 08:19:10  taurel
+// - It is now possiblt to fill the polling buffer even for attribute which are
+// not read only
+//
 // Revision 3.15  2006/03/03 10:51:56  taurel
 // - Added a new python debugger flag
 //
@@ -212,12 +253,12 @@
 #include <pollext.h>
 #include <new>
 
-#ifndef WIN32
+#ifndef _TG_WINDOWS_
 	#include <unistd.h>
 #else
 	#include <coutbuf.h>
 	#include <w32win.h>
-#endif /* WIN32 */
+#endif /* _TG_WINDOWS_ */
 
 
 //
@@ -252,7 +293,7 @@ class EventSupplier;
 class PyLock;
 class CreatePyLock;
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 class CoutBuf;
 class W32Win;
 #endif
@@ -326,7 +367,7 @@ public:
  * one instance of this class and its constructor is not public.
  *
  * $Author: taurel $
- * $Revision: 3.15 $
+ * $Revision: 3.26 $
  */
  
 class Util
@@ -352,7 +393,7 @@ public:
  * @return The Util object reference
  */
 	static Util *init(int argc,char *argv[]);
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 /**
  * Create and get the singleton object reference.
  *
@@ -420,14 +461,14 @@ public:
  *
  * @param level The new process level
  */	
-	void set_trace_level(long level) {_tracelevel = level;}
+	void set_trace_level(int level) {_tracelevel = level;}
 	
 /**
  * Get the process trace level.
  *
  * @return The process trace level
  */
-	long get_trace_level() {return _tracelevel;}
+	int get_trace_level() {return _tracelevel;}
 
 #ifndef TANGO_HAS_LOG4TANGO
 /**
@@ -509,7 +550,15 @@ public:
  * @return The device server process identifier
  */
 	TangoSys_Pid get_pid() {return pid;}
-
+	
+/**
+ * Get the TANGO library version number.
+ *
+ * @return The Tango library release number coded in 3 digits
+ * (for instance 550,551,552,600,....)
+ */	
+	long get_tango_lib_release();
+	
 /**
  * Get the IDL TANGO version.
  *
@@ -741,7 +790,7 @@ public:
 /**
  * The process trace level
  */	
-	static long		_tracelevel;
+	static int		_tracelevel;
 /**
  * The database use flag (Use with extreme care). Implemented for device
  * server started without database usage.
@@ -767,7 +816,7 @@ public:
 	
 //@}	
 
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 /**@name Windows specific methods */
 //@{
 /**
@@ -821,7 +870,7 @@ protected:
  *
  */
 	Util(int argc,char *argv[]);
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
 /**
  * Constructs a newly allocated Util object for Windows non-console
  * device server.
@@ -838,7 +887,7 @@ protected:
 private:
 	TANGO_IMP static Util		*_instance;
 	static bool		_constructed;
-#ifdef WIN32
+#ifdef _TG_WINDOWS_
         static bool             _win;
         int                     argc;
         char                    **argv;
@@ -902,6 +951,7 @@ private:
         }
 	void check_args(int, char *[]);
 	void display_help_message();
+	DeviceImpl *find_device_name_core(string &);
 
 	bool  display_help;				// display help message flag
 	const vector<DeviceClass *>	*cl_list_ptr;	// Ptr to server device class list	
@@ -994,11 +1044,11 @@ public:
 //
 
 		Tango::AttrWriteType w_type = att.get_writable();
-		if (w_type != Tango::READ)
+		if (w_type == Tango::WRITE)
 		{
 			TangoSys_OMemStream o;
 			o << "Attribute " << att_name;
-			o << " of device " << dev->get_name() << " is not READ only" << ends;
+			o << " of device " << dev->get_name() << " is WRITE only" << ends;
 
 			Except::throw_exception((const char *)"API_DeviceNotPolled",o.str(),
 				   		(const char *)"Util::fill_attr_polling_buffer");
@@ -1008,8 +1058,8 @@ public:
 // Check that history is not larger than polling buffer
 //
 
-		long nb_elt = data.length();
-		long nb_poll = dev->get_attr_poll_ring_depth(att_name);
+		unsigned long nb_elt = data.length();
+		unsigned long nb_poll = dev->get_attr_poll_ring_depth(att_name);
 
 		if (nb_elt > nb_poll)
 		{
@@ -1101,7 +1151,6 @@ public:
 // Set Attribute object value
 //
 
-					att.wanted_date(false);
 					att.set_value((T *)(data.get_data())[i].ptr,
 						      (data.get_data())[i].x,
 						      (data.get_data())[i].y,
@@ -1113,6 +1162,10 @@ public:
 // Init remaining fields in AttributeValueList
 //
 
+					if ((w_type == Tango::READ_WRITE) ||
+				    	    (w_type == Tango::READ_WITH_WRITE))
+						dev->get_device_attr()->add_write_value(att);
+						
 					CORBA::Any &a = (*back)[0].value;
 					switch (att.get_data_type())
 					{
@@ -1388,14 +1441,14 @@ inline CORBA::Any *return_empty_any(const char *cmd)
 		TangoSys_MemStream o;
 		
 		o << cmd << "::execute";
-#if !defined(WIN32) || (defined(_MSC_VER) && _MSC_VER >= 1300)
+#if !defined(_TG_WINDOWS_) || (defined(_MSC_VER) && _MSC_VER >= 1300)
 		//cout3 << "Bad allocation while in " << cmd << "::execute()" << endl;
 #endif
 		Tango::Except::throw_exception((const char *)"API_MemoryAllocation",
 					     (const char *)"Can't allocate memory in server",
 					     o.str());
 	}
-#if !defined(WIN32) || (defined(_MSC_VER) && _MSC_VER >= 1300)
+#if !defined(_TG_WINDOWS_) || (defined(_MSC_VER) && _MSC_VER >= 1300)
 	//cout4 << "Leaving " << cmd << "::execute()" << endl;	
 #endif
 	return(out_any);
@@ -1422,31 +1475,15 @@ inline DbDevice *DeviceImpl::get_db_device()
 void clear_att_dim(Tango::AttributeValue_3 &att_val);
 
 
-//=============================================================================
+//------------------------------------------------------------------------
 //
-//			The PollConfigureThread class
+//			Python device server classes
 //
-// description :	This class is just a helper class for the thread
-//			used during DS startup phase to send orders to the
-//			polling tread
-//
-//=============================================================================
-
-
-class PollConfigureThread: public omni_thread
-{
-public:
-	PollConfigureThread() {}
-	
-	void run(void *);	
-};
-
-
-
+//-----------------------------------------------------------------------
 
 
 //
-// For thread creation interceptor (Python device servers
+// For thread creation interceptor (Python device servers)
 //
 
 void create_PyPerThData(omni::omniInterceptors::createThread_T::info_T &);
@@ -1482,17 +1519,19 @@ public:
 	DevVarStringArray 		PerTh_dvsa;
 	DevVarLongStringArray		PerTh_dvlsa;
 	DevVarDoubleStringArray		PerTh_dvdsa;
+	DevVarLong64Array		PerTh_dvl64a;
+	DevVarULong64Array		PerTh_dvul64a;
 	
-	string				PerTh_string;
-	DevFailed			PerTh_df;
-	vector<string>			PerTh_vec_str;
-	vector<double>			PerTh_vec_db;
-	DevErrorList			PerTh_del;
+	string					PerTh_string;
+	DevFailed				PerTh_df;
+	vector<string>				PerTh_vec_str;
+	vector<double>				PerTh_vec_db;
+	DevErrorList				PerTh_del;
 	
-	bool				rec_state;
-	bool				rec_status;
+	bool						rec_state;
+	bool						rec_status;
 	
-	PyLock				*PerTh_py_lock;
+	PyLock						*PerTh_py_lock;
 };
 
 
@@ -1503,6 +1542,7 @@ public:
 	~AutoPyLock();
 };
 
+long _convert_tango_lib_release(const char *vers_str);
 
 } // End of Tango namespace
 		     
